@@ -28,9 +28,9 @@
 #include <util/log.h>
 
 #include "closure.h"
+#include "gjs/jsapi-wrapper.h"
+#include "gjs/mem.h"
 #include "keep-alive.h"
-#include <gjs/gjs-module.h>
-#include <gjs/compat.h>
 
 typedef struct {
     GClosure base;
@@ -95,7 +95,7 @@ global_context_finalized(JSObject *obj,
                          void     *data)
 {
     Closure *c;
-    gboolean need_unref;
+    bool need_unref;
 
     c = (Closure *) data;
 
@@ -107,7 +107,7 @@ global_context_finalized(JSObject *obj,
      * invalid memory access
      */
     need_unref = c->unref_on_global_object_finalized;
-    c->unref_on_global_object_finalized = FALSE;
+    c->unref_on_global_object_finalized = false;
 
     if (c->obj != NULL) {
         g_assert(c->obj == obj);
@@ -207,7 +207,7 @@ closure_invalidated(gpointer data,
                           "until global object finalize)",
                           closure);
 
-        c->unref_on_global_object_finalized = TRUE;
+        c->unref_on_global_object_finalized = true;
         g_closure_ref(&c->base);
     } else {
         /* If the context still exists, then remove our destroy
@@ -246,10 +246,10 @@ closure_set_invalid(gpointer  data,
 }
 
 void
-gjs_closure_invoke(GClosure *closure,
-                   int       argc,
-                   jsval    *argv,
-                   jsval    *retval)
+gjs_closure_invoke(GClosure              *closure,
+                   int                    argc,
+                   JS::Value             *argv,
+                   JS::MutableHandleValue retval)
 {
     Closure *c;
     JSContext *context;
@@ -270,34 +270,35 @@ gjs_closure_invoke(GClosure *closure,
 
     if (JS_IsExceptionPending(context)) {
         gjs_debug_closure("Exception was pending before invoking callback??? "
-                          "Not expected");
+                          "Not expected - closure %p", closure);
         gjs_log_exception(context);
     }
 
+    JS::RootedValue v_closure(context, JS::ObjectValue(*c->obj));
     if (!gjs_call_function_value(context,
-                                 NULL, /* "this" object; NULL is some kind of default presumably */
-                                 OBJECT_TO_JSVAL(c->obj),
-                                 argc,
-                                 argv,
-                                 retval)) {
+                                 /* "this" object; null is some kind of default presumably */
+                                 JS::NullPtr(),
+                                 v_closure, argc, argv, retval)) {
         /* Exception thrown... */
         gjs_debug_closure("Closure invocation failed (exception should "
                           "have been thrown) closure %p callable %p",
                           closure, c->obj);
         if (!gjs_log_exception(context))
-            gjs_debug_closure("Closure invocation failed but no exception was set?");
+            gjs_debug_closure("Closure invocation failed but no exception was set?"
+                              "closure %p", closure);
         goto out;
     }
 
     if (gjs_log_exception(context)) {
-        gjs_debug_closure("Closure invocation succeeded but an exception was set");
+        gjs_debug_closure("Closure invocation succeeded but an exception was set"
+                          " - closure %p", closure);
     }
 
  out:
     JS_EndRequest(context);
 }
 
-gboolean
+bool
 gjs_closure_is_valid(GClosure *closure)
 {
     Closure *c;
@@ -345,7 +346,7 @@ GClosure*
 gjs_closure_new(JSContext  *context,
                 JSObject   *callable,
                 const char *description,
-                gboolean    root_function)
+                bool        root_function)
 {
     Closure *c;
 
@@ -360,7 +361,7 @@ gjs_closure_new(JSContext  *context,
     JS_BeginRequest(context);
 
     c->obj = callable;
-    c->unref_on_global_object_finalized = FALSE;
+    c->unref_on_global_object_finalized = false;
 
     GJS_INC_COUNTER(closure);
 
