@@ -33,9 +33,10 @@
 #include "gtype.h"
 #include "proxyutils.h"
 #include "repo.h"
+#include "gjs/jsapi-wrapper.h"
+#include "gjs/mem.h"
 
-#include <gjs/gjs-module.h>
-#include <gjs/compat.h>
+#include <gjs/context.h>
 #include <util/log.h>
 #include <girepository.h>
 
@@ -55,7 +56,7 @@ typedef struct _Fundamental {
     GIObjectInfoGetValueFunction  get_value_function;
     GIObjectInfoSetValueFunction  set_value_function;
 
-    jsid                          constructor_name;
+    JS::Heap<jsid>                constructor_name;
     GICallableInfo               *constructor_info;
 } Fundamental;
 
@@ -132,14 +133,14 @@ static inline Fundamental *
 proto_priv_from_js(JSContext *context,
                    JSObject  *obj)
 {
-    JSObject *proto;
-    JS_GetPrototype(context, obj, &proto);
+    JS::RootedObject proto(context);
+    JS_GetPrototype(context, obj, proto.address());
     return (Fundamental*) priv_from_js(context, proto);
 }
 
 static FundamentalInstance *
-init_fundamental_instance(JSContext *context,
-                          JSObject  *object)
+init_fundamental_instance(JSContext       *context,
+                          JS::HandleObject object)
 {
     Fundamental *proto_priv;
     FundamentalInstance *priv;
@@ -155,7 +156,7 @@ init_fundamental_instance(JSContext *context,
 
     gjs_debug_lifecycle(GJS_DEBUG_GFUNDAMENTAL,
                         "fundamental instance constructor, obj %p priv %p",
-                        object, priv);
+                        object.get(), priv);
 
     proto_priv = proto_priv_from_js(context, object);
     g_assert(proto_priv != NULL);
@@ -168,10 +169,10 @@ init_fundamental_instance(JSContext *context,
 }
 
 static void
-associate_js_instance_to_fundamental(JSContext *context,
-                                     JSObject  *object,
-                                     void      *gfundamental,
-                                     gboolean   owned_ref)
+associate_js_instance_to_fundamental(JSContext       *context,
+                                     JS::HandleObject object,
+                                     void            *gfundamental,
+                                     bool             owned_ref)
 {
     FundamentalInstance *priv;
 
@@ -183,7 +184,7 @@ associate_js_instance_to_fundamental(JSContext *context,
 
     gjs_debug_lifecycle(GJS_DEBUG_GFUNDAMENTAL,
                         "associated JSObject %p with fundamental %p",
-                        object, gfundamental);
+                        object.get(), gfundamental);
 
     if (!owned_ref)
         priv->prototype->ref_function(gfundamental);
@@ -225,7 +226,7 @@ find_fundamental_constructor(JSContext    *context,
 
 /**/
 
-static JSBool
+static bool
 fundamental_instance_new_resolve_interface(JSContext    *context,
                                            JS::HandleObject obj,
                                            JS::MutableHandleObject objp,
@@ -233,12 +234,12 @@ fundamental_instance_new_resolve_interface(JSContext    *context,
                                            char         *name)
 {
     GIFunctionInfo *method_info;
-    JSBool ret;
+    bool ret;
     GType *interfaces;
     guint n_interfaces;
     guint i;
 
-    ret = JS_TRUE;
+    ret = true;
     interfaces = g_type_interfaces(proto_priv->gtype, &n_interfaces);
     for (i = 0; i < n_interfaces; i++) {
         GIBaseInfo *base_info;
@@ -267,7 +268,7 @@ fundamental_instance_new_resolve_interface(JSContext    *context,
                                         (GICallableInfo *) method_info)) {
                     objp.set(obj);
                 } else {
-                    ret = JS_FALSE;
+                    ret = false;
                 }
             }
 
@@ -292,7 +293,7 @@ fundamental_instance_new_resolve_interface(JSContext    *context,
  * was not resolved; and non-null, referring to obj or one of its prototypes,
  * if id was resolved.
  */
-static JSBool
+static bool
 fundamental_instance_new_resolve(JSContext  *context,
                                  JS::HandleObject obj,
                                  JS::HandleId id,
@@ -301,15 +302,15 @@ fundamental_instance_new_resolve(JSContext  *context,
 {
     FundamentalInstance *priv;
     char *name;
-    JSBool ret = JS_FALSE;
+    bool ret = false;
 
     if (!gjs_get_string_id(context, id, &name))
-        return JS_TRUE; /* not resolved, but no error */
+        return true; /* not resolved, but no error */
 
     priv = priv_from_js(context, obj);
     gjs_debug_jsprop(GJS_DEBUG_GFUNDAMENTAL,
                      "Resolve prop '%s' hook obj %p priv %p",
-                     name, (void *)obj, priv);
+                     name, obj.get(), priv);
 
     if (priv == NULL)
         goto out; /* wrong class */
@@ -339,7 +340,7 @@ fundamental_instance_new_resolve(JSContext  *context,
                               g_base_info_get_namespace((GIBaseInfo *) proto_priv->info),
                               g_base_info_get_name((GIBaseInfo *) proto_priv->info));
                     g_base_info_unref((GIBaseInfo *) method_info);
-                    ret = JS_TRUE;
+                    ret = true;
                     goto out;
                 }
 
@@ -373,48 +374,46 @@ fundamental_instance_new_resolve(JSContext  *context,
          */
     }
 
-    ret = JS_TRUE;
+    ret = true;
  out:
     g_free(name);
     return ret;
 }
 
-static JSBool
+static bool
 fundamental_invoke_constructor(FundamentalInstance *priv,
                                JSContext           *context,
-                               JSObject            *obj,
+                               JS::HandleObject     obj,
                                unsigned             argc,
-                               jsval               *argv,
+                               JS::Value           *argv,
                                GArgument           *rvalue)
 {
-    jsval js_constructor, js_constructor_func;
-    jsid constructor_const;
-    JSBool ret = JS_FALSE;
+    JS::RootedId constructor_const(context,
+        gjs_context_get_const_string(context, GJS_STRING_CONSTRUCTOR));
+    JS::RootedObject js_constructor(context);
 
-    constructor_const = gjs_context_get_const_string(context, GJS_STRING_CONSTRUCTOR);
-    if (!gjs_object_require_property(context, obj, NULL,
-                                     constructor_const, &js_constructor) ||
+    if (!gjs_object_require_property_value(context, obj, NULL,
+                                           constructor_const, &js_constructor) ||
         priv->prototype->constructor_name == JSID_VOID) {
         gjs_throw (context,
                    "Couldn't find a constructor for type %s.%s",
                    g_base_info_get_namespace((GIBaseInfo*) priv->prototype->info),
                    g_base_info_get_name((GIBaseInfo*) priv->prototype->info));
-        goto end;
+        return false;
     }
 
-    if (!gjs_object_require_property(context, JSVAL_TO_OBJECT(js_constructor), NULL,
-                                     priv->prototype->constructor_name, &js_constructor_func)) {
+    JS::RootedObject constructor(context);
+    if (!gjs_object_require_property_value(context, js_constructor, NULL,
+                                           priv->prototype->constructor_name,
+                                           &constructor)) {
         gjs_throw (context,
                    "Couldn't find a constructor for type %s.%s",
                    g_base_info_get_namespace((GIBaseInfo*) priv->prototype->info),
                    g_base_info_get_name((GIBaseInfo*) priv->prototype->info));
-        goto end;
+        return false;
     }
 
-    ret = gjs_invoke_constructor_from_c(context, JSVAL_TO_OBJECT(js_constructor_func), obj, argc, argv, rvalue);
-
- end:
-    return ret;
+    return gjs_invoke_constructor_from_c(context, constructor, obj, argc, argv, rvalue);
 }
 
 /* If we set JSCLASS_CONSTRUCT_PROTOTYPE flag, then this is called on
@@ -438,12 +437,12 @@ GJS_NATIVE_CONSTRUCTOR_DECLARE(fundamental_instance)
 
     gjs_debug_lifecycle(GJS_DEBUG_GFUNDAMENTAL,
                         "fundamental constructor, obj %p priv %p",
-                        object, priv);
+                        object.get(), priv);
 
     if (!fundamental_invoke_constructor(priv, context, object, argc, argv.array(), &ret_value))
-        return JS_FALSE;
+        return false;
 
-    associate_js_instance_to_fundamental(context, object, ret_value.v_pointer, FALSE);
+    associate_js_instance_to_fundamental(context, object, ret_value.v_pointer, false);
 
     g_callable_info_load_return_type((GICallableInfo*) priv->prototype->constructor_info, &return_info);
 
@@ -451,11 +450,11 @@ GJS_NATIVE_CONSTRUCTOR_DECLARE(fundamental_instance)
                                  g_callable_info_get_caller_owns((GICallableInfo*) priv->prototype->constructor_info),
                                  &return_info,
                                  &ret_value))
-        return JS_FALSE;
+        return false;
 
     GJS_NATIVE_CONSTRUCTOR_FINISH(fundamental_instance);
 
-    return JS_TRUE;
+    return true;
 }
 
 static void
@@ -498,17 +497,9 @@ fundamental_finalize(JSFreeOp  *fop,
 static JSBool
 to_string_func(JSContext *context,
                unsigned   argc,
-               jsval     *vp)
+               JS::Value *vp)
 {
-    JS::CallReceiver rec = JS::CallReceiverFromVp(vp);
-    JSObject *obj = JSVAL_TO_OBJECT(rec.thisv());
-
-    FundamentalInstance *priv;
-    JSBool ret = JS_FALSE;
-    jsval retval;
-
-    if (!priv_from_js_with_typecheck(context, obj, &priv))
-        goto out;
+    GJS_GET_PRIV(context, argc, vp, rec, obj, FundamentalInstance, priv);
 
     if (!priv->prototype) {
         Fundamental *proto_priv = (Fundamental *) priv;
@@ -517,21 +508,28 @@ to_string_func(JSContext *context,
                                        (GIBaseInfo *) proto_priv->info,
                                        proto_priv->gtype,
                                        proto_priv->gfundamental,
-                                       &retval))
-            goto out;
+                                       rec.rval()))
+            return false;
     } else {
         if (!_gjs_proxy_to_string_func(context, obj, "fundamental",
                                        (GIBaseInfo *) priv->prototype->info,
                                        priv->prototype->gtype,
                                        priv->gfundamental,
-                                       &retval))
-            goto out;
+                                       rec.rval()))
+            return false;
     }
 
-    ret = JS_TRUE;
-    rec.rval().set(retval);
- out:
-    return ret;
+    return true;
+}
+
+static void
+fundamental_trace(JSTracer *tracer,
+                  JSObject *obj)
+{
+    Fundamental *priv = reinterpret_cast<Fundamental *>(JS_GetPrivate(obj));
+
+    JS_CallHeapIdTracer(tracer, &priv->constructor_name,
+                        "Fundamental::constructor_name");
 }
 
 /* The bizarre thing about this vtable is that it applies to both
@@ -560,20 +558,20 @@ struct JSClass gjs_fundamental_instance_class = {
     (JSResolveOp) fundamental_instance_new_resolve, /* needs cast since it's the new resolve signature */
     JS_ConvertStub,
     fundamental_finalize,
-    NULL,
-    NULL,
-    NULL,
-    NULL,
-    NULL
+    NULL,  /* checkAccess */
+    NULL,  /* call */
+    NULL,  /* hasInstance */
+    NULL,  /* construct */
+    fundamental_trace
 };
 
 static JSPropertySpec gjs_fundamental_instance_proto_props[] = {
-    { NULL }
+    JS_PS_END
 };
 
 static JSFunctionSpec gjs_fundamental_instance_proto_funcs[] = {
-    { "toString", JSOP_WRAPPER((JSNative)to_string_func), 0, 0 },
-    { NULL }
+    JS_FS("toString", to_string_func, 0, 0),
+    JS_FS_END
 };
 
 static JSObject *
@@ -581,10 +579,8 @@ gjs_lookup_fundamental_prototype(JSContext    *context,
                                  GIObjectInfo *info,
                                  GType         gtype)
 {
-    JSObject *in_object;
-    JSObject *constructor;
+    JS::RootedObject in_object(context);
     const char *constructor_name;
-    jsval value;
 
     if (info) {
         in_object = gjs_lookup_namespace_object(context, (GIBaseInfo*) info);
@@ -597,19 +593,21 @@ gjs_lookup_fundamental_prototype(JSContext    *context,
     if (G_UNLIKELY (!in_object))
         return NULL;
 
-    if (!JS_GetProperty(context, in_object, constructor_name, &value))
+    JS::RootedValue value(context);
+    if (!JS_GetProperty(context, in_object, constructor_name, value.address()))
         return NULL;
 
-    if (JSVAL_IS_VOID(value)) {
+    JS::RootedObject constructor(context);
+    if (value.isUndefined()) {
         /* In case we're looking for a private type, and we don't find it,
            we need to define it first.
         */
         gjs_define_fundamental_class(context, in_object, info, &constructor, NULL);
     } else {
-        if (G_UNLIKELY (!JSVAL_IS_OBJECT(value) || JSVAL_IS_NULL(value)))
+        if (G_UNLIKELY (!value.isObject()))
             return NULL;
 
-        constructor = JSVAL_TO_OBJECT(value);
+        constructor = &value.toObject();
     }
 
     g_assert(constructor != NULL);
@@ -618,10 +616,10 @@ gjs_lookup_fundamental_prototype(JSContext    *context,
                                        GJS_STRING_PROTOTYPE, &value))
         return NULL;
 
-    if (G_UNLIKELY (!JSVAL_IS_OBJECT(value)))
+    if (G_UNLIKELY (!value.isObjectOrNull()))
         return NULL;
 
-    return JSVAL_TO_OBJECT(value);
+    return value.toObjectOrNull();
 }
 
 static JSObject*
@@ -640,20 +638,18 @@ gjs_lookup_fundamental_prototype_from_gtype(JSContext *context,
     return proto;
 }
 
-JSBool
-gjs_define_fundamental_class(JSContext     *context,
-                             JSObject      *in_object,
-                             GIObjectInfo  *info,
-                             JSObject     **constructor_p,
-                             JSObject     **prototype_p)
+bool
+gjs_define_fundamental_class(JSContext              *context,
+                             JS::HandleObject        in_object,
+                             GIObjectInfo           *info,
+                             JS::MutableHandleObject constructor,
+                             JS::MutableHandleObject prototype)
 {
     const char *constructor_name;
-    JSObject *prototype;
-    jsval value;
+    JS::Value value;
     jsid js_constructor_name = JSID_VOID;
-    JSObject *parent_proto;
+    JS::RootedObject parent_proto(context);
     Fundamental *priv;
-    JSObject *constructor;
     GType parent_gtype;
     GType gtype;
     GIFunctionInfo *constructor_info;
@@ -668,7 +664,6 @@ gjs_define_fundamental_class(JSContext     *context,
 
     gtype = g_registered_type_info_get_g_type (info);
     parent_gtype = g_type_parent(gtype);
-    parent_proto = NULL;
     if (parent_gtype != G_TYPE_INVALID)
         parent_proto = gjs_lookup_fundamental_prototype_from_gtype(context,
                                                                    parent_gtype);
@@ -693,8 +688,8 @@ gjs_define_fundamental_class(JSContext     *context,
                                 NULL,
                                 /* funcs of constructor, MyConstructor.myfunc() */
                                 NULL,
-                                &prototype,
-                                &constructor)) {
+                                prototype,
+                                constructor)) {
         gjs_log_exception(context);
         g_error("Can't init class %s", constructor_name);
     }
@@ -719,8 +714,8 @@ gjs_define_fundamental_class(JSContext     *context,
 
     gjs_debug(GJS_DEBUG_GFUNDAMENTAL,
               "Defined class %s prototype is %p class %p in object %p constructor %s.%s.%s",
-              constructor_name, prototype, JS_GetClass(prototype),
-              in_object,
+              constructor_name, prototype.get(), JS_GetClass(prototype),
+              in_object.get(),
               constructor_info != NULL ? g_base_info_get_namespace(constructor_info) : "unknown",
               constructor_info != NULL ? g_base_info_get_name(g_base_info_get_container(constructor_info)) : "unknown",
               constructor_info != NULL ? g_base_info_get_name(constructor_info) : "unknown");
@@ -735,16 +730,11 @@ gjs_define_fundamental_class(JSContext     *context,
 
     gjs_object_define_static_methods(context, constructor, gtype, info);
 
-    value = OBJECT_TO_JSVAL(gjs_gtype_create_gtype_wrapper(context, gtype));
+    value = JS::ObjectOrNullValue(gjs_gtype_create_gtype_wrapper(context, gtype));
     JS_DefineProperty(context, constructor, "$gtype", value,
                       NULL, NULL, JSPROP_PERMANENT);
 
-    if (constructor_p)
-        *constructor_p = constructor;
-    if (prototype_p)
-        *prototype_p = prototype;
-
-    return JS_TRUE;
+    return true;
 }
 
 JSObject*
@@ -753,12 +743,11 @@ gjs_object_from_g_fundamental(JSContext    *context,
                               void         *gfundamental)
 {
     JSObject *proto;
-    JSObject *object;
 
     if (gfundamental == NULL)
         return NULL;
 
-    object = _fundamental_lookup_object(gfundamental);
+    JS::RootedObject object(context, _fundamental_lookup_object(gfundamental));
     if (object)
         return object;
 
@@ -780,7 +769,7 @@ gjs_object_from_g_fundamental(JSContext    *context,
 
     init_fundamental_instance(context, object);
 
-    associate_js_instance_to_fundamental(context, object, gfundamental, FALSE);
+    associate_js_instance_to_fundamental(context, object, gfundamental, false);
 
  out:
     return object;
@@ -791,11 +780,11 @@ gjs_fundamental_from_g_value(JSContext    *context,
                              const GValue *value,
                              GType         gtype)
 {
-    JSObject *proto;
     Fundamental *proto_priv;
     void *fobj;
 
-    proto = gjs_lookup_fundamental_prototype_from_gtype(context, gtype);
+    JS::RootedObject proto(context,
+                           gjs_lookup_fundamental_prototype_from_gtype(context, gtype));
     if (!proto)
         return NULL;
 
@@ -812,8 +801,8 @@ gjs_fundamental_from_g_value(JSContext    *context,
 }
 
 void*
-gjs_g_fundamental_from_object(JSContext *context,
-                              JSObject  *obj)
+gjs_g_fundamental_from_object(JSContext       *context,
+                              JS::HandleObject obj)
 {
     FundamentalInstance *priv;
 
@@ -824,7 +813,7 @@ gjs_g_fundamental_from_object(JSContext *context,
 
     if (priv == NULL) {
         gjs_throw(context,
-                  "No introspection information for %p", obj);
+                  "No introspection information for %p", obj.get());
         return NULL;
     }
 
@@ -839,25 +828,17 @@ gjs_g_fundamental_from_object(JSContext *context,
     return priv->gfundamental;
 }
 
-JSBool
-gjs_typecheck_is_fundamental(JSContext     *context,
-                             JSObject      *object,
-                             JSBool         throw_error)
-{
-    return do_base_typecheck(context, object, throw_error);
-}
-
-JSBool
-gjs_typecheck_fundamental(JSContext *context,
-                          JSObject  *object,
-                          GType      expected_gtype,
-                          JSBool     throw_error)
+bool
+gjs_typecheck_fundamental(JSContext       *context,
+                          JS::HandleObject object,
+                          GType            expected_gtype,
+                          bool             throw_error)
 {
     FundamentalInstance *priv;
-    JSBool result;
+    bool result;
 
     if (!do_base_typecheck(context, object, throw_error))
-        return JS_FALSE;
+        return false;
 
     priv = priv_from_js(context, object);
     g_assert(priv != NULL);
@@ -871,13 +852,13 @@ gjs_typecheck_fundamental(JSContext *context,
                       proto_priv->info ? g_base_info_get_name((GIBaseInfo *) proto_priv->info) : g_type_name(proto_priv->gtype));
         }
 
-        return JS_FALSE;
+        return false;
     }
 
     if (expected_gtype != G_TYPE_NONE)
         result = g_type_is_a(priv->prototype->gtype, expected_gtype);
     else
-        result = JS_TRUE;
+        result = true;
 
     if (!result && throw_error) {
         if (priv->prototype->info) {
@@ -901,11 +882,9 @@ void *
 gjs_fundamental_ref(JSContext     *context,
                     void          *gfundamental)
 {
-    JSObject *proto;
     Fundamental *proto_priv;
-
-    proto = gjs_lookup_fundamental_prototype_from_gtype(context,
-                                                        G_TYPE_FROM_INSTANCE(gfundamental));
+    JS::RootedObject proto(context,
+        gjs_lookup_fundamental_prototype_from_gtype(context, G_TYPE_FROM_INSTANCE(gfundamental)));
 
     proto_priv = (Fundamental *) priv_from_js(context, proto);
 
@@ -916,11 +895,9 @@ void
 gjs_fundamental_unref(JSContext    *context,
                       void         *gfundamental)
 {
-    JSObject *proto;
     Fundamental *proto_priv;
-
-    proto = gjs_lookup_fundamental_prototype_from_gtype(context,
-                                                        G_TYPE_FROM_INSTANCE(gfundamental));
+    JS::RootedObject proto(context,
+        gjs_lookup_fundamental_prototype_from_gtype(context, G_TYPE_FROM_INSTANCE(gfundamental)));
 
     proto_priv = (Fundamental *) priv_from_js(context, proto);
 
