@@ -25,12 +25,16 @@
 #include <config.h>
 
 #include "gtype.h"
-
-#include <gjs/gjs-module.h>
-#include <gjs/compat.h>
+#include "gjs/jsapi-wrapper.h"
 #include <util/log.h>
 #include <girepository.h>
 
+static JS::Value
+gjs_gtype_create_proto(JSContext       *context,
+                       JS::HandleObject module,
+                       const char      *proto_name,
+                       JS::HandleObject parent);
+
 GJS_DEFINE_PROTO_ABSTRACT("GIRepositoryGType", gtype, 0);
 
 /* priv_from_js adds a "*", so this returns "void *" */
@@ -64,67 +68,52 @@ gjs_gtype_finalize(JSFreeOp *fop,
 static JSBool
 to_string_func(JSContext *context,
                unsigned   argc,
-               jsval     *vp)
+               JS::Value *vp)
 {
-    JS::CallReceiver rec = JS::CallReceiverFromVp(vp);
-    JSObject *obj = JSVAL_TO_OBJECT(rec.thisv());
-
+    GJS_GET_PRIV(context, argc, vp, rec, obj, void, priv);
     GType gtype;
     gchar *strval;
-    JSBool ret;
-    jsval retval;
-    
-    gtype = GPOINTER_TO_SIZE(priv_from_js(context, obj));
+    bool ret;
+
+    gtype = GPOINTER_TO_SIZE(priv);
 
     if (gtype == 0)
         strval = g_strdup("[object GType prototype]");
     else
         strval = g_strdup_printf("[object GType for '%s']",
                                  g_type_name(gtype));
-    ret = gjs_string_from_utf8(context, strval, -1, &retval);
-    if (ret)
-        rec.rval().set(retval);
+    ret = gjs_string_from_utf8(context, strval, -1, rec.rval());
     g_free(strval);
     return ret;
 }
 
 static JSBool
 get_name_func (JSContext *context,
-               JS::HandleObject obj,
-               JS::HandleId id,
-               JS::MutableHandleValue vp)
+               unsigned   argc,
+               JS::Value *vp)
 {
+    GJS_GET_PRIV(context, argc, vp, rec, obj, void, priv);
     GType gtype;
-    JSBool ret;
-    jsval retval;
-    JS::CallReceiver rec = JS::CallReceiverFromVp(vp.address());
 
-    gtype = GPOINTER_TO_SIZE(priv_from_js(context, obj));
+    gtype = GPOINTER_TO_SIZE(priv);
 
     if (gtype == 0) {
-        rec.rval().set(JSVAL_NULL);
-        return TRUE;
-    } else {
-        ret = gjs_string_from_utf8(context, g_type_name(gtype), -1, &retval);
-        if (ret)
-            rec.rval().set(retval);
-        return ret;
+        rec.rval().setNull();
+        return true;
     }
+    return gjs_string_from_utf8(context, g_type_name(gtype), -1, rec.rval());
 }
 
 /* Properties */
 JSPropertySpec gjs_gtype_proto_props[] = {
-    { "name", 0,
-      JSPROP_READONLY | JSPROP_PERMANENT | JSPROP_SHARED,
-      JSOP_WRAPPER((JSPropertyOp)get_name_func),
-      JSOP_WRAPPER(JS_StrictPropertyStub) },
-    { NULL },
+    JS_PSG("name", get_name_func, JSPROP_READONLY | JSPROP_PERMANENT),
+    JS_PS_END,
 };
 
 /* Functions */
 JSFunctionSpec gjs_gtype_proto_funcs[] = {
-    { "toString", JSOP_WRAPPER((JSNative)to_string_func), 0, 0 },
-    { NULL }
+    JS_FS("toString", to_string_func, 0, 0),
+    JS_FS_END
 };
 
 JSObject *
@@ -132,13 +121,12 @@ gjs_gtype_create_gtype_wrapper (JSContext *context,
                                 GType      gtype)
 {
     JSObject *object;
-    JSObject *global;
 
     JS_BeginRequest(context);
 
     /* put constructor for GIRepositoryGType() in the global namespace */
-    global = gjs_get_import_global(context);
-    gjs_gtype_create_proto(context, global, "GIRepositoryGType", NULL);
+    JS::RootedObject global(context, gjs_get_import_global(context));
+    gjs_gtype_create_proto(context, global, "GIRepositoryGType", JS::NullPtr());
 
     object = (JSObject*) g_type_get_qdata(gtype, gjs_get_gtype_wrapper_quark());
     if (object != NULL)
@@ -157,12 +145,12 @@ gjs_gtype_create_gtype_wrapper (JSContext *context,
 }
 
 static GType
-_gjs_gtype_get_actual_gtype (JSContext *context,
-                             JSObject  *object,
-                             int        recurse)
+_gjs_gtype_get_actual_gtype(JSContext       *context,
+                            JS::HandleObject object,
+                            int              recurse)
 {
     GType gtype = G_TYPE_INVALID;
-    jsval gtype_val = JSVAL_VOID;
+    JS::Value gtype_val = JS::UndefinedValue();
 
     JS_BeginRequest(context);
     if (JS_InstanceOf(context, object, &gjs_gtype_class, NULL)) {
@@ -173,7 +161,7 @@ _gjs_gtype_get_actual_gtype (JSContext *context,
     /* OK, we don't have a GType wrapper object -- grab the "$gtype"
      * property on that and hope it's a GType wrapper object */
     if (!JS_GetProperty(context, object, "$gtype", &gtype_val) ||
-        !JSVAL_IS_OBJECT(gtype_val)) {
+        !gtype_val.isObject()) {
 
         /* OK, so we're not a class. But maybe we're an instance. Check
            for "constructor" and recurse on that. */
@@ -181,8 +169,10 @@ _gjs_gtype_get_actual_gtype (JSContext *context,
             goto out;
     }
 
-    if (recurse > 0 && JSVAL_IS_OBJECT(gtype_val))
-        gtype = _gjs_gtype_get_actual_gtype(context, JSVAL_TO_OBJECT(gtype_val), recurse - 1);
+    if (recurse > 0 && gtype_val.isObject()) {
+        JS::RootedObject gtype_obj(context, &gtype_val.toObject());
+        gtype = _gjs_gtype_get_actual_gtype(context, gtype_obj, recurse - 1);
+    }
 
  out:
     JS_EndRequest(context);
@@ -190,8 +180,8 @@ _gjs_gtype_get_actual_gtype (JSContext *context,
 }
 
 GType
-gjs_gtype_get_actual_gtype (JSContext *context,
-                            JSObject  *object)
+gjs_gtype_get_actual_gtype(JSContext       *context,
+                           JS::HandleObject object)
 {
     /* 2 means: recurse at most three times (including this
        call).
@@ -204,10 +194,10 @@ gjs_gtype_get_actual_gtype (JSContext *context,
     return _gjs_gtype_get_actual_gtype(context, object, 2);
 }
 
-JSBool
+bool
 gjs_typecheck_gtype (JSContext             *context,
-                     JSObject              *obj,
-                     JSBool                 throw_error)
+                     JS::HandleObject       obj,
+                     bool                   throw_error)
 {
     return do_base_typecheck(context, obj, throw_error);
 }
