@@ -28,9 +28,9 @@
 #include "gtype.h"
 #include "interface.h"
 #include "repo.h"
+#include "gjs/jsapi-wrapper.h"
+#include "gjs/mem.h"
 
-#include <gjs/gjs-module.h>
-#include <gjs/compat.h>
 #include <util/log.h>
 
 #include <girepository.h>
@@ -38,6 +38,9 @@
 typedef struct {
     GIInterfaceInfo *info;
     GType gtype;
+    /* the GTypeInterface vtable wrapped by this JS Object (only used for
+       prototypes) */
+    GTypeInterface *vtable;
 } Interface;
 
 extern struct JSClass gjs_interface_class;
@@ -60,13 +63,15 @@ interface_finalize(JSFreeOp *fop,
     if (priv->info != NULL)
         g_base_info_unref((GIBaseInfo*)priv->info);
 
+    g_clear_pointer(&priv->vtable, (GDestroyNotify)g_type_default_interface_unref);
+
     GJS_DEC_COUNTER(interface);
     g_slice_free(Interface, priv);
 }
 
-static JSBool
+static bool
 gjs_define_static_methods(JSContext       *context,
-                          JSObject        *constructor,
+                          JS::HandleObject constructor,
                           GType            gtype,
                           GIInterfaceInfo *info)
 {
@@ -96,10 +101,10 @@ gjs_define_static_methods(JSContext       *context,
 
         g_base_info_unref((GIBaseInfo*) meth_info);
     }
-    return JS_TRUE;
+    return true;
 }
 
-static JSBool
+static bool
 interface_new_resolve(JSContext *context,
                       JS::HandleObject obj,
                       JS::HandleId id,
@@ -108,17 +113,25 @@ interface_new_resolve(JSContext *context,
 {
     Interface *priv;
     char *name;
-    JSBool ret = JS_FALSE;
+    bool ret = false;
     GIFunctionInfo *method_info;
 
     if (!gjs_get_string_id(context, id, &name))
-        return JS_TRUE;
+        return true;
 
     priv = priv_from_js(context, obj);
 
     if (priv == NULL)
         goto out;
 
+    /* If we have no GIRepository information then this interface was defined
+     * from within GJS. In that case, it has no properties that need to be
+     * resolved from within C code, as interfaces cannot inherit. */
+    if (priv->info == NULL) {
+        ret = true;
+        goto out;
+    }
+
     method_info = g_interface_info_find_method((GIInterfaceInfo*) priv->info, name);
 
     if (method_info != NULL) {
@@ -136,7 +149,7 @@ interface_new_resolve(JSContext *context,
         g_base_info_unref((GIBaseInfo*)method_info);
     }
 
-    ret = JS_TRUE;
+    ret = true;
 
  out:
     g_free (name);
@@ -156,35 +169,36 @@ struct JSClass gjs_interface_class = {
     (JSResolveOp) interface_new_resolve,
     JS_ConvertStub,
     interface_finalize,
-    NULL,
-    NULL,
-    NULL, NULL, NULL
+    JSCLASS_NO_OPTIONAL_MEMBERS
 };
 
 JSPropertySpec gjs_interface_proto_props[] = {
-    { NULL }
+    JS_PS_END
 };
 
 JSFunctionSpec gjs_interface_proto_funcs[] = {
-    { NULL }
+    JS_FS_END
 };
 
-JSBool
-gjs_define_interface_class(JSContext       *context,
-                           JSObject        *in_object,
-                           GIInterfaceInfo *info)
+bool
+gjs_define_interface_class(JSContext              *context,
+                           JS::HandleObject        in_object,
+                           GIInterfaceInfo        *info,
+                           GType                   gtype,
+                           JS::MutableHandleObject constructor)
 {
     Interface *priv;
     const char *constructor_name;
-    JSObject *constructor;
-    JSObject *prototype;
-    jsval value;
+    const char *ns;
+    JS::RootedObject prototype(context);
+    JS::Value value;
 
-    constructor_name = g_base_info_get_name((GIBaseInfo*)info);
+    ns = gjs_get_names_from_gtype_and_gi_info(gtype, (GIBaseInfo *) info,
+                                              &constructor_name);
 
     if (!gjs_init_class_dynamic(context, in_object,
-                                NULL,
-                                g_base_info_get_namespace((GIBaseInfo*)info),
+                                JS::NullPtr(),
+                                ns,
                                 constructor_name,
                                 &gjs_interface_class,
                                 gjs_interface_constructor, 0,
@@ -197,30 +211,33 @@ gjs_define_interface_class(JSContext       *context,
                                 /* funcs of constructor, MyConstructor.myfunc() */
                                 NULL,
                                 &prototype,
-                                &constructor)) {
+                                constructor)) {
         g_error("Can't init class %s", constructor_name);
     }
 
     GJS_INC_COUNTER(interface);
     priv = g_slice_new0(Interface);
-    priv->info = info;
-    priv->gtype = g_registered_type_info_get_g_type(priv->info);
-    g_base_info_ref((GIBaseInfo*)priv->info);
+    priv->info = info == NULL ? NULL : g_base_info_ref((GIBaseInfo *) info);
+    priv->gtype = gtype;
+    priv->vtable = (GTypeInterface *) g_type_default_interface_ref(gtype);
     JS_SetPrivate(prototype, priv);
 
-    gjs_define_static_methods(context, constructor, priv->gtype, priv->info);
+    /* If we have no GIRepository information, then this interface was defined
+     * from within GJS and therefore has no C static methods to be defined. */
+    if (priv->info)
+        gjs_define_static_methods(context, constructor, priv->gtype, priv->info);
 
-    value = OBJECT_TO_JSVAL(gjs_gtype_create_gtype_wrapper(context, priv->gtype));
+    value = JS::ObjectOrNullValue(gjs_gtype_create_gtype_wrapper(context, priv->gtype));
     JS_DefineProperty(context, constructor, "$gtype", value,
                       NULL, NULL, JSPROP_PERMANENT);
 
-    return JS_TRUE;
+    return true;
 }
 
-JSBool
-gjs_lookup_interface_constructor(JSContext *context,
-                                 GType      gtype,
-                                 jsval     *value_p)
+bool
+gjs_lookup_interface_constructor(JSContext             *context,
+                                 GType                  gtype,
+                                 JS::MutableHandleValue value_p)
 {
     JSObject *constructor;
     GIBaseInfo *interface_info;
@@ -230,7 +247,7 @@ gjs_lookup_interface_constructor(JSContext *context,
     if (interface_info == NULL) {
         gjs_throw(context, "Cannot expose non introspectable interface %s",
                   g_type_name(gtype));
-        return JS_FALSE;
+        return false;
     }
 
     g_assert(g_base_info_get_type(interface_info) ==
@@ -238,10 +255,10 @@ gjs_lookup_interface_constructor(JSContext *context,
 
     constructor = gjs_lookup_generic_constructor(context, interface_info);
     if (G_UNLIKELY (constructor == NULL))
-        return JS_FALSE;
+        return false;
 
     g_base_info_unref(interface_info);
 
-    *value_p = OBJECT_TO_JSVAL(constructor);
-    return JS_TRUE;
+    value_p.setObject(*constructor);
+    return true;
 }
