@@ -26,8 +26,8 @@
 #include "ns.h"
 #include "repo.h"
 #include "param.h"
-#include <gjs/gjs-module.h>
-#include <gjs/compat.h>
+#include "gjs/jsapi-wrapper.h"
+#include "gjs/mem.h"
 
 #include <util/log.h>
 #include <girepository.h>
@@ -55,7 +55,7 @@ GJS_DEFINE_PRIV_FROM_JS(Ns, gjs_ns_class)
  * was not resolved; and non-null, referring to obj or one of its prototypes,
  * if id was resolved.
  */
-static JSBool
+static bool
 ns_new_resolve(JSContext *context,
                JS::HandleObject obj,
                JS::HandleId id,
@@ -66,26 +66,26 @@ ns_new_resolve(JSContext *context,
     char *name;
     GIRepository *repo;
     GIBaseInfo *info;
-    JSBool ret = JS_FALSE;
-    gboolean defined;
+    bool ret = false;
+    bool defined;
 
     if (!gjs_get_string_id(context, id, &name))
-        return JS_TRUE; /* not resolved, but no error */
+        return true; /* not resolved, but no error */
 
     /* let Object.prototype resolve these */
     if (strcmp(name, "valueOf") == 0 ||
         strcmp(name, "toString") == 0) {
-        ret = JS_TRUE;
+        ret = true;
         goto out;
     }
 
     priv = priv_from_js(context, obj);
     gjs_debug_jsprop(GJS_DEBUG_GNAMESPACE,
                      "Resolve prop '%s' hook obj %p priv %p",
-                     name, (void *)obj, priv);
+                     name, obj.get(), priv);
 
     if (priv == NULL) {
-        ret = JS_TRUE; /* we are the prototype, or have the wrong class */
+        ret = true; /* we are the prototype, or have the wrong class */
         goto out;
     }
 
@@ -95,9 +95,9 @@ ns_new_resolve(JSContext *context,
 
     info = g_irepository_find_by_name(repo, priv->gi_namespace, name);
     if (info == NULL) {
-        /* No property defined, but no error either, so return TRUE */
+        /* No property defined, but no error either, so return true */
         JS_EndRequest(context);
-        ret = JS_TRUE;
+        ret = true;
         goto out;
     }
 
@@ -111,7 +111,7 @@ ns_new_resolve(JSContext *context,
         g_base_info_unref(info);
         if (defined)
             objp.set(obj); /* we defined the property in this object */
-        ret = JS_TRUE;
+        ret = true;
     } else {
         gjs_debug(GJS_DEBUG_GNAMESPACE,
                   "Failed to define info '%s'",
@@ -128,26 +128,15 @@ ns_new_resolve(JSContext *context,
 
 static JSBool
 get_name (JSContext *context,
-          JS::HandleObject obj,
-          JS::HandleId id,
-          jsval *vp)
+          unsigned   argc,
+          JS::Value *vp)
 {
-    Ns *priv;
-    jsval retval;
-    JSBool ret = JS_FALSE;
-
-    priv = priv_from_js(context, obj);
+    GJS_GET_PRIV(context, argc, vp, args, obj, Ns, priv);
 
     if (priv == NULL)
-        goto out;
+        return false;
 
-    if (gjs_string_from_utf8(context, priv->gi_namespace, -1, &retval)) {
-        *vp = retval;
-        ret = JS_TRUE;
-    }
-
- out:
-    return ret;
+    return gjs_string_from_utf8(context, priv->gi_namespace, -1, args.rval());
 }
 
 GJS_NATIVE_CONSTRUCTOR_DEFINE_ABSTRACT(ns)
@@ -191,25 +180,23 @@ struct JSClass gjs_ns_class = {
 };
 
 JSPropertySpec gjs_ns_proto_props[] = {
-    { "__name__", 0, GJS_MODULE_PROP_FLAGS | JSPROP_READONLY, (JSPropertyOp)get_name, NULL },
-    { NULL }
+    JS_PSG("__name__", get_name, GJS_MODULE_PROP_FLAGS | JSPROP_READONLY),
+    JS_PS_END
 };
 
 JSFunctionSpec gjs_ns_proto_funcs[] = {
-    { NULL }
+    JS_FS_END
 };
 
 static JSObject*
 ns_new(JSContext    *context,
        const char   *ns_name)
 {
-    JSObject *ns;
-    JSObject *global;
     Ns *priv;
     JSBool found;
 
     /* put constructor in the global namespace */
-    global = gjs_get_import_global(context);
+    JS::RootedObject global(context, gjs_get_import_global(context));
 
     if (!JS_HasProperty(context, global, gjs_ns_class.name, &found))
         return NULL;
@@ -244,7 +231,7 @@ ns_new(JSContext    *context,
                   gjs_ns_class.name, prototype);
     }
 
-    ns = JS_NewObject(context, &gjs_ns_class, NULL, global);
+    JS::RootedObject ns(context, JS_NewObject(context, &gjs_ns_class, NULL, global));
     if (ns == NULL)
         g_error("No memory to create ns object");
 
@@ -255,7 +242,8 @@ ns_new(JSContext    *context,
     g_assert(priv_from_js(context, ns) == NULL);
     JS_SetPrivate(ns, priv);
 
-    gjs_debug_lifecycle(GJS_DEBUG_GNAMESPACE, "ns constructor, obj %p priv %p", ns, priv);
+    gjs_debug_lifecycle(GJS_DEBUG_GNAMESPACE, "ns constructor, obj %p priv %p",
+                        ns.get(), priv);
 
     priv = priv_from_js(context, ns);
     priv->gi_namespace = g_strdup(ns_name);
