@@ -31,8 +31,8 @@
 #include "repo.h"
 #include "gtype.h"
 #include "function.h"
-#include <gjs/gjs-module.h>
-#include <gjs/compat.h>
+#include "gjs/jsapi-wrapper.h"
+#include "gjs/mem.h"
 
 #include <util/log.h>
 
@@ -57,7 +57,7 @@ GJS_DEFINE_PRIV_FROM_JS(Param, gjs_param_class)
  * was not resolved; and non-null, referring to obj or one of its prototypes,
  * if id was resolved.
  */
-static JSBool
+static bool
 param_new_resolve(JSContext *context,
                   JS::HandleObject obj,
                   JS::HandleId id,
@@ -68,16 +68,16 @@ param_new_resolve(JSContext *context,
     GIFunctionInfo *method_info;
     Param *priv;
     char *name;
-    JSBool ret = JS_FALSE;
+    bool ret = false;
 
     if (!gjs_get_string_id(context, id, &name))
-        return JS_TRUE; /* not resolved, but no error */
+        return true; /* not resolved, but no error */
 
     priv = priv_from_js(context, obj);
 
     if (priv != NULL) {
         /* instance, not prototype */
-        ret = JS_TRUE;
+        ret = true;
         goto out;
     }
 
@@ -85,7 +85,7 @@ param_new_resolve(JSContext *context,
     method_info = g_object_info_find_method(info, name);
 
     if (method_info == NULL) {
-        ret = JS_TRUE;
+        ret = true;
         goto out;
     }
 #if GJS_VERBOSE_ENABLE_GI_USAGE
@@ -107,7 +107,7 @@ param_new_resolve(JSContext *context,
 
     g_base_info_unref( (GIBaseInfo*) method_info);
 
-    ret = JS_TRUE;
+    ret = true;
  out:
     g_free(name);
     if (info != NULL)
@@ -122,7 +122,7 @@ GJS_NATIVE_CONSTRUCTOR_DECLARE(param)
     GJS_NATIVE_CONSTRUCTOR_PRELUDE(param);
     GJS_INC_COUNTER(param);
     GJS_NATIVE_CONSTRUCTOR_FINISH(param);
-    return JS_TRUE;
+    return true;
 }
 
 static void
@@ -164,70 +164,65 @@ struct JSClass gjs_param_class = {
     (JSResolveOp) param_new_resolve,
     JS_ConvertStub,
     param_finalize,
-    NULL,
-    NULL,
-    NULL, NULL, NULL
+    JSCLASS_NO_OPTIONAL_MEMBERS
 };
 
 JSPropertySpec gjs_param_proto_props[] = {
-    { NULL }
+    JS_PS_END
 };
 
 JSFunctionSpec gjs_param_proto_funcs[] = {
-    { NULL }
+    JS_FS_END
 };
 
 static JSFunctionSpec gjs_param_constructor_funcs[] = {
-    { NULL }
+    JS_FS_END
 };
 
 static JSObject*
 gjs_lookup_param_prototype(JSContext    *context)
 {
     JSObject *in_object;
-    JSObject *constructor;
-    jsid gobject_name;
-    jsval value;
 
-    gobject_name = gjs_intern_string_to_id(context, "GObject");
+    JS::RootedId gobject_name(context, gjs_intern_string_to_id(context, "GObject"));
     in_object = gjs_lookup_namespace_object_by_name(context, gobject_name);
 
     if (G_UNLIKELY (!in_object))
         return NULL;
 
-    if (!JS_GetProperty(context, in_object, "ParamSpec", &value))
+    JS::RootedValue value(context);
+    if (!JS_GetProperty(context, in_object, "ParamSpec", value.address()))
         return NULL;
 
-    if (G_UNLIKELY (!JSVAL_IS_OBJECT(value) || JSVAL_IS_NULL(value)))
+    if (G_UNLIKELY (!value.isObject()))
         return NULL;
 
-    constructor = JSVAL_TO_OBJECT(value);
+    JS::RootedObject constructor(context, &value.toObject());
     g_assert(constructor != NULL);
 
     if (!gjs_object_get_property_const(context, constructor,
                                        GJS_STRING_PROTOTYPE, &value))
         return NULL;
 
-    if (G_UNLIKELY (!JSVAL_IS_OBJECT(value)))
+    if (G_UNLIKELY (!value.isObjectOrNull()))
         return NULL;
 
-    return JSVAL_TO_OBJECT(value);
+    return value.toObjectOrNull();
 }
 
 void
-gjs_define_param_class(JSContext    *context,
-                       JSObject     *in_object)
+gjs_define_param_class(JSContext       *context,
+                       JS::HandleObject in_object)
 {
     const char *constructor_name;
-    JSObject *prototype;
-    jsval value;
-    JSObject *constructor;
+    JS::Value value;
+    JS::RootedObject prototype(context), constructor(context);
     GIObjectInfo *info;
 
     constructor_name = "ParamSpec";
 
     if (!gjs_init_class_dynamic(context, in_object,
-                                NULL,
+                                JS::NullPtr(),
                                 "GObject",
                                 constructor_name,
                                 &gjs_param_class,
@@ -245,7 +240,7 @@ gjs_define_param_class(JSContext    *context,
         g_error("Can't init class %s", constructor_name);
     }
 
-    value = OBJECT_TO_JSVAL(gjs_gtype_create_gtype_wrapper(context, G_TYPE_PARAM));
+    value = JS::ObjectOrNullValue(gjs_gtype_create_gtype_wrapper(context, G_TYPE_PARAM));
     JS_DefineProperty(context, constructor, "$gtype", value,
                       NULL, NULL, JSPROP_PERMANENT);
 
@@ -254,7 +249,8 @@ gjs_define_param_class(JSContext    *context,
     g_base_info_unref( (GIBaseInfo*) info);
 
     gjs_debug(GJS_DEBUG_GPARAM, "Defined class %s prototype is %p class %p in object %p",
-              constructor_name, prototype, JS_GetClass(prototype), in_object);
+              constructor_name, prototype.get(), JS_GetClass(prototype),
+              in_object.get());
 }
 
 JSObject*
@@ -294,8 +290,8 @@ gjs_param_from_g_param(JSContext    *context,
 }
 
 GParamSpec*
-gjs_g_param_from_param(JSContext    *context,
-                       JSObject     *obj)
+gjs_g_param_from_param(JSContext       *context,
+                       JS::HandleObject obj)
 {
     Param *priv;
 
@@ -307,17 +303,17 @@ gjs_g_param_from_param(JSContext    *context,
     return priv->gparam;
 }
 
-JSBool
-gjs_typecheck_param(JSContext     *context,
-                    JSObject      *object,
-                    GType          expected_type,
-                    JSBool         throw_error)
+bool
+gjs_typecheck_param(JSContext       *context,
+                    JS::HandleObject object,
+                    GType            expected_type,
+                    bool             throw_error)
 {
     Param *priv;
-    JSBool result;
+    bool result;
 
     if (!do_base_typecheck(context, object, throw_error))
-        return JS_FALSE;
+        return false;
 
     priv = priv_from_js(context, object);
 
@@ -328,13 +324,13 @@ gjs_typecheck_param(JSContext     *context,
                              "cannot convert to a GObject.ParamSpec instance");
         }
 
-        return JS_FALSE;
+        return false;
     }
 
     if (expected_type != G_TYPE_NONE)
         result = g_type_is_a (G_TYPE_FROM_INSTANCE (priv->gparam), expected_type);
     else
-        result = JS_TRUE;
+        result = true;
 
     if (!result && throw_error) {
         gjs_throw_custom(context, "TypeError",
