@@ -36,9 +36,9 @@
 #include "fundamental.h"
 #include "interface.h"
 #include "gerror.h"
-
-#include <gjs/compat.h>
-#include <gjs/jsapi-private.h>
+#include "gjs/jsapi-wrapper.h"
+#include "gjs/jsapi-private.h"
+#include "gjs/mem.h"
 
 #include <util/misc.h>
 
@@ -54,55 +54,49 @@ extern struct JSClass gjs_repo_class;
 
 GJS_DEFINE_PRIV_FROM_JS(Repo, gjs_repo_class)
 
-static JSObject * lookup_override_function(JSContext *, jsid);
+static JSObject *lookup_override_function(JSContext *, JS::HandleId);
 
-static JSBool
-get_version_for_ns (JSContext *context,
-                    JSObject  *repo_obj,
-                    jsid       ns_id,
-                    char     **version)
+static bool
+get_version_for_ns (JSContext       *context,
+                    JS::HandleObject repo_obj,
+                    JS::HandleId     ns_id,
+                    char           **version)
 {
-    jsid versions_name;
-    jsval versions_val;
-    JSObject *versions;
-    jsval version_val;
+    JS::RootedObject versions(context);
+    JS::RootedValue version_val(context);
+    JS::RootedId versions_name(context,
+        gjs_context_get_const_string(context, GJS_STRING_GI_VERSIONS));
 
-    versions_name = gjs_context_get_const_string(context, GJS_STRING_GI_VERSIONS);
-    if (!gjs_object_require_property(context, repo_obj, "GI repository object", versions_name, &versions_val) ||
-        !JSVAL_IS_OBJECT(versions_val)) {
-        gjs_throw(context, "No 'versions' property in GI repository object");
-        return JS_FALSE;
+    if (!gjs_object_require_property_value(context, repo_obj,
+                                           "GI repository object", versions_name,
+                                           &versions))
+        return false;
+
+    if (!gjs_object_require_property_value(context, versions, NULL, ns_id, version)) {
+        /* Property not actually required, so clear an exception */
+        JS_ClearPendingException(context);
+        *version = NULL;
     }
 
-    versions = JSVAL_TO_OBJECT(versions_val);
-
-    *version = NULL;
-    if (JS_GetPropertyById(context, versions, ns_id, &version_val) &&
-        JSVAL_IS_STRING(version_val)) {
-        gjs_string_to_utf8(context, version_val, version);
-    }
-
-    return JS_TRUE;
+    return true;
 }
 
-static JSBool
-resolve_namespace_object(JSContext  *context,
-                         JSObject   *repo_obj,
-                         jsid        ns_id,
-                         const char *ns_name)
+static bool
+resolve_namespace_object(JSContext       *context,
+                         JS::HandleObject repo_obj,
+                         JS::HandleId     ns_id,
+                         const char      *ns_name)
 {
     GIRepository *repo;
     GError *error;
     char *version;
     JSObject *override;
-    jsval result;
-    JSObject *gi_namespace = NULL;
-    JSBool ret = JS_FALSE;
+    JS::Value result;
 
-    JS_BeginRequest(context);
+    JSAutoRequest ar(context);
 
     if (!get_version_for_ns(context, repo_obj, ns_id, &version))
-        goto out;
+        return false;
 
     repo = g_irepository_get_default();
 
@@ -115,7 +109,7 @@ resolve_namespace_object(JSContext  *context,
 
         g_error_free(error);
         g_free(version);
-        goto out;
+        return false;
     }
 
     g_free(version);
@@ -124,13 +118,12 @@ resolve_namespace_object(JSContext  *context,
      * with the given namespace name, pointing to that namespace
      * in the repo.
      */
-    gi_namespace = gjs_create_ns(context, ns_name);
-    JS_AddObjectRoot(context, &gi_namespace);
+    JS::RootedObject gi_namespace(context, gjs_create_ns(context, ns_name));
 
     /* Define the property early, to avoid reentrancy issues if
        the override module looks for namespaces that import this */
     if (!JS_DefineProperty(context, repo_obj,
-                           ns_name, OBJECT_TO_JSVAL(gi_namespace),
+                           ns_name, JS::ObjectValue(*gi_namespace),
                            NULL, NULL,
                            GJS_MODULE_PROP_FLAGS))
         g_error("no memory to define ns property");
@@ -138,23 +131,18 @@ resolve_namespace_object(JSContext  *context,
     override = lookup_override_function(context, ns_id);
     if (override && !JS_CallFunctionValue (context,
                                            gi_namespace, /* thisp */
-                                           OBJECT_TO_JSVAL(override), /* callee */
+                                           JS::ObjectValue(*override), /* callee */
                                            0, /* argc */
                                            NULL, /* argv */
                                            &result))
-        goto out;
+        return false;
 
     gjs_debug(GJS_DEBUG_GNAMESPACE,
-              "Defined namespace '%s' %p in GIRepository %p", ns_name, gi_namespace, repo_obj);
+              "Defined namespace '%s' %p in GIRepository %p", ns_name,
+              gi_namespace.get(), repo_obj.get());
 
-    ret = JS_TRUE;
     gjs_schedule_gc_if_needed(context);
-
- out:
-    if (gi_namespace)
-        JS_RemoveObjectRoot(context, &gi_namespace);
-    JS_EndRequest(context);
-    return ret;
+    return true;
 }
 
 /*
@@ -170,7 +158,7 @@ resolve_namespace_object(JSContext  *context,
  * was not resolved; and non-null, referring to obj or one of its prototypes,
  * if id was resolved.
  */
-static JSBool
+static bool
 repo_new_resolve(JSContext *context,
                  JS::HandleObject obj,
                  JS::HandleId id,
@@ -179,10 +167,10 @@ repo_new_resolve(JSContext *context,
 {
     Repo *priv;
     char *name;
-    JSBool ret = JS_TRUE;
+    bool ret = true;
 
     if (!gjs_get_string_id(context, id, &name))
-        return JS_TRUE; /* not resolved, but no error */
+        return true; /* not resolved, but no error */
 
     /* let Object.prototype resolve these */
     if (strcmp(name, "valueOf") == 0 ||
@@ -191,13 +179,13 @@ repo_new_resolve(JSContext *context,
 
     priv = priv_from_js(context, obj);
     gjs_debug_jsprop(GJS_DEBUG_GREPO, "Resolve prop '%s' hook obj %p priv %p",
-                     name, (void *)obj, priv);
+                     name, obj.get(), priv);
 
     if (priv == NULL) /* we are the prototype, or have the wrong class */
         goto out;
 
     if (!resolve_namespace_object(context, obj, id, name)) {
-        ret = JS_FALSE;
+        ret = false;
     } else {
         objp.set(obj); /* store the object we defined the prop in */
     }
@@ -245,18 +233,17 @@ struct JSClass gjs_repo_class = {
 };
 
 JSPropertySpec gjs_repo_proto_props[] = {
-    { NULL }
+    JS_PS_END
 };
 
 JSFunctionSpec gjs_repo_proto_funcs[] = {
-    { NULL }
+    JS_FS_END
 };
 
 static JSObject*
 repo_new(JSContext *context)
 {
     Repo *priv;
-    JSObject *repo;
     JSObject *global;
     JSObject *versions;
     JSObject *private_ns;
@@ -298,7 +285,7 @@ repo_new(JSContext *context)
                   gjs_repo_class.name, prototype);
     }
 
-    repo = JS_NewObject(context, &gjs_repo_class, NULL, global);
+    JS::RootedObject repo(context, JS_NewObject(context, &gjs_repo_class, NULL, global));
     if (repo == NULL) {
         gjs_throw(context, "No memory to create repo object");
         return NULL;
@@ -312,13 +299,13 @@ repo_new(JSContext *context)
     JS_SetPrivate(repo, priv);
 
     gjs_debug_lifecycle(GJS_DEBUG_GREPO,
-                        "repo constructor, obj %p priv %p", repo, priv);
+                        "repo constructor, obj %p priv %p", repo.get(), priv);
 
     versions = JS_NewObject(context, NULL, NULL, global);
     versions_name = gjs_context_get_const_string(context, GJS_STRING_GI_VERSIONS);
     JS_DefinePropertyById(context, repo,
                           versions_name,
-                          OBJECT_TO_JSVAL(versions),
+                          JS::ObjectValue(*versions),
                           NULL, NULL,
                           JSPROP_PERMANENT);
 
@@ -326,48 +313,44 @@ repo_new(JSContext *context)
     private_ns_name = gjs_context_get_const_string(context, GJS_STRING_PRIVATE_NS_MARKER);
     JS_DefinePropertyById(context, repo,
                           private_ns_name,
-                          OBJECT_TO_JSVAL(private_ns),
+                          JS::ObjectValue(*private_ns),
                           NULL, NULL, JSPROP_PERMANENT);
 
     /* FIXME - hack to make namespaces load, since
      * gobject-introspection does not yet search a path properly.
      */
     {
-        jsval value;
+        JS::Value value;
         JS_GetProperty(context, repo, "GLib", &value);
     }
 
     return repo;
 }
 
-JSBool
-gjs_define_repo(JSContext  *context,
-                JSObject  **module_out,
-                const char *name)
+bool
+gjs_define_repo(JSContext              *cx,
+                JS::MutableHandleObject repo,
+                const char             *name)
 {
-    JSObject *repo;
-
-    repo = repo_new(context);
-    *module_out = repo;
-
-    return JS_TRUE;
+    repo.set(repo_new(cx));
+    return true;
 }
 
-static JSBool
+static bool
 gjs_define_constant(JSContext      *context,
                     JSObject       *in_object,
                     GIConstantInfo *info)
 {
-    jsval value;
+    JS::RootedValue value(context);
     GArgument garg = { 0, };
     GITypeInfo *type_info;
     const char *name;
-    JSBool ret = JS_FALSE;
+    bool ret = false;
 
     type_info = g_constant_info_get_type(info);
     g_constant_info_get_value(info, &garg);
 
-    if (!gjs_value_from_g_argument(context, &value, type_info, &garg, TRUE))
+    if (!gjs_value_from_g_argument(context, &value, type_info, &garg, true))
         goto out;
 
     name = g_base_info_get_name((GIBaseInfo*) info);
@@ -376,7 +359,7 @@ gjs_define_constant(JSContext      *context,
                           name, value,
                           NULL, NULL,
                           GJS_MODULE_PROP_FLAGS))
-        ret = JS_TRUE;
+        ret = true;
 
  out:
     g_constant_info_free_value (info, &garg);
@@ -432,7 +415,7 @@ _gjs_log_info_usage(GIBaseInfo *info)
 
             details = g_strdup_printf(".details = { .func = { .retval_transfer = GI_TRANSFER_%s, .n_args = %d, .args = %s } }",
                                       TRANSFER_STRING(retval_transfer), n_args, args->str);
-            g_string_free(args, TRUE);
+            g_string_free(args, true);
         } else {
             details = g_strdup_printf(".details = { .nothing = {} }");
         }
@@ -450,17 +433,17 @@ _gjs_log_info_usage(GIBaseInfo *info)
 }
 #endif /* GJS_VERBOSE_ENABLE_GI_USAGE */
 
-JSBool
-gjs_define_info(JSContext  *context,
-                JSObject   *in_object,
-                GIBaseInfo *info,
-                gboolean   *defined)
+bool
+gjs_define_info(JSContext       *context,
+                JS::HandleObject in_object,
+                GIBaseInfo      *info,
+                bool            *defined)
 {
 #if GJS_VERBOSE_ENABLE_GI_USAGE
     _gjs_log_info_usage(info);
 #endif
 
-    *defined = TRUE;
+    *defined = true;
 
     switch (g_base_info_get_type(info)) {
     case GI_INFO_TYPE_FUNCTION:
@@ -468,7 +451,7 @@ gjs_define_info(JSContext  *context,
             JSObject *f;
             f = gjs_define_function(context, in_object, 0, (GICallableInfo*) info);
             if (f == NULL)
-                return JS_FALSE;
+                return false;
         }
         break;
     case GI_INFO_TYPE_OBJECT:
@@ -479,19 +462,24 @@ gjs_define_info(JSContext  *context,
             if (g_type_is_a (gtype, G_TYPE_PARAM)) {
                 gjs_define_param_class(context, in_object);
             } else if (g_type_is_a (gtype, G_TYPE_OBJECT)) {
-                gjs_define_object_class(context, in_object, (GIObjectInfo*) info, gtype, NULL);
+                JS::RootedObject ignored(context);
+                gjs_define_object_class(context, in_object,
+                                        (GIObjectInfo *) info, gtype, &ignored);
             } else if (G_TYPE_IS_INSTANTIATABLE(gtype)) {
-                if (!gjs_define_fundamental_class(context, in_object, (GIObjectInfo*)info, NULL, NULL)) {
+                JS::RootedObject ignored1(context), ignored2(context);
+                if (!gjs_define_fundamental_class(context, in_object,
+                                                  (GIObjectInfo*)info,
+                                                  &ignored1, &ignored2)) {
                     gjs_throw (context,
                                "Unsupported fundamental class creation for type %s",
                                g_type_name(gtype));
-                    return JS_FALSE;
+                    return false;
                 }
             } else {
                 gjs_throw (context,
                            "Unsupported type %s, deriving from fundamental %s",
                            g_type_name(gtype), g_type_name(g_type_fundamental(gtype)));
-                return JS_FALSE;
+                return false;
             }
         }
         break;
@@ -501,7 +489,7 @@ gjs_define_info(JSContext  *context,
            as static methods
         */
         if (g_struct_info_is_gtype_struct((GIStructInfo*) info)) {
-            *defined = FALSE;
+            *defined = false;
             break;
         }
         /* Fall through */
@@ -511,7 +499,7 @@ gjs_define_info(JSContext  *context,
         break;
     case GI_INFO_TYPE_UNION:
         if (!gjs_define_union_class(context, in_object, (GIUnionInfo*) info))
-            return JS_FALSE;
+            return false;
         break;
     case GI_INFO_TYPE_ENUM:
         if (g_enum_info_get_error_domain((GIEnumInfo*) info)) {
@@ -523,35 +511,49 @@ gjs_define_info(JSContext  *context,
 
     case GI_INFO_TYPE_FLAGS:
         if (!gjs_define_enumeration(context, in_object, (GIEnumInfo*) info))
-            return JS_FALSE;
+            return false;
         break;
     case GI_INFO_TYPE_CONSTANT:
         if (!gjs_define_constant(context, in_object, (GIConstantInfo*) info))
-            return JS_FALSE;
+            return false;
         break;
     case GI_INFO_TYPE_INTERFACE:
-        gjs_define_interface_class(context, in_object, (GIInterfaceInfo *) info,
-                                   g_registered_type_info_get_g_type((GIRegisteredTypeInfo *) info),
-                                   NULL);
+        {
+            JS::RootedObject ignored(context);
+            gjs_define_interface_class(context, in_object,
+                                       (GIInterfaceInfo *) info,
+                                       g_registered_type_info_get_g_type((GIRegisteredTypeInfo *) info),
+                                       &ignored);
+        }
         break;
+    case GI_INFO_TYPE_INVALID:
+    case GI_INFO_TYPE_INVALID_0:
+    case GI_INFO_TYPE_CALLBACK:
+    case GI_INFO_TYPE_VALUE:
+    case GI_INFO_TYPE_SIGNAL:
+    case GI_INFO_TYPE_VFUNC:
+    case GI_INFO_TYPE_PROPERTY:
+    case GI_INFO_TYPE_FIELD:
+    case GI_INFO_TYPE_ARG:
+    case GI_INFO_TYPE_TYPE:
+    case GI_INFO_TYPE_UNRESOLVED:
     default:
         gjs_throw(context, "API of type %s not implemented, cannot define %s.%s",
                   gjs_info_type_name(g_base_info_get_type(info)),
                   g_base_info_get_namespace(info),
                   g_base_info_get_name(info));
-        return JS_FALSE;
+        return false;
     }
 
-    return JS_TRUE;
+    return true;
 }
 
 /* Get the "unknown namespace", which should be used for unnamespaced types */
 JSObject*
 gjs_lookup_private_namespace(JSContext *context)
 {
-    jsid ns_name;
-
-    ns_name = gjs_context_get_const_string(context, GJS_STRING_PRIVATE_NS_MARKER);
+    JS::RootedId ns_name(context,
+        gjs_context_get_const_string(context, GJS_STRING_PRIVATE_NS_MARKER));
     return gjs_lookup_namespace_object_by_name(context, ns_name);
 }
 
@@ -561,7 +563,6 @@ gjs_lookup_namespace_object(JSContext  *context,
                             GIBaseInfo *info)
 {
     const char *ns;
-    jsid ns_name;
 
     ns = g_base_info_get_namespace(info);
     if (ns == NULL) {
@@ -572,99 +573,75 @@ gjs_lookup_namespace_object(JSContext  *context,
         return NULL;
     }
 
-    ns_name = gjs_intern_string_to_id(context, ns);
+    JS::RootedId ns_name(context, gjs_intern_string_to_id(context, ns));
     return gjs_lookup_namespace_object_by_name(context, ns_name);
 }
 
 static JSObject*
-lookup_override_function(JSContext  *context,
-                         jsid        ns_name)
+lookup_override_function(JSContext   *cx,
+                         JS::HandleId ns_name)
 {
-    jsval importer;
-    jsval overridespkg;
-    jsval module;
-    jsval function;
-    jsid overrides_name, object_init_name;
+    JSAutoRequest ar(cx);
 
-    JS_BeginRequest(context);
+    JS::RootedValue importer(cx, gjs_get_global_slot(cx, GJS_GLOBAL_SLOT_IMPORTS));
+    g_assert(importer.isObject());
 
-    importer = gjs_get_global_slot(context, GJS_GLOBAL_SLOT_IMPORTS);
-    g_assert(JSVAL_IS_OBJECT(importer));
+    JS::RootedValue function(cx);
+    JS::RootedId overrides_name(cx,
+        gjs_context_get_const_string(cx, GJS_STRING_GI_OVERRIDES));
+    JS::RootedId object_init_name(cx,
+        gjs_context_get_const_string(cx, GJS_STRING_GOBJECT_INIT));
+    JS::RootedObject overridespkg(cx), module(cx);
+    JS::RootedObject importer_obj(cx, &importer.toObject());
 
-    overridespkg = JSVAL_VOID;
-    overrides_name = gjs_context_get_const_string(context, GJS_STRING_GI_OVERRIDES);
-    if (!gjs_object_require_property(context, JSVAL_TO_OBJECT(importer), "importer",
-                                     overrides_name, &overridespkg) ||
-        !JSVAL_IS_OBJECT(overridespkg))
+    if (!gjs_object_require_property_value(cx, importer_obj, "importer",
+                                           overrides_name, &overridespkg))
         goto fail;
 
-    module = JSVAL_VOID;
-    if (!gjs_object_require_property(context, JSVAL_TO_OBJECT(overridespkg), "GI repository object", ns_name, &module)
-        || !JSVAL_IS_OBJECT(module))
+    if (!gjs_object_require_property_value(cx, overridespkg, "GI repository object",
+                                           ns_name, &module))
         goto fail;
 
-    object_init_name = gjs_context_get_const_string(context, GJS_STRING_GOBJECT_INIT);
-    if (!gjs_object_require_property(context, JSVAL_TO_OBJECT(module), "override module",
+    if (!gjs_object_require_property(cx, module, "override module",
                                      object_init_name, &function) ||
-        !JSVAL_IS_OBJECT(function))
+        !function.isObjectOrNull())
         goto fail;
 
-    JS_EndRequest(context);
-    return JSVAL_TO_OBJECT(function);
+    return function.toObjectOrNull();
 
  fail:
-    JS_ClearPendingException(context);
-    JS_EndRequest(context);
+    /* COMPAT: Use JS::AutoSaveExceptionState and move rooting to be late */
+    JS_ClearPendingException(cx);
     return NULL;
 }
 
 JSObject*
 gjs_lookup_namespace_object_by_name(JSContext      *context,
-                                    jsid            ns_name)
+                                    JS::HandleId    ns_name)
 {
-    JSObject *repo_obj;
-    jsval importer;
-    jsval girepository;
-    jsval ns_obj;
-    jsid gi_name;
+    JSAutoRequest ar(context);
 
-    JS_BeginRequest(context);
+    JS::RootedValue importer(context,
+        gjs_get_global_slot(context, GJS_GLOBAL_SLOT_IMPORTS));
+    g_assert(importer.isObject());
 
-    importer = gjs_get_global_slot(context, GJS_GLOBAL_SLOT_IMPORTS);
-    g_assert(JSVAL_IS_OBJECT(importer));
+    JS::RootedId gi_name(context,
+        gjs_context_get_const_string(context, GJS_STRING_GI_MODULE));
+    JS::RootedObject repo(context), importer_obj(context, &importer.toObject());
 
-    girepository = JSVAL_VOID;
-    gi_name = gjs_context_get_const_string(context, GJS_STRING_GI_MODULE);
-    if (!gjs_object_require_property(context, JSVAL_TO_OBJECT(importer), "importer",
-                                     gi_name, &girepository) ||
-        !JSVAL_IS_OBJECT(girepository)) {
+    if (!gjs_object_require_property_value(context, importer_obj, "importer",
+                                           gi_name, &repo)) {
         gjs_log_exception(context);
         gjs_throw(context, "No gi property in importer");
-        goto fail;
+        return NULL;
     }
 
-    repo_obj = JSVAL_TO_OBJECT(girepository);
+    JS::RootedObject retval(context);
+    if (!gjs_object_require_property_value(context, repo, "GI repository object",
+                                           ns_name, &retval))
+        return NULL;
 
-    if (!gjs_object_require_property(context, repo_obj, "GI repository object", ns_name, &ns_obj)) {
-        goto fail;
-    }
-
-    if (!JSVAL_IS_OBJECT(ns_obj)) {
-        char *name;
-
-        gjs_get_string_id(context, ns_name, &name);
-        gjs_throw(context, "Namespace '%s' is not an object?", name);
-
-        g_free(name);
-        goto fail;
-    }
-
-    JS_EndRequest(context);
-    return JSVAL_TO_OBJECT(ns_obj);
-
- fail:
-    JS_EndRequest(context);
-    return NULL;
+    return retval;
 }
 
 const char*
@@ -712,9 +689,9 @@ gjs_info_type_name(GIInfoType type)
     case GI_INFO_TYPE_INVALID_0:
         g_assert_not_reached();
         break;
+    default:
+      return "???";
     }
-
-    return "???";
 }
 
 char*
@@ -722,25 +699,25 @@ gjs_camel_from_hyphen(const char *hyphen_name)
 {
     GString *s;
     const char *p;
-    gboolean next_upper;
+    bool next_upper;
 
     s = g_string_sized_new(strlen(hyphen_name) + 1);
 
-    next_upper = FALSE;
+    next_upper = false;
     for (p = hyphen_name; *p; ++p) {
         if (*p == '-' || *p == '_') {
-            next_upper = TRUE;
+            next_upper = true;
         } else {
             if (next_upper) {
                 g_string_append_c(s, g_ascii_toupper(*p));
-                next_upper = FALSE;
+                next_upper = false;
             } else {
                 g_string_append_c(s, *p);
             }
         }
     }
 
-    return g_string_free(s, FALSE);
+    return g_string_free(s, false);
 }
 
 char*
@@ -761,7 +738,7 @@ gjs_hyphen_from_camel(const char *camel_name)
         }
     }
 
-    return g_string_free(s, FALSE);
+    return g_string_free(s, false);
 }
 
 JSObject *
@@ -769,9 +746,8 @@ gjs_lookup_generic_constructor(JSContext  *context,
                                GIBaseInfo *info)
 {
     JSObject *in_object;
-    JSObject *constructor;
     const char *constructor_name;
-    jsval value;
+    JS::Value value;
 
     in_object = gjs_lookup_namespace_object(context, (GIBaseInfo*) info);
     constructor_name = g_base_info_get_name((GIBaseInfo*) info);
@@ -782,32 +758,28 @@ gjs_lookup_generic_constructor(JSContext  *context,
     if (!JS_GetProperty(context, in_object, constructor_name, &value))
         return NULL;
 
-    if (G_UNLIKELY (!JSVAL_IS_OBJECT(value) || JSVAL_IS_NULL(value)))
+    if (G_UNLIKELY (!value.isObject()))
         return NULL;
 
-    constructor = JSVAL_TO_OBJECT(value);
-    g_assert(constructor != NULL);
-
-    return constructor;
+    return &value.toObject();
 }
 
 JSObject *
 gjs_lookup_generic_prototype(JSContext  *context,
                              GIBaseInfo *info)
 {
-    JSObject *constructor;
-    jsval value;
-
-    constructor = gjs_lookup_generic_constructor(context, info);
+    JS::RootedObject constructor(context,
+                                 gjs_lookup_generic_constructor(context, info));
     if (G_UNLIKELY (constructor == NULL))
         return NULL;
 
+    JS::RootedValue value(context);
     if (!gjs_object_get_property_const(context, constructor,
                                        GJS_STRING_PROTOTYPE, &value))
         return NULL;
 
-    if (G_UNLIKELY (!JSVAL_IS_OBJECT(value)))
+    if (G_UNLIKELY (!value.isObjectOrNull()))
         return NULL;
 
-    return JSVAL_TO_OBJECT(value);
+    return value.toObjectOrNull();
 }
