@@ -31,8 +31,8 @@
 #include "union.h"
 #include "arg.h"
 #include "object.h"
-#include <gjs/gjs-module.h>
-#include <gjs/compat.h>
+#include "gjs/jsapi-wrapper.h"
+#include "gjs/mem.h"
 #include "repo.h"
 #include "proxyutils.h"
 #include "function.h"
@@ -62,7 +62,7 @@ GJS_DEFINE_PRIV_FROM_JS(Union, gjs_union_class)
  * was not resolved; and non-null, referring to obj or one of its prototypes,
  * if id was resolved.
  */
-static JSBool
+static bool
 union_new_resolve(JSContext *context,
                   JS::HandleObject obj,
                   JS::HandleId id,
@@ -71,17 +71,17 @@ union_new_resolve(JSContext *context,
 {
     Union *priv;
     char *name;
-    JSBool ret = JS_TRUE;
+    bool ret = true;
 
     if (!gjs_get_string_id(context, id, &name))
-        return JS_TRUE; /* not resolved, but no error */
+        return true; /* not resolved, but no error */
 
     priv = priv_from_js(context, obj);
     gjs_debug_jsprop(GJS_DEBUG_GBOXED, "Resolve prop '%s' hook obj %p priv %p",
-                     name, (void *)obj, priv);
+                     name, obj.get(), priv);
 
     if (priv == NULL) {
-        ret = JS_FALSE; /* wrong class */
+        ret = false; /* wrong class */
         goto out;
     }
 
@@ -93,7 +93,6 @@ union_new_resolve(JSContext *context,
                                                name);
 
         if (method_info != NULL) {
-            JSObject *union_proto;
             const char *method_name;
 
 #if GJS_VERBOSE_ENABLE_GI_USAGE
@@ -108,17 +107,16 @@ union_new_resolve(JSContext *context,
                           g_base_info_get_namespace( (GIBaseInfo*) priv->info),
                           g_base_info_get_name( (GIBaseInfo*) priv->info));
 
-                union_proto = obj;
-
-                if (gjs_define_function(context, union_proto,
+                /* obj is union proto */
+                if (gjs_define_function(context, obj,
                                         g_registered_type_info_get_g_type(priv->info),
                                         method_info) == NULL) {
                     g_base_info_unref( (GIBaseInfo*) method_info);
-                    ret = JS_FALSE;
+                    ret = false;
                     goto out;
                 }
 
-                objp.set(union_proto); /* we defined the prop in object_proto */
+                objp.set(obj); /* we defined the prop in object_proto */
             }
 
             g_base_info_unref( (GIBaseInfo*) method_info);
@@ -139,9 +137,9 @@ union_new_resolve(JSContext *context,
 }
 
 static void*
-union_new(JSContext   *context,
-          JSObject    *obj, /* "this" for constructor */
-          GIUnionInfo *info)
+union_new(JSContext       *context,
+          JS::HandleObject obj, /* "this" for constructor */
+          GIUnionInfo     *info)
 {
     int n_methods;
     int i;
@@ -160,9 +158,9 @@ union_new(JSContext   *context,
         if ((flags & GI_FUNCTION_IS_CONSTRUCTOR) != 0 &&
             g_callable_info_get_n_args((GICallableInfo*) func_info) == 0) {
 
-            jsval rval;
+            JS::Value rval;
 
-            rval = JSVAL_NULL;
+            rval = JS::NullValue();
             gjs_invoke_c_function_uncached(context, func_info, obj,
                                            0, NULL, &rval);
 
@@ -172,10 +170,12 @@ union_new(JSContext   *context,
              * creates a JSObject wrapper for the union that we immediately
              * discard.
              */
-            if (JSVAL_IS_NULL(rval))
+            if (rval.isNull()) {
                 return NULL;
-            else
-                return gjs_c_union_from_union(context, JSVAL_TO_OBJECT(rval));
+            } else {
+                JS::RootedObject rval_obj(context, &rval.toObject());
+                return gjs_c_union_from_union(context, rval_obj);
+            }
         }
 
         g_base_info_unref((GIBaseInfo*) func_info);
@@ -192,7 +192,7 @@ GJS_NATIVE_CONSTRUCTOR_DECLARE(union)
     GJS_NATIVE_CONSTRUCTOR_VARIABLES(union)
     Union *priv;
     Union *proto_priv;
-    JSObject *proto;
+    JS::RootedObject proto(context);
     void *gboxed;
 
     GJS_NATIVE_CONSTRUCTOR_PRELUDE(union);
@@ -206,10 +206,11 @@ GJS_NATIVE_CONSTRUCTOR_DECLARE(union)
 
     gjs_debug_lifecycle(GJS_DEBUG_GBOXED,
                         "union constructor, obj %p priv %p",
-                        object, priv);
+                        object.get(), priv);
 
-    JS_GetPrototype(context, object, &proto);
-    gjs_debug_lifecycle(GJS_DEBUG_GBOXED, "union instance __proto__ is %p", proto);
+    JS_GetPrototype(context, object, proto.address());
+    gjs_debug_lifecycle(GJS_DEBUG_GBOXED, "union instance __proto__ is %p",
+                        proto.get());
 
     /* If we're the prototype, then post-construct we'll fill in priv->info.
      * If we are not the prototype, though, then we'll get ->info from the
@@ -219,7 +220,7 @@ GJS_NATIVE_CONSTRUCTOR_DECLARE(union)
     if (proto_priv == NULL) {
         gjs_debug(GJS_DEBUG_GBOXED,
                   "Bad prototype set on union? Must match JSClass of object. JS error should have been reported.");
-        return JS_FALSE;
+        return false;
     }
 
     priv->info = proto_priv->info;
@@ -227,17 +228,17 @@ GJS_NATIVE_CONSTRUCTOR_DECLARE(union)
     priv->gtype = proto_priv->gtype;
 
     /* union_new happens to be implemented by calling
-     * gjs_invoke_c_function(), which returns a jsval.
-     * The returned "gboxed" here is owned by that jsval,
+     * gjs_invoke_c_function(), which returns a JS::Value.
+     * The returned "gboxed" here is owned by that JS::Value,
      * not by us.
      */
     gboxed = union_new(context, object, priv->info);
 
     if (gboxed == NULL) {
-        return JS_FALSE;
+        return false;
     }
 
-    /* Because "gboxed" is owned by a jsval and will
+    /* Because "gboxed" is owned by a JS::Value and will
      * be garbage collected, we make a copy here to be
      * owned by us.
      */
@@ -249,7 +250,7 @@ GJS_NATIVE_CONSTRUCTOR_DECLARE(union)
 
     GJS_NATIVE_CONSTRUCTOR_FINISH(union);
 
-    return JS_TRUE;
+    return true;
 }
 
 static void
@@ -282,26 +283,12 @@ union_finalize(JSFreeOp *fop,
 static JSBool
 to_string_func(JSContext *context,
                unsigned   argc,
-               jsval     *vp)
+               JS::Value *vp)
 {
-    JS::CallReceiver rec = JS::CallReceiverFromVp(vp);
-    JSObject *obj = JSVAL_TO_OBJECT(rec.thisv());
-
-    Union *priv;
-    JSBool ret = JS_FALSE;
-    jsval retval;
-
-    if (!priv_from_js_with_typecheck(context, obj, &priv))
-        goto out;
-    
-    if (!_gjs_proxy_to_string_func(context, obj, "union", (GIBaseInfo*)priv->info,
-                                   priv->gtype, priv->gboxed, &retval))
-        goto out;
-
-    ret = JS_TRUE;
-    rec.rval().set(retval);
- out:
-    return ret;
+    GJS_GET_PRIV(context, argc, vp, rec, obj, Union, priv);
+    return _gjs_proxy_to_string_func(context, obj, "union",
+                                     (GIBaseInfo*)priv->info, priv->gtype,
+                                     priv->gboxed, rec.rval());
 }
 
 /* The bizarre thing about this vtable is that it applies to both
@@ -320,31 +307,28 @@ struct JSClass gjs_union_class = {
     (JSResolveOp) union_new_resolve, /* needs cast since it's the new resolve signature */
     JS_ConvertStub,
     union_finalize,
-    NULL,
-    NULL,
-    NULL, NULL, NULL
+    JSCLASS_NO_OPTIONAL_MEMBERS
 };
 
 JSPropertySpec gjs_union_proto_props[] = {
-    { NULL }
+    JS_PS_END
 };
 
 JSFunctionSpec gjs_union_proto_funcs[] = {
-    { "toString", JSOP_WRAPPER((JSNative)to_string_func), 0, 0 },
-    { NULL }
+    JS_FS("toString", to_string_func, 0, 0),
+    JS_FS_END
 };
 
-JSBool
-gjs_define_union_class(JSContext    *context,
-                       JSObject     *in_object,
-                       GIUnionInfo  *info)
+bool
+gjs_define_union_class(JSContext       *context,
+                       JS::HandleObject in_object,
+                       GIUnionInfo     *info)
 {
     const char *constructor_name;
-    JSObject *prototype;
-    jsval value;
+    JS::Value value;
     Union *priv;
     GType gtype;
-    JSObject *constructor;
+    JS::RootedObject prototype(context), constructor(context);
 
     /* For certain unions, we may be able to relax this in the future by
      * directly allocating union memory, as we do for structures in boxed.c
@@ -352,7 +336,7 @@ gjs_define_union_class(JSContext    *context,
     gtype = g_registered_type_info_get_g_type( (GIRegisteredTypeInfo*) info);
     if (gtype == G_TYPE_NONE) {
         gjs_throw(context, "Unions must currently be registered as boxed types");
-        return JS_FALSE;
+        return false;
     }
 
     /* See the comment in gjs_define_object_class() for an
@@ -363,7 +347,7 @@ gjs_define_union_class(JSContext    *context,
     constructor_name = g_base_info_get_name( (GIBaseInfo*) info);
 
     if (!gjs_init_class_dynamic(context, in_object,
-                                NULL,
+                                JS::NullPtr(),
                                 g_base_info_get_namespace( (GIBaseInfo*) info),
                                 constructor_name,
                                 &gjs_union_class,
@@ -389,13 +373,14 @@ gjs_define_union_class(JSContext    *context,
     JS_SetPrivate(prototype, priv);
 
     gjs_debug(GJS_DEBUG_GBOXED, "Defined class %s prototype is %p class %p in object %p",
-              constructor_name, prototype, JS_GetClass(prototype), in_object);
+              constructor_name, prototype.get(), JS_GetClass(prototype),
+              in_object.get());
 
-    value = OBJECT_TO_JSVAL(gjs_gtype_create_gtype_wrapper(context, gtype));
+    value = JS::ObjectOrNullValue(gjs_gtype_create_gtype_wrapper(context, gtype));
     JS_DefineProperty(context, constructor, "$gtype", value,
                       NULL, NULL, JSPROP_PERMANENT);
 
-    return JS_TRUE;
+    return true;
 }
 
 JSObject*
@@ -442,8 +427,8 @@ gjs_union_from_c_union(JSContext    *context,
 }
 
 void*
-gjs_c_union_from_union(JSContext    *context,
-                       JSObject     *obj)
+gjs_c_union_from_union(JSContext       *context,
+                       JS::HandleObject obj)
 {
     Union *priv;
 
@@ -455,18 +440,18 @@ gjs_c_union_from_union(JSContext    *context,
     return priv->gboxed;
 }
 
-JSBool
-gjs_typecheck_union(JSContext     *context,
-                    JSObject      *object,
-                    GIStructInfo  *expected_info,
-                    GType          expected_type,
-                    JSBool         throw_error)
+bool
+gjs_typecheck_union(JSContext       *context,
+                    JS::HandleObject object,
+                    GIStructInfo    *expected_info,
+                    GType            expected_type,
+                    bool             throw_error)
 {
     Union *priv;
-    JSBool result;
+    bool result;
 
     if (!do_base_typecheck(context, object, throw_error))
-        return JS_FALSE;
+        return false;
 
     priv = priv_from_js(context, object);
 
@@ -478,7 +463,7 @@ gjs_typecheck_union(JSContext     *context,
                              g_base_info_get_name( (GIBaseInfo*) priv->info));
         }
 
-        return JS_FALSE;
+        return false;
     }
 
     if (expected_type != G_TYPE_NONE)
@@ -486,7 +471,7 @@ gjs_typecheck_union(JSContext     *context,
     else if (expected_info != NULL)
         result = g_base_info_equal((GIBaseInfo*) priv->info, (GIBaseInfo*) expected_info);
     else
-        result = JS_TRUE;
+        result = true;
 
     if (!result && throw_error) {
         if (expected_info != NULL) {
