@@ -25,9 +25,9 @@
 #include <string.h>
 #include <glib.h>
 #include "byteArray.h"
-#include "../gi/boxed.h"
-#include <gjs/gjs-module.h>
-#include <gjs/compat.h>
+#include "gi/boxed.h"
+#include "jsapi-wrapper.h"
+#include "jsapi-util-args.h"
 #include <girepository.h>
 #include <util/log.h>
 
@@ -39,14 +39,14 @@ typedef struct {
 extern struct JSClass gjs_byte_array_class;
 GJS_DEFINE_PRIV_FROM_JS(ByteArrayInstance, gjs_byte_array_class)
 
-static JSBool byte_array_get_prop      (JSContext    *context,
+static bool   byte_array_get_prop      (JSContext    *context,
                                         JS::HandleObject obj,
                                         JS::HandleId id,
                                         JS::MutableHandleValue value_p);
-static JSBool byte_array_set_prop      (JSContext    *context,
+static bool   byte_array_set_prop      (JSContext    *context,
                                         JS::HandleObject obj,
                                         JS::HandleId id,
-                                        JSBool        strict,
+                                        bool                   strict,
                                         JS::MutableHandleValue value_p);
 GJS_NATIVE_CONSTRUCTOR_DECLARE(byte_array);
 static void   byte_array_finalize      (JSFreeOp     *fop,
@@ -65,30 +65,24 @@ struct JSClass gjs_byte_array_class = {
     JS_ResolveStub,
     JS_ConvertStub,
     byte_array_finalize,
-    NULL,
-    NULL,
-    NULL, NULL, NULL
+    JSCLASS_NO_OPTIONAL_MEMBERS
 };
 
-JSBool
-gjs_typecheck_bytearray(JSContext     *context,
-                        JSObject      *object,
-                        JSBool         throw_error)
+bool
+gjs_typecheck_bytearray(JSContext       *context,
+                        JS::HandleObject object,
+                        bool             throw_error)
 {
     return do_base_typecheck(context, object, throw_error);
 }
 
-static JSBool
-gjs_value_from_gsize(JSContext         *context,
-                     gsize              v,
-                     JS::MutableHandleValue value_p)
+static JS::Value
+gjs_value_from_gsize(gsize v)
 {
-    if (v > (gsize) JSVAL_INT_MAX) {
-        value_p.set(INT_TO_JSVAL(v));
-        return JS_TRUE;
-    } else {
-        return JS_NewNumberValue(context, v, value_p.address());
+    if (v <= (gsize) JSVAL_INT_MAX) {
+        return JS::Int32Value(v);
     }
+    return JS::NumberValue(v);
 }
 
 static void
@@ -113,62 +107,59 @@ byte_array_ensure_gbytes (ByteArrayInstance  *priv)
     }
 }
 
-static JSBool
+static bool
 gjs_value_to_gsize(JSContext         *context,
-                   jsval              value,
+                   JS::HandleValue    value,
                    gsize             *v_p)
 {
     guint32 val32;
 
-    /* Just JS_ValueToECMAUint32() would work. However,
-     * we special case ints for two reasons:
-     *  - JS_ValueToECMAUint32() always goes via a double which is slow
-     *  - nicer error message on negative indices
+    /* Just JS::ToUint32() would work. However, we special case ints for a nicer
+     * error message on negative indices.
      */
-    if (JSVAL_IS_INT(value)) {
-        int i = JSVAL_TO_INT(value);
+    if (value.isInt32()) {
+        int i = value.toInt32();
         if (i < 0) {
             gjs_throw(context, "Negative length or index %d is not allowed for ByteArray",
                       i);
-            return JS_FALSE;
+            return false;
         }
         *v_p = i;
-        return JS_TRUE;
+        return true;
     } else {
-        JSBool ret;
+        bool ret;
         /* This is pretty liberal (it converts about anything to
          * a number) but it's what we use elsewhere in gjs too.
          */
 
-        ret = JS_ValueToECMAUint32(context, value,
-                                   &val32);
+        ret = JS::ToUint32(context, value, &val32);
         *v_p = val32;
         return ret;
     }
 }
 
-static JSBool
+static bool
 gjs_value_to_byte(JSContext         *context,
-                  jsval              value,
+                  JS::HandleValue    value,
                   guint8            *v_p)
 {
     gsize v;
 
     if (!gjs_value_to_gsize(context, value, &v))
-        return JS_FALSE;
+        return false;
 
     if (v >= 256) {
         gjs_throw(context,
                   "Value %" G_GSIZE_FORMAT " is not a valid byte; must be in range [0,255]",
                   v);
-        return JS_FALSE;
+        return false;
     }
 
     *v_p = v;
-    return JS_TRUE;
+    return true;
 }
 
-static JSBool
+static bool
 byte_array_get_index(JSContext         *context,
                      JS::HandleObject obj,
                      ByteArrayInstance *priv,
@@ -185,39 +176,39 @@ byte_array_get_index(JSContext         *context,
                   "Index %" G_GSIZE_FORMAT " is out of range for ByteArray length %lu",
                   idx,
                   (unsigned long)len);
-        return JS_FALSE;
+        return false;
     }
 
-    value_p.set(INT_TO_JSVAL(data[idx]));
+    value_p.setInt32(data[idx]);
 
-    return JS_TRUE;
+    return true;
 }
 
 /* a hook on getting a property; set value_p to override property's value.
- * Return value is JS_FALSE on OOM/exception.
+ * Return value is false on OOM/exception.
  */
-static JSBool
+static bool
 byte_array_get_prop(JSContext *context,
                     JS::HandleObject obj,
                     JS::HandleId id,
                     JS::MutableHandleValue value_p)
 {
     ByteArrayInstance *priv;
-    jsval id_value;
 
     priv = priv_from_js(context, obj);
 
     if (priv == NULL)
-        return JS_TRUE; /* prototype, not an instance. */
+        return true; /* prototype, not an instance. */
 
-    if (!JS_IdToValue(context, id, &id_value))
-        return JS_FALSE;
+    JS::RootedValue id_value(context);
+    if (!JS_IdToValue(context, id, id_value.address()))
+        return false;
 
     /* First handle array indexing */
-    if (JSVAL_IS_NUMBER(id_value)) {
+    if (id_value.isNumber()) {
         gsize idx;
         if (!gjs_value_to_gsize(context, id_value, &idx))
-            return JS_FALSE;
+            return false;
         return byte_array_get_index(context, obj, priv, idx, value_p);
     }
 
@@ -225,57 +216,54 @@ byte_array_get_prop(JSContext *context,
      * allow string versions of ints for the index, we don't bother.
      */
 
-    return JS_TRUE;
+    return true;
 }
 
 static JSBool
 byte_array_length_getter(JSContext *context,
-                         JS::HandleObject obj,
-                         JS::HandleId id,
-                         JS::MutableHandleValue value_p)
+                         unsigned   argc,
+                         JS::Value *vp)
 {
-    ByteArrayInstance *priv;
+    GJS_GET_PRIV(context, argc, vp, args, to, ByteArrayInstance, priv);
     gsize len = 0;
 
-    priv = priv_from_js(context, obj);
-
     if (priv == NULL)
-        return JS_TRUE; /* prototype, not an instance. */
+        return true; /* prototype, not an instance. */
 
     if (priv->array != NULL)
         len = priv->array->len;
     else if (priv->bytes != NULL)
         len = g_bytes_get_size (priv->bytes);
-    return gjs_value_from_gsize(context, len, value_p);
+    args.rval().set(gjs_value_from_gsize(len));
+    return true;
 }
 
 static JSBool
 byte_array_length_setter(JSContext *context,
-                         JS::HandleObject obj,
-                         JS::HandleId id,
-                         JSBool strict,
-                         JS::MutableHandleValue value_p)
+                         unsigned   argc,
+                         JS::Value *vp)
 {
-    ByteArrayInstance *priv;
+    GJS_GET_PRIV(context, argc, vp, args, to, ByteArrayInstance, priv);
     gsize len = 0;
 
-    priv = priv_from_js(context, obj);
-
     if (priv == NULL)
-        return JS_TRUE; /* prototype, not instance */
+        return true; /* prototype, not instance */
 
     byte_array_ensure_array(priv);
 
-    if (!gjs_value_to_gsize(context, value_p, &len)) {
+    // COMPAT: Indexing JS::CallArgs should provide a handle in mozjs31
+    JS::RootedValue arg(context, args[0]);
+    if (!gjs_value_to_gsize(context, arg, &len)) {
         gjs_throw(context,
                   "Can't set ByteArray length to non-integer");
-        return JS_FALSE;
+        return false;
     }
     g_byte_array_set_size(priv->array, len);
-    return JS_TRUE;
+    args.rval().setUndefined();
+    return true;
 }
 
-static JSBool
+static bool
 byte_array_set_index(JSContext         *context,
                      JS::HandleObject obj,
                      ByteArrayInstance *priv,
@@ -285,7 +273,7 @@ byte_array_set_index(JSContext         *context,
     guint8 v;
 
     if (!gjs_value_to_byte(context, value_p, &v)) {
-        return JS_FALSE;
+        return false;
     }
 
     byte_array_ensure_array(priv);
@@ -299,44 +287,44 @@ byte_array_set_index(JSContext         *context,
     g_array_index(priv->array, guint8, idx) = v;
 
     /* Stop JS from storing a copy of the value */
-    value_p.set(JSVAL_VOID);
+    value_p.setUndefined();
 
-    return JS_TRUE;
+    return true;
 }
 
 /* a hook on setting a property; set value_p to override property value to
- * be set. Return value is JS_FALSE on OOM/exception.
+ * be set. Return value is false on OOM/exception.
  */
-static JSBool
+static bool
 byte_array_set_prop(JSContext *context,
                     JS::HandleObject obj,
                     JS::HandleId id,
-                    JSBool strict,
+                    bool strict,
                     JS::MutableHandleValue value_p)
 {
     ByteArrayInstance *priv;
-    jsval id_value;
 
     priv = priv_from_js(context, obj);
 
     if (priv == NULL)
-        return JS_TRUE; /* prototype, not an instance. */
+        return true; /* prototype, not an instance. */
 
-    if (!JS_IdToValue(context, id, &id_value))
-        return JS_FALSE;
+    JS::RootedValue id_value(context);
+    if (!JS_IdToValue(context, id, id_value.address()))
+        return false;
 
     /* First handle array indexing */
-    if (JSVAL_IS_NUMBER(id_value)) {
+    if (id_value.isNumber()) {
         gsize idx;
         if (!gjs_value_to_gsize(context, id_value, &idx))
-            return JS_FALSE;
+            return false;
 
         return byte_array_set_index(context, obj, priv, idx, value_p);
     }
 
     /* We don't special-case anything else for now */
 
-    return JS_TRUE;
+    return true;
 }
 
 static GByteArray *
@@ -348,8 +336,8 @@ gjs_g_byte_array_new(int preallocated_length)
      * We nul-terminate too for ease of toString() and for security
      * paranoia.
      */
-    array =  (GByteArray*) g_array_sized_new (TRUE, /* nul-terminated */
-                                              TRUE, /* clear to zero */
+    array =  (GByteArray*) g_array_sized_new (true, /* nul-terminated */
+                                              true, /* clear to zero */
                                               1, /* element size */
                                               preallocated_length);
    if (preallocated_length > 0) {
@@ -372,10 +360,12 @@ GJS_NATIVE_CONSTRUCTOR_DECLARE(byte_array)
 
     preallocated_length = 0;
     if (argc >= 1) {
-        if (!gjs_value_to_gsize(context, argv[0], &preallocated_length)) {
+        // COMPAT: in mozjs31 indexing CallArgs will already yield a rooted value
+        JS::RootedValue val(context, argv[0]);
+        if (!gjs_value_to_gsize(context, val, &preallocated_length)) {
             gjs_throw(context,
                       "Argument to ByteArray constructor should be a positive number for array length");
-            return JS_FALSE;
+            return false;
         }
     }
 
@@ -386,7 +376,7 @@ GJS_NATIVE_CONSTRUCTOR_DECLARE(byte_array)
 
     GJS_NATIVE_CONSTRUCTOR_FINISH(byte_array);
 
-    return JS_TRUE;
+    return true;
 }
 
 static void
@@ -401,7 +391,7 @@ byte_array_finalize(JSFreeOp *fop,
         return; /* prototype, not instance */
 
     if (priv->array) {
-        g_byte_array_free(priv->array, TRUE);
+        g_byte_array_free(priv->array, true);
         priv->array = NULL;
     } else if (priv->bytes) {
         g_clear_pointer(&priv->bytes, g_bytes_unref);
@@ -414,40 +404,35 @@ byte_array_finalize(JSFreeOp *fop,
 static JSBool
 to_string_func(JSContext *context,
                unsigned   argc,
-               jsval     *vp)
+               JS::Value *vp)
 {
-    JS::CallArgs argv = JS::CallArgsFromVp (argc, vp);
-    JSObject *object = JSVAL_TO_OBJECT(argv.thisv());
-    ByteArrayInstance *priv;
+    GJS_GET_PRIV(context, argc, vp, argv, to, ByteArrayInstance, priv);
     char *encoding;
-    gboolean encoding_is_utf8;
+    bool encoding_is_utf8;
     gchar *data;
 
-    priv = priv_from_js(context, object);
-
     if (priv == NULL)
-        return JS_TRUE; /* prototype, not instance */
+        return true; /* prototype, not instance */
 
     byte_array_ensure_array(priv);
 
-    if (argc >= 1 &&
-        JSVAL_IS_STRING(argv[0])) {
+    if (argc >= 1 && argv[0].isString()) {
         if (!gjs_string_to_utf8(context, argv[0], &encoding))
-            return JS_FALSE;
+            return false;
 
         /* maybe we should be smarter about utf8 synonyms here.
          * doesn't matter much though. encoding_is_utf8 is
          * just an optimization anyway.
          */
         if (strcmp(encoding, "UTF-8") == 0) {
-            encoding_is_utf8 = TRUE;
+            encoding_is_utf8 = true;
             g_free(encoding);
             encoding = NULL;
         } else {
-            encoding_is_utf8 = FALSE;
+            encoding_is_utf8 = false;
         }
     } else {
-        encoding_is_utf8 = TRUE;
+        encoding_is_utf8 = true;
     }
 
     if (priv->array->len == 0)
@@ -460,22 +445,15 @@ to_string_func(JSContext *context,
         /* optimization, avoids iconv overhead and runs
          * libmozjs hardwired utf8-to-utf16
          */
-        jsval retval;
-        JSBool ok;
-
-        ok = gjs_string_from_utf8(context,
-                                  data,
-                                  priv->array->len,
-                                  &retval);
-        if (ok)
-            argv.rval().set(retval);
-        return ok;
+        return gjs_string_from_utf8(context, data, priv->array->len,
+                                    argv.rval());
     } else {
-        JSBool ok = JS_FALSE;
+        bool ok = false;
         gsize bytes_written;
         GError *error;
         JSString *s;
         char *u16_str;
+        jschar *u16_out;
 
         error = NULL;
         u16_str = g_convert(data,
@@ -489,7 +467,7 @@ to_string_func(JSContext *context,
         if (u16_str == NULL) {
             /* frees the GError */
             gjs_throw_g_error(context, error);
-            return JS_FALSE;
+            return false;
         }
 
         /* bytes_written should be bytes in a UTF-16 string so
@@ -497,15 +475,16 @@ to_string_func(JSContext *context,
          */
         g_assert((bytes_written % 2) == 0);
 
-        s = JS_NewUCStringCopyN(context,
-                                (jschar*) u16_str,
-                                bytes_written / 2);
+        u16_out = g_new(jschar, bytes_written / 2);
+        memcpy(u16_out, u16_str, bytes_written);
+        s = JS_NewUCStringCopyN(context, u16_out, bytes_written / 2);
         if (s != NULL) {
-            ok = JS_TRUE;
-            argv.rval().set(STRING_TO_JSVAL(s));
+            ok = true;
+            argv.rval().setString(s);
         }
 
         g_free(u16_str);
+        g_free(u16_out);
         return ok;
     }
 }
@@ -513,26 +492,23 @@ to_string_func(JSContext *context,
 static JSBool
 to_gbytes_func(JSContext *context,
                unsigned   argc,
-               jsval     *vp)
+               JS::Value *vp)
 {
-    JS::CallReceiver rec = JS::CallReceiverFromVp(vp);
-    JSObject *object = JSVAL_TO_OBJECT(rec.thisv());
-    ByteArrayInstance *priv;
+    GJS_GET_PRIV(context, argc, vp, rec, to, ByteArrayInstance, priv);
     JSObject *ret_bytes_obj;
     GIBaseInfo *gbytes_info;
 
-    priv = priv_from_js(context, object);
     if (priv == NULL)
-        return JS_TRUE; /* prototype, not instance */
-    
+        return true; /* prototype, not instance */
+
     byte_array_ensure_gbytes(priv);
 
     gbytes_info = g_irepository_find_by_gtype(NULL, G_TYPE_BYTES);
     ret_bytes_obj = gjs_boxed_from_c_struct(context, (GIStructInfo*)gbytes_info,
                                             priv->bytes, GJS_BOXED_CREATION_NONE);
 
-    rec.rval().set(OBJECT_TO_JSVAL(ret_bytes_obj));
-    return JS_TRUE;
+    rec.rval().setObjectOrNull(ret_bytes_obj);
+    return true;
 }
 
 /* Ensure that the module and class objects exists, and that in turn
@@ -540,29 +516,27 @@ to_gbytes_func(JSContext *context,
 static JSObject *
 byte_array_get_prototype(JSContext *context)
 {
-    jsval retval;
-    JSObject *prototype;
+    JS::RootedValue retval(context,
+        gjs_get_global_slot(context, GJS_GLOBAL_SLOT_BYTE_ARRAY_PROTOTYPE));
 
-    retval = gjs_get_global_slot (context, GJS_GLOBAL_SLOT_BYTE_ARRAY_PROTOTYPE);
-
-    if (!JSVAL_IS_OBJECT (retval)) {
-        if (!gjs_eval_with_scope(context, NULL,
+    if (!retval.isObject()) {
+        if (!gjs_eval_with_scope(context, JS::NullPtr(),
                                  "imports.byteArray.ByteArray.prototype;", -1,
                                  "<internal>", &retval))
             g_error ("Could not import byte array prototype\n");
     }
 
-    return JSVAL_TO_OBJECT(retval);
+    return &retval.toObject();
 }
 
 static JSObject*
 byte_array_new(JSContext *context)
 {
-    JSObject *array;
     ByteArrayInstance *priv;
 
-    array = JS_NewObject(context, &gjs_byte_array_class,
-                         byte_array_get_prototype(context), NULL);
+    JS::RootedObject array(context,
+                           JS_NewObject(context, &gjs_byte_array_class,
+                                        byte_array_get_prototype(context), NULL));
 
     priv = g_slice_new0(ByteArrayInstance);
 
@@ -576,20 +550,16 @@ byte_array_new(JSContext *context)
 static JSBool
 from_string_func(JSContext *context,
                  unsigned   argc,
-                 jsval     *vp)
+                 JS::Value *vp)
 {
     JS::CallArgs argv = JS::CallArgsFromVp (argc, vp);
     ByteArrayInstance *priv;
     char *encoding;
-    gboolean encoding_is_utf8;
-    JSObject *obj;
-    JSBool retval = JS_FALSE;
+    bool encoding_is_utf8;
+    JS::RootedObject obj(context, byte_array_new(context));
 
-    obj = byte_array_new(context);
     if (obj == NULL)
-        return JS_FALSE;
-
-    JS_AddObjectRoot(context, &obj);
+        return false;
 
     priv = priv_from_js(context, obj);
     g_assert (priv != NULL);
@@ -598,30 +568,29 @@ from_string_func(JSContext *context,
 
     priv->array = gjs_g_byte_array_new(0);
 
-    if (!JSVAL_IS_STRING(argv[0])) {
+    if (!argv[0].isString()) {
         gjs_throw(context,
                   "byteArray.fromString() called with non-string as first arg");
-        goto out;
+        return false;
     }
 
-    if (argc > 1 &&
-        JSVAL_IS_STRING(argv[1])) {
+    if (argc > 1 && argv[1].isString()) {
         if (!gjs_string_to_utf8(context, argv[1], &encoding))
-            goto out;
+            return false;
 
         /* maybe we should be smarter about utf8 synonyms here.
          * doesn't matter much though. encoding_is_utf8 is
          * just an optimization anyway.
          */
         if (strcmp(encoding, "UTF-8") == 0) {
-            encoding_is_utf8 = TRUE;
+            encoding_is_utf8 = true;
             g_free(encoding);
             encoding = NULL;
         } else {
-            encoding_is_utf8 = FALSE;
+            encoding_is_utf8 = false;
         }
     } else {
-        encoding_is_utf8 = TRUE;
+        encoding_is_utf8 = true;
     }
 
     if (encoding_is_utf8) {
@@ -632,7 +601,7 @@ from_string_func(JSContext *context,
         if (!gjs_string_to_utf8(context,
                                 argv[0],
                                 &utf8))
-            goto out;
+            return false;
 
         g_byte_array_set_size(priv->array, 0);
         g_byte_array_append(priv->array, (guint8*) utf8, strlen(utf8));
@@ -644,9 +613,9 @@ from_string_func(JSContext *context,
         const jschar *u16_chars;
         gsize u16_len;
 
-        u16_chars = JS_GetStringCharsAndLength(context, JSVAL_TO_STRING(argv[0]), &u16_len);
+        u16_chars = JS_GetStringCharsAndLength(context, argv[0].toString(), &u16_len);
         if (u16_chars == NULL)
-            goto out;
+            return false;
 
         error = NULL;
         encoded = g_convert((char*) u16_chars,
@@ -660,7 +629,7 @@ from_string_func(JSContext *context,
         if (encoded == NULL) {
             /* frees the GError */
             gjs_throw_g_error(context, error);
-            goto out;
+            return false;
         }
 
         g_byte_array_set_size(priv->array, 0);
@@ -669,32 +638,24 @@ from_string_func(JSContext *context,
         g_free(encoded);
     }
 
-    argv.rval().set(OBJECT_TO_JSVAL(obj));
-
-    retval = JS_TRUE;
- out:
-    JS_RemoveObjectRoot(context, &obj);
-    return retval;
+    argv.rval().setObject(*obj);
+    return true;
 }
 
 /* fromArray() function implementation */
 static JSBool
 from_array_func(JSContext *context,
                 unsigned   argc,
-                jsval     *vp)
+                JS::Value *vp)
 {
     JS::CallArgs argv = JS::CallArgsFromVp (argc, vp);
     ByteArrayInstance *priv;
     guint32 len;
     guint32 i;
-    JSObject *obj;
-    JSBool ret = JS_FALSE;
+    JS::RootedObject obj(context, byte_array_new(context));
 
-    obj = byte_array_new(context);
     if (obj == NULL)
-        return JS_FALSE;
-
-    JS_AddObjectRoot(context, &obj);
+        return false;
 
     priv = priv_from_js(context, obj);
     g_assert (priv != NULL);
@@ -703,94 +664,88 @@ from_array_func(JSContext *context,
 
     priv->array = gjs_g_byte_array_new(0);
 
-    if (!JS_IsArrayObject(context, JSVAL_TO_OBJECT(argv[0]))) {
+    if (!JS_IsArrayObject(context, &argv[0].toObject())) {
         gjs_throw(context,
                   "byteArray.fromArray() called with non-array as first arg");
-        goto out;
+        return false;
     }
 
-    if (!JS_GetArrayLength(context, JSVAL_TO_OBJECT(argv[0]), &len)) {
+    if (!JS_GetArrayLength(context, &argv[0].toObject(), &len)) {
         gjs_throw(context,
                   "byteArray.fromArray() can't get length of first array arg");
-        goto out;
+        return false;
     }
 
     g_byte_array_set_size(priv->array, len);
 
+    JS::RootedValue elem(context);
     for (i = 0; i < len; ++i) {
-        jsval elem;
         guint8 b;
 
-        elem = JSVAL_VOID;
-        if (!JS_GetElement(context, JSVAL_TO_OBJECT(argv[0]), i, &elem)) {
-            /* this means there was an exception, while elem == JSVAL_VOID
+        elem = JS::UndefinedValue();
+        if (!JS_GetElement(context, &argv[0].toObject(), i, elem.address())) {
+            /* this means there was an exception, while elem.isUndefined()
              * means no element found
              */
-            goto out;
+            return false;
         }
 
-        if (JSVAL_IS_VOID(elem))
+        if (elem.isUndefined())
             continue;
 
         if (!gjs_value_to_byte(context, elem, &b))
-            goto out;
+            return false;
 
         g_array_index(priv->array, guint8, i) = b;
     }
 
-    ret = JS_TRUE;
-    argv.rval().set(OBJECT_TO_JSVAL(obj));
- out:
-    JS_RemoveObjectRoot(context, &obj);
-    return ret;
+    argv.rval().setObject(*obj);
+    return true;
 }
 
 static JSBool
 from_gbytes_func(JSContext *context,
                  unsigned   argc,
-                 jsval     *vp)
+                 JS::Value *vp)
 {
     JS::CallArgs argv = JS::CallArgsFromVp (argc, vp);
-    JSObject *bytes_obj;
+    JS::RootedObject bytes_obj(context);
     GBytes *gbytes;
     ByteArrayInstance *priv;
-    JSObject *obj;
-    JSBool ret = JS_FALSE;
 
-    if (!gjs_parse_call_args(context, "overrides_gbytes_to_array", "o", argv,
-                        "bytes", &bytes_obj))
-        return JS_FALSE;
+    if (!gjs_parse_call_args(context, "overrides_gbytes_to_array", argv, "o",
+                             "bytes", &bytes_obj))
+        return false;
 
-    if (!gjs_typecheck_boxed(context, bytes_obj, NULL, G_TYPE_BYTES, TRUE))
-        return JS_FALSE;
+    if (!gjs_typecheck_boxed(context, bytes_obj, NULL, G_TYPE_BYTES, true))
+        return false;
 
     gbytes = (GBytes*) gjs_c_struct_from_boxed(context, bytes_obj);
 
-    obj = byte_array_new(context);
+    JS::RootedObject obj(context, byte_array_new(context));
     if (obj == NULL)
-        return JS_FALSE;
+        return false;
     priv = priv_from_js(context, obj);
     g_assert (priv != NULL);
 
     priv->bytes = g_bytes_ref(gbytes);
 
-    ret = JS_TRUE;
-    argv.rval().set(OBJECT_TO_JSVAL(obj));
-    return ret;
+    argv.rval().setObject(*obj);
+    return true;
 }
 
 JSObject *
 gjs_byte_array_from_byte_array (JSContext *context,
                                 GByteArray *array)
 {
-    JSObject *object;
     ByteArrayInstance *priv;
 
     g_return_val_if_fail(context != NULL, NULL);
     g_return_val_if_fail(array != NULL, NULL);
 
-    object = JS_NewObject(context, &gjs_byte_array_class,
-                          byte_array_get_prototype(context), NULL);
+    JS::RootedObject object(context,
+                            JS_NewObject(context, &gjs_byte_array_class,
+                                         byte_array_get_prototype(context), NULL));
     if (!object) {
         gjs_throw(context, "failed to create byte array");
         return NULL;
@@ -806,34 +761,9 @@ gjs_byte_array_from_byte_array (JSContext *context,
     return object;
 }
 
-JSObject *
-gjs_byte_array_from_bytes (JSContext *context,
-                           GBytes    *bytes)
-{
-    JSObject *object;
-    ByteArrayInstance *priv;
-
-    g_return_val_if_fail(context != NULL, NULL);
-    g_return_val_if_fail(bytes != NULL, NULL);
-
-    object = JS_NewObject(context, &gjs_byte_array_class,
-                          byte_array_get_prototype(context), NULL);
-    if (!object) {
-        gjs_throw(context, "failed to create byte array");
-        return NULL;
-    }
-
-    priv = g_slice_new0(ByteArrayInstance);
-    g_assert(priv_from_js(context, object) == NULL);
-    JS_SetPrivate(object, priv);
-    priv->bytes = g_bytes_ref (bytes);
-
-    return object;
-}
-
 GBytes *
-gjs_byte_array_get_bytes (JSContext  *context,
-                          JSObject   *object)
+gjs_byte_array_get_bytes (JSContext       *context,
+                          JS::HandleObject object)
 {
     ByteArrayInstance *priv;
     priv = priv_from_js(context, object);
@@ -845,8 +775,8 @@ gjs_byte_array_get_bytes (JSContext  *context,
 }
 
 GByteArray *
-gjs_byte_array_get_byte_array (JSContext   *context,
-                               JSObject    *obj)
+gjs_byte_array_get_byte_array (JSContext       *context,
+                               JS::HandleObject obj)
 {
     ByteArrayInstance *priv;
     priv = priv_from_js(context, obj);
@@ -858,10 +788,10 @@ gjs_byte_array_get_byte_array (JSContext   *context,
 }
 
 void
-gjs_byte_array_peek_data (JSContext  *context,
-                          JSObject   *obj,
-                          guint8    **out_data,
-                          gsize      *out_len)
+gjs_byte_array_peek_data (JSContext       *context,
+                          JS::HandleObject obj,
+                          guint8         **out_data,
+                          gsize           *out_len)
 {
     ByteArrayInstance *priv;
     priv = priv_from_js(context, obj);
@@ -877,43 +807,32 @@ gjs_byte_array_peek_data (JSContext  *context,
     }
 }
 
-/* no idea what this is used for. examples in
- * spidermonkey use -1, -2, -3, etc. for tinyids.
- */
-enum ByteArrayTinyId {
-    BYTE_ARRAY_TINY_ID_LENGTH = -1
-};
-
 JSPropertySpec gjs_byte_array_proto_props[] = {
-    { "length", BYTE_ARRAY_TINY_ID_LENGTH,
-      JSPROP_PERMANENT,
-      JSOP_WRAPPER ((JSPropertyOp) byte_array_length_getter),
-      JSOP_WRAPPER ((JSStrictPropertyOp) byte_array_length_setter),
-    },
-    { NULL }
+    JS_PSGS("length", byte_array_length_getter, byte_array_length_setter,
+            JSPROP_PERMANENT),
+    JS_PS_END
 };
 
 JSFunctionSpec gjs_byte_array_proto_funcs[] = {
-    { "toString", JSOP_WRAPPER ((JSNative) to_string_func), 0, 0 },
-    { "toGBytes", JSOP_WRAPPER ((JSNative) to_gbytes_func), 0, 0 },
-    { NULL }
+    JS_FS("toString", to_string_func, 0, 0),
+    JS_FS("toGBytes", to_gbytes_func, 0, 0),
+    JS_FS_END
 };
 
 static JSFunctionSpec gjs_byte_array_module_funcs[] = {
-    { "fromString", JSOP_WRAPPER (from_string_func), 1, 0 },
-    { "fromArray", JSOP_WRAPPER (from_array_func), 1, 0 },
-    { "fromGBytes", JSOP_WRAPPER (from_gbytes_func), 1, 0 },
-    { NULL }
+    JS_FS("fromString", from_string_func, 1, 0),
+    JS_FS("fromArray", from_array_func, 1, 0),
+    JS_FS("fromGBytes", from_gbytes_func, 1, 0),
+    JS_FS_END
 };
 
-JSBool
-gjs_define_byte_array_stuff(JSContext  *context,
-                            JSObject  **module_out)
+bool
+gjs_define_byte_array_stuff(JSContext              *context,
+                            JS::MutableHandleObject module)
 {
-    JSObject *module;
     JSObject *prototype;
 
-    module = JS_NewObject (context, NULL, NULL, NULL);
+    module.set(JS_NewObject(context, NULL, NULL, NULL));
 
     prototype = JS_InitClass(context, module,
                              NULL,
@@ -926,12 +845,11 @@ gjs_define_byte_array_stuff(JSContext  *context,
                              NULL);
 
     if (!JS_DefineFunctions(context, module, &gjs_byte_array_module_funcs[0]))
-        return JS_FALSE;
+        return false;
 
-    g_assert(JSVAL_IS_VOID(gjs_get_global_slot(context, GJS_GLOBAL_SLOT_BYTE_ARRAY_PROTOTYPE)));
+    g_assert(gjs_get_global_slot(context, GJS_GLOBAL_SLOT_BYTE_ARRAY_PROTOTYPE).isUndefined());
     gjs_set_global_slot(context, GJS_GLOBAL_SLOT_BYTE_ARRAY_PROTOTYPE,
-                        OBJECT_TO_JSVAL(prototype));
+                        JS::ObjectOrNullValue(prototype));
 
-    *module_out = module;
-    return JS_TRUE;
+    return true;
 }
