@@ -25,11 +25,10 @@
 #define __GJS_CONTEXT_PRIVATE_H__
 
 #include "context.h"
-#include "compat.h"
 
 G_BEGIN_DECLS
 
-gboolean     _gjs_context_destroying                  (GjsContext *js_context);
+bool         _gjs_context_destroying                  (GjsContext *js_context);
 
 void         _gjs_context_schedule_gc_if_needed       (GjsContext *js_context);
 
