@@ -29,9 +29,9 @@
 #include "importer.h"
 #include "jsapi-private.h"
 #include "jsapi-util.h"
+#include "jsapi-wrapper.h"
 #include "native.h"
 #include "byteArray.h"
-#include "compat.h"
 #include "runtime.h"
 
 #include "gi.h"
@@ -61,13 +61,13 @@ struct _GjsContext {
 
     JSRuntime *runtime;
     JSContext *context;
-    JSObject *global;
+    JS::Heap<JSObject*> global;
 
     char *program_name;
 
     char **search_path;
 
-    gboolean destroying;
+    bool destroying;
 
     guint    auto_gc_id;
 
@@ -105,7 +105,7 @@ static GList *all_contexts = NULL;
 static JSBool
 gjs_log(JSContext *context,
         unsigned   argc,
-        jsval     *vp)
+        JS::Value *vp)
 {
     JS::CallArgs argv = JS::CallArgsFromVp (argc, vp);
     char *s;
@@ -114,7 +114,7 @@ gjs_log(JSContext *context,
 
     if (argc != 1) {
         gjs_throw(context, "Must pass a single argument to log()");
-        return JS_FALSE;
+        return false;
     }
 
     JS_BeginRequest(context);
@@ -124,65 +124,64 @@ gjs_log(JSContext *context,
     exc_state = JS_SaveExceptionState(context);
     jstr = JS_ValueToString(context, argv[0]);
     if (jstr != NULL)
-        argv[0] = STRING_TO_JSVAL(jstr);    // GC root
+        argv[0] = JS::StringValue(jstr);    // GC root
     JS_RestoreExceptionState(context, exc_state);
 
     if (jstr == NULL) {
         g_message("JS LOG: <cannot convert value to string>");
         JS_EndRequest(context);
-        return JS_TRUE;
+        return true;
     }
 
-    if (!gjs_string_to_utf8(context, STRING_TO_JSVAL(jstr), &s)) {
+    if (!gjs_string_to_utf8(context, JS::StringValue(jstr), &s)) {
         JS_EndRequest(context);
-        return JS_FALSE;
+        return false;
     }
 
     g_message("JS LOG: %s", s);
     g_free(s);
 
     JS_EndRequest(context);
-    argv.rval().set(JSVAL_VOID);
-    return JS_TRUE;
+    argv.rval().setUndefined();
+    return true;
 }
 
 static JSBool
 gjs_log_error(JSContext *context,
               unsigned   argc,
-              jsval     *vp)
+              JS::Value *vp)
 {
     JS::CallArgs argv = JS::CallArgsFromVp (argc, vp);
     JSExceptionState *exc_state;
-    JSString *jstr;
 
-    if ((argc != 1 && argc != 2) ||
-        !JSVAL_IS_OBJECT (argv[0])) {
+    if ((argc != 1 && argc != 2) || !argv[0].isObject()) {
         gjs_throw(context, "Must pass an exception and optionally a message to logError()");
-        return JS_FALSE;
+        return false;
     }
 
     JS_BeginRequest(context);
 
+    JS::RootedString jstr(context);
+
     if (argc == 2) {
         /* JS_ValueToString might throw, in which we will only
          *log that the value could be converted to string */
         exc_state = JS_SaveExceptionState(context);
         jstr = JS_ValueToString(context, argv[1]);
         if (jstr != NULL)
-            argv[1] = STRING_TO_JSVAL(jstr);    // GC root
+            argv[1] = JS::StringValue(jstr);    // GC root
         JS_RestoreExceptionState(context, exc_state);
-    } else {
-        jstr = NULL;
     }
 
-    gjs_log_exception_full(context, argv[0], jstr);
+    /* COMPAT: JS::CallArgs::operator[] will yield Handle in mozjs31 */
+    gjs_log_exception_full(context, argv.handleOrUndefinedAt(0), jstr);
 
     JS_EndRequest(context);
-    argv.rval().set(JSVAL_VOID);
-    return JS_TRUE;
+    argv.rval().setUndefined();
+    return true;
 }
 
-static JSBool
+static bool
 gjs_print_parse_args(JSContext *context,
                      JS::CallArgs &argv,
                      char     **buffer)
@@ -204,15 +203,15 @@ gjs_print_parse_args(JSContext *context,
 
         jstr = JS_ValueToString(context, argv[n]);
         if (jstr != NULL)
-            argv[n] = STRING_TO_JSVAL(jstr); // GC root
+            argv[n] = JS::StringValue(jstr); // GC root
 
         JS_RestoreExceptionState(context, exc_state);
 
         if (jstr != NULL) {
-            if (!gjs_string_to_utf8(context, STRING_TO_JSVAL(jstr), &s)) {
+            if (!gjs_string_to_utf8(context, JS::StringValue(jstr), &s)) {
                 JS_EndRequest(context);
-                g_string_free(str, TRUE);
-                return JS_FALSE;
+                g_string_free(str, true);
+                return false;
             }
 
             g_string_append(str, s);
@@ -221,55 +220,55 @@ gjs_print_parse_args(JSContext *context,
                 g_string_append_c(str, ' ');
         } else {
             JS_EndRequest(context);
-            *buffer = g_string_free(str, TRUE);
+            *buffer = g_string_free(str, true);
             if (!*buffer)
                 *buffer = g_strdup("<invalid string>");
-            return JS_TRUE;
+            return true;
         }
 
     }
-    *buffer = g_string_free(str, FALSE);
+    *buffer = g_string_free(str, false);
 
     JS_EndRequest(context);
-    return JS_TRUE;
+    return true;
 }
 
 static JSBool
 gjs_print(JSContext *context,
           unsigned   argc,
-          jsval     *vp)
+          JS::Value *vp)
 {
     JS::CallArgs argv = JS::CallArgsFromVp (argc, vp);
     char *buffer;
 
     if (!gjs_print_parse_args(context, argv, &buffer)) {
-        return FALSE;
+        return false;
     }
 
     g_print("%s\n", buffer);
     g_free(buffer);
 
-    argv.rval().set(JSVAL_VOID);
-    return JS_TRUE;
+    argv.rval().setUndefined();
+    return true;
 }
 
 static JSBool
 gjs_printerr(JSContext *context,
              unsigned   argc,
-             jsval     *vp)
+             JS::Value *vp)
 {
     JS::CallArgs argv = JS::CallArgsFromVp (argc, vp);
     char *buffer;
 
     if (!gjs_print_parse_args(context, argv, &buffer)) {
-        return FALSE;
+        return false;
     }
 
     g_printerr("%s\n", buffer);
     g_free(buffer);
 
-    argv.rval().set(JSVAL_VOID);
-    return JS_TRUE;
+    argv.rval().setUndefined();
+    return true;
 }
 
 static void
@@ -325,6 +324,13 @@ gjs_context_class_init(GjsContextClass *klass)
     gjs_register_static_modules();
 }
 
+static void
+gjs_context_tracer(JSTracer *trc, void *data)
+{
+    GjsContext *gjs_context = reinterpret_cast<GjsContext *>(data);
+    JS_CallHeapObjectTracer(trc, &gjs_context->global, "GJS global object");
+}
+
 static void
 gjs_context_dispose(GObject *object)
 {
@@ -332,16 +338,13 @@ gjs_context_dispose(GObject *object)
 
     js_context = GJS_CONTEXT(object);
 
-    if (js_context->global != NULL) {
-        js_context->global = NULL;
-    }
-
     if (js_context->context != NULL) {
 
         gjs_debug(GJS_DEBUG_CONTEXT,
                   "Destroying JS context");
 
         JS_BeginRequest(js_context->context);
+
         /* Do a full GC here before tearing down, since once we do
          * that we may not have the JS_GetPrivate() to access the
          * context
@@ -349,7 +352,7 @@ gjs_context_dispose(GObject *object)
         JS_GC(js_context->runtime);
         JS_EndRequest(js_context->context);
 
-        js_context->destroying = TRUE;
+        js_context->destroying = true;
 
         /* Now, release all native objects, to avoid recursion between
          * the JS teardown and the C teardown.  The JSObject proxies
@@ -362,6 +365,9 @@ gjs_context_dispose(GObject *object)
             js_context->auto_gc_id = 0;
         }
 
+        JS_RemoveExtraGCRootsTracer(js_context->runtime, gjs_context_tracer,
+                                    js_context);
+
         /* Tear down JS */
         JS_DestroyContext(js_context->context);
         js_context->context = NULL;
@@ -399,11 +405,11 @@ gjs_context_finalize(GObject *object)
 }
 
 static JSFunctionSpec global_funcs[] = {
-    { "log", JSOP_WRAPPER (gjs_log), 1, GJS_MODULE_PROP_FLAGS },
-    { "logError", JSOP_WRAPPER (gjs_log_error), 2, GJS_MODULE_PROP_FLAGS },
-    { "print", JSOP_WRAPPER (gjs_print), 0, GJS_MODULE_PROP_FLAGS },
-    { "printerr", JSOP_WRAPPER (gjs_printerr), 0, GJS_MODULE_PROP_FLAGS },
-    { NULL },
+    JS_FS("log", gjs_log, 1, GJS_MODULE_PROP_FLAGS),
+    JS_FS("logError", gjs_log_error, 2, GJS_MODULE_PROP_FLAGS),
+    JS_FS("print", gjs_print, 0, GJS_MODULE_PROP_FLAGS),
+    JS_FS("printerr", gjs_printerr, 0, GJS_MODULE_PROP_FLAGS),
+    JS_FS_END
 };
 
 static void
@@ -427,21 +433,26 @@ gjs_context_constructed(GObject *object)
 
     /* set ourselves as the private data */
     JS_SetContextPrivate(js_context->context, js_context);
+    JS::RootedObject global(js_context->context);
 
-    if (!gjs_init_context_standard(js_context->context, &js_context->global))
+    if (!gjs_init_context_standard(js_context->context, &global))
         g_error("Failed to initialize context");
 
-    JSAutoCompartment ac(js_context->context, js_context->global);
+    JSAutoCompartment ac(js_context->context, global);
 
-    if (!JS_DefineProperty(js_context->context, js_context->global,
-                           "window", OBJECT_TO_JSVAL(js_context->global),
+    JS::RootedValue v_global(js_context->context, JS::ObjectValue(*global));
+    if (!JS_DefineProperty(js_context->context, global,
+                           "window", v_global,
                            NULL, NULL,
                            JSPROP_READONLY | JSPROP_PERMANENT))
         g_error("No memory to export global object as 'window'");
 
-    if (!JS_DefineFunctions(js_context->context, js_context->global, &global_funcs[0]))
+    if (!JS_DefineFunctions(js_context->context, global, &global_funcs[0]))
         g_error("Failed to define properties on the global object");
 
+    js_context->global.set(global);
+    JS_AddExtraGCRootsTracer(js_context->runtime, gjs_context_tracer, js_context);
+
     /* We create the global-to-runtime root importer with the
      * passed-in search path. If someone else already created
      * the root importer, this is a no-op.
@@ -450,7 +461,7 @@ gjs_context_constructed(GObject *object)
                                   js_context->search_path ?
                                   (const char**) js_context->search_path :
                                   NULL,
-                                  TRUE))
+                                  true))
         g_error("Failed to create root importer");
 
     /* Now copy the global root importer (which we just created,
@@ -525,7 +536,7 @@ gjs_context_new_with_search_path(char** search_path)
                          NULL);
 }
 
-gboolean
+bool
 _gjs_context_destroying (GjsContext *context)
 {
     return context->destroying;
@@ -537,7 +548,7 @@ trigger_gc_if_needed (gpointer user_data)
     GjsContext *js_context = GJS_CONTEXT(user_data);
     js_context->auto_gc_id = 0;
     gjs_gc_if_needed(js_context->context);
-    return FALSE;
+    return G_SOURCE_REMOVE;
 }
 
 void
@@ -626,7 +637,7 @@ gjs_context_get_native_context (GjsContext *js_context)
     return js_context->context;
 }
 
-gboolean
+bool
 gjs_context_eval(GjsContext   *js_context,
                  const char   *script,
                  gssize        script_len,
@@ -634,15 +645,16 @@ gjs_context_eval(GjsContext   *js_context,
                  int          *exit_status_p,
                  GError      **error)
 {
-    gboolean ret = FALSE;
-    jsval retval;
+    bool ret = false;
 
     JSAutoCompartment ac(js_context->context, js_context->global);
     JSAutoRequest ar(js_context->context);
 
     g_object_ref(G_OBJECT(js_context));
 
-    if (!gjs_eval_with_scope(js_context->context, NULL, script, script_len, filename, &retval)) {
+    JS::RootedValue retval(js_context->context);
+    if (!gjs_eval_with_scope(js_context->context, JS::NullPtr(), script,
+                             script_len, filename, &retval)) {
         gjs_log_exception(js_context->context);
         g_set_error(error,
                     GJS_ERROR,
@@ -652,27 +664,25 @@ gjs_context_eval(GjsContext   *js_context,
     }
 
     if (exit_status_p) {
-        if (JSVAL_IS_INT(retval)) {
-            int code;
-            if (JS_ValueToInt32(js_context->context, retval, &code)) {
-                gjs_debug(GJS_DEBUG_CONTEXT,
-                          "Script returned integer code %d", code);
-                *exit_status_p = code;
-            }
+        if (retval.isInt32()) {
+            int code = retval.toInt32();
+            gjs_debug(GJS_DEBUG_CONTEXT,
+                      "Script returned integer code %d", code);
+            *exit_status_p = code;
         } else {
             /* Assume success if no integer was returned */
             *exit_status_p = 0;
         }
     }
 
-    ret = TRUE;
+    ret = true;
 
  out:
     g_object_unref(G_OBJECT(js_context));
     return ret;
 }
 
-gboolean
+bool
 gjs_context_eval_file(GjsContext    *js_context,
                       const char    *filename,
                       int           *exit_status_p,
@@ -680,22 +690,22 @@ gjs_context_eval_file(GjsContext    *js_context,
 {
     char     *script = NULL;
     gsize    script_len;
-    gboolean ret = TRUE;
+    bool ret = true;
 
     GFile *file = g_file_new_for_commandline_arg(filename);
 
     if (!g_file_query_exists(file, NULL)) {
-        ret = FALSE;
+        ret = false;
         goto out;
     }
 
     if (!g_file_load_contents(file, NULL, &script, &script_len, NULL, error)) {
-        ret = FALSE;
+        ret = false;
         goto out;
     }
 
     if (!gjs_context_eval(js_context, script, script_len, filename, exit_status_p, error)) {
-        ret = FALSE;
+        ret = false;
         goto out;
     }
 
@@ -705,7 +715,7 @@ out:
     return ret;
 }
 
-gboolean
+bool
 gjs_context_define_string_array(GjsContext  *js_context,
                                 const char    *array_name,
                                 gssize         array_length,
@@ -722,10 +732,10 @@ gjs_context_define_string_array(GjsContext  *js_context,
                     GJS_ERROR,
                     GJS_ERROR_FAILED,
                     "gjs_define_string_array() failed");
-        return FALSE;
+        return false;
     }
 
-    return TRUE;
+    return true;
 }
 
 static GjsContext *current_context;
@@ -752,15 +762,14 @@ gjs_context_get_const_string(JSContext      *context,
     return gjs_context->const_strings[name];
 }
 
-gboolean
-gjs_object_get_property_const(JSContext      *context,
-                              JSObject       *obj,
-                              GjsConstString  property_name,
-                              jsval          *value_p)
+bool
+gjs_object_get_property_const(JSContext             *cx,
+                              JS::HandleObject       obj,
+                              GjsConstString         property_name,
+                              JS::MutableHandleValue value_p)
 {
-    jsid pname;
-    pname = gjs_context_get_const_string(context, property_name);
-    return JS_GetPropertyById(context, obj, pname, value_p);
+    JS::RootedId pname(cx, gjs_context_get_const_string(cx, property_name));
+    return JS_GetPropertyById(cx, obj, pname, value_p.address());
 }
 
 /**
