@@ -26,9 +26,10 @@
 #include <util/log.h>
 #include <util/glib.h>
 
-#include <gjs/gjs-module.h>
-#include <gjs/importer.h>
-#include <gjs/compat.h>
+#include "importer.h"
+#include "jsapi-wrapper.h"
+#include "mem.h"
+#include "native.h"
 
 #include <gio/gio.h>
 
@@ -39,7 +40,7 @@
 static char **gjs_search_path = NULL;
 
 typedef struct {
-    gboolean is_root;
+    bool is_root;
 } Importer;
 
 typedef struct {
@@ -51,14 +52,14 @@ extern struct JSClass gjs_importer_class;
 
 GJS_DEFINE_PRIV_FROM_JS(Importer, gjs_importer_class)
 
-static JSBool
-define_meta_properties(JSContext  *context,
-                       JSObject   *module_obj,
-                       const char *full_path,
-                       const char *module_name,
-                       JSObject   *parent)
+static bool
+define_meta_properties(JSContext       *context,
+                       JS::HandleObject module_obj,
+                       const char      *full_path,
+                       const char      *module_name,
+                       JS::HandleObject parent)
 {
-    gboolean parent_is_module;
+    bool parent_is_module;
 
     /* We define both __moduleName__ and __parentModule__ to null
      * on the root importer
@@ -66,50 +67,54 @@ define_meta_properties(JSContext  *context,
     parent_is_module = parent && JS_InstanceOf(context, parent, &gjs_importer_class, NULL);
 
     gjs_debug(GJS_DEBUG_IMPORTER, "Defining parent %p of %p '%s' is mod %d",
-              parent, module_obj, module_name ? module_name : "<root>", parent_is_module);
+              parent.get(), module_obj.get(),
+              module_name ? module_name : "<root>", parent_is_module);
 
     if (full_path != NULL) {
-        if (!JS_DefineProperty(context, module_obj,
-                               "__file__",
-                               STRING_TO_JSVAL(JS_NewStringCopyZ(context, full_path)),
+        JS::RootedValue file_val(context,
+            JS::StringValue(JS_NewStringCopyZ(context, full_path)));
+        if (!JS_DefineProperty(context, module_obj, "__file__", file_val,
                                NULL, NULL,
                                /* don't set ENUMERATE since we wouldn't want to copy
                                 * this symbol to any other object for example.
                                 */
                                JSPROP_READONLY | JSPROP_PERMANENT))
-            return JS_FALSE;
+            return false;
+    }
+
+    JS::RootedValue module_name_val(context, JS::NullValue());
+    JS::RootedValue parent_module_val(context, JS::NullValue());
+    if (parent_is_module) {
+        module_name_val.setString(JS_NewStringCopyZ(context, module_name));
+        parent_module_val.setObject(*parent);
     }
 
     if (!JS_DefineProperty(context, module_obj,
-                           "__moduleName__",
-                           parent_is_module ?
-                           STRING_TO_JSVAL(JS_NewStringCopyZ(context, module_name)) :
-                           JSVAL_NULL,
+                           "__moduleName__", module_name_val,
                            NULL, NULL,
                            /* don't set ENUMERATE since we wouldn't want to copy
                             * this symbol to any other object for example.
                             */
                            JSPROP_READONLY | JSPROP_PERMANENT))
-        return JS_FALSE;
+        return false;
 
     if (!JS_DefineProperty(context, module_obj,
-                           "__parentModule__",
-                           parent_is_module ? OBJECT_TO_JSVAL(parent) : JSVAL_NULL,
+                           "__parentModule__", parent_module_val,
                            NULL, NULL,
                            /* don't set ENUMERATE since we wouldn't want to copy
                             * this symbol to any other object for example.
                             */
                            JSPROP_READONLY | JSPROP_PERMANENT))
-        return JS_FALSE;
+        return false;
 
-    return JS_TRUE;
+    return true;
 }
 
-static JSBool
-import_directory(JSContext   *context,
-                 JSObject    *obj,
-                 const char  *name,
-                 const char **full_paths)
+static bool
+import_directory(JSContext       *context,
+                 JS::HandleObject obj,
+                 const char      *name,
+                 const char     **full_paths)
 {
     JSObject *importer;
 
@@ -121,36 +126,33 @@ import_directory(JSContext   *context,
      * its search path. gjs_define_importer() exits if it fails, so
      * this always succeeds.
      */
-    importer = gjs_define_importer(context, obj, name, full_paths, FALSE);
-    if (importer == NULL)
-        return JS_FALSE;
-
-    return JS_TRUE;
+    importer = gjs_define_importer(context, obj, name, full_paths, false);
+    return importer != NULL;
 }
 
-static JSBool
-define_import(JSContext  *context,
-              JSObject   *obj,
-              JSObject   *module_obj,
-              const char *name)
+static bool
+define_import(JSContext       *context,
+              JS::HandleObject obj,
+              JS::HandleObject module_obj,
+              const char      *name)
 {
-    if (!JS_DefineProperty(context, obj,
-                           name, OBJECT_TO_JSVAL(module_obj),
+    JS::RootedValue module_val(context, JS::ObjectValue(*module_obj));
+    if (!JS_DefineProperty(context, obj, name, module_val,
                            NULL, NULL,
                            GJS_MODULE_PROP_FLAGS & ~JSPROP_PERMANENT)) {
         gjs_debug(GJS_DEBUG_IMPORTER,
                   "Failed to define '%s' in importer",
                   name);
-        return JS_FALSE;
+        return false;
     }
 
-    return JS_TRUE;
+    return true;
 }
 
 /* Make the property we set in define_import permament;
  * we do this after the import succesfully completes.
  */
-static JSBool
+static bool
 seal_import(JSContext  *context,
             JSObject   *obj,
             const char *name)
@@ -163,7 +165,7 @@ seal_import(JSContext  *context,
         gjs_debug(GJS_DEBUG_IMPORTER,
                   "Failed to get attributes to seal '%s' in importer",
                   name);
-        return JS_FALSE;
+        return false;
     }
 
     attrs |= JSPROP_PERMANENT;
@@ -173,10 +175,10 @@ seal_import(JSContext  *context,
         gjs_debug(GJS_DEBUG_IMPORTER,
                   "Failed to set attributes to seal '%s' in importer",
                   name);
-        return JS_FALSE;
+        return false;
     }
 
-    return JS_TRUE;
+    return true;
 }
 
 /* An import failed. Delete the property pointing to the import
@@ -198,9 +200,9 @@ seal_import(JSContext  *context,
  * the attempt to clean up.)
  */
 static void
-cancel_import(JSContext  *context,
-              JSObject   *obj,
-              const char *name)
+cancel_import(JSContext       *context,
+              JS::HandleObject obj,
+              const char      *name)
 {
     gjs_debug(GJS_DEBUG_IMPORTER,
               "Cleaning up from failed import of '%s'",
@@ -213,40 +215,33 @@ cancel_import(JSContext  *context,
     }
 }
 
-static JSBool
-import_native_file(JSContext  *context,
-                   JSObject   *obj,
-                   const char *name)
+static bool
+import_native_file(JSContext       *context,
+                   JS::HandleObject obj,
+                   const char      *name)
 {
-    JSObject *module_obj;
-    JSBool retval = JS_FALSE;
+    JS::RootedObject module_obj(context);
 
     gjs_debug(GJS_DEBUG_IMPORTER, "Importing '%s'", name);
 
     if (!gjs_import_native_module(context, name, &module_obj))
-        goto out;
+        return false;
 
     if (!define_meta_properties(context, module_obj, NULL, name, obj))
-        goto out;
+        return false;
 
     if (JS_IsExceptionPending(context)) {
         /* I am not sure whether this can happen, but if it does we want to trap it.
          */
         gjs_debug(GJS_DEBUG_IMPORTER,
-                  "Module '%s' reported an exception but gjs_import_native_module() returned TRUE",
+                  "Module '%s' reported an exception but gjs_import_native_module() returned true",
                   name);
-        goto out;
+        return false;
     }
 
-    if (!JS_DefineProperty(context, obj,
-                           name, OBJECT_TO_JSVAL(module_obj),
-                           NULL, NULL, GJS_MODULE_PROP_FLAGS))
-        goto out;
-
-    retval = JS_TRUE;
-
- out:
-    return retval;
+    JS::RootedValue v_module(context, JS::ObjectValue(*module_obj));
+    return JS_DefineProperty(context, obj, name, v_module,
+                             NULL, NULL, GJS_MODULE_PROP_FLAGS);
 }
 
 static JSObject *
@@ -255,17 +250,16 @@ create_module_object(JSContext *context)
     return JS_NewObject(context, NULL, NULL, NULL);
 }
 
-static JSBool
-import_file(JSContext  *context,
-            const char *name,
-            GFile      *file,
-            JSObject   *module_obj)
+static bool
+import_file(JSContext       *context,
+            const char      *name,
+            GFile           *file,
+            JS::HandleObject module_obj)
 {
-    JSBool ret = JS_FALSE;
+    bool ret = false;
     char *script = NULL;
     char *full_path = NULL;
     gsize script_len = 0;
-    jsval script_retval;
     GError *error = NULL;
 
     JS::CompileOptions options(context);
@@ -289,7 +283,7 @@ import_file(JSContext  *context,
                              full_path, NULL))
         goto out;
 
-    ret = JS_TRUE;
+    ret = true;
 
  out:
     g_free(script);
@@ -298,35 +292,33 @@ import_file(JSContext  *context,
 }
 
 static JSObject *
-load_module_init(JSContext  *context,
-                 JSObject   *in_object,
-                 const char *full_path)
+load_module_init(JSContext       *context,
+                 JS::HandleObject in_object,
+                 const char      *full_path)
 {
-    JSObject *module_obj;
     JSBool found;
-    jsid module_init_name;
     GFile *file;
 
     /* First we check if js module has already been loaded  */
-    module_init_name = gjs_context_get_const_string(context, GJS_STRING_MODULE_INIT);
+    JS::RootedId module_init_name(context,
+        gjs_context_get_const_string(context, GJS_STRING_MODULE_INIT));
     if (JS_HasPropertyById(context, in_object, module_init_name, &found) && found) {
-        jsval module_obj_val;
-
+        JS::RootedValue module_obj_val(context);
         if (JS_GetPropertyById(context,
                                in_object,
                                module_init_name,
-                               &module_obj_val)) {
-            return JSVAL_TO_OBJECT(module_obj_val);
+                               module_obj_val.address())) {
+            return &module_obj_val.toObject();
         }
     }
 
-    module_obj = create_module_object (context);
+    JS::RootedObject module_obj(context, create_module_object(context));
     file = g_file_new_for_commandline_arg(full_path);
     if (!import_file (context, "__init__", file, module_obj))
         goto out;
 
     if (!JS_DefinePropertyById(context, in_object,
-                               module_init_name, OBJECT_TO_JSVAL(module_obj),
+                               module_init_name, JS::ObjectValue(*module_obj),
                                NULL, NULL,
                                GJS_MODULE_PROP_FLAGS & ~JSPROP_PERMANENT))
         goto out;
@@ -337,14 +329,13 @@ load_module_init(JSContext  *context,
 }
 
 static void
-load_module_elements(JSContext *context,
-                     JSObject *in_object,
+load_module_elements(JSContext        *context,
+                     JS::HandleObject  in_object,
                      ImporterIterator *iter,
-                     const char *init_path) {
-    JSObject *module_obj;
-    JSObject *jsiter;
-
-    module_obj = load_module_init(context, in_object, init_path);
+                     const char       *init_path)
+{
+    JS::RootedObject jsiter(context),
+        module_obj(context, load_module_init(context, in_object, init_path));
 
     if (module_obj != NULL) {
         jsid idp;
@@ -376,17 +367,16 @@ load_module_elements(JSContext *context,
     }
 }
 
-static JSBool
-import_file_on_module(JSContext  *context,
-                      JSObject   *obj,
-                      const char *name,
-                      GFile      *file)
+static bool
+import_file_on_module(JSContext       *context,
+                      JS::HandleObject obj,
+                      const char      *name,
+                      GFile           *file)
 {
-    JSObject *module_obj;
-    JSBool retval = JS_FALSE;
+    bool retval = false;
     char *full_path = NULL;
 
-    module_obj = create_module_object (context);
+    JS::RootedObject module_obj(context, create_module_object(context));
 
     if (!define_import(context, obj, module_obj, name))
         goto out;
@@ -401,7 +391,7 @@ import_file_on_module(JSContext  *context,
     if (!seal_import(context, obj, name))
         goto out;
 
-    retval = JS_TRUE;
+    retval = true;
 
  out:
     if (!retval)
@@ -411,79 +401,73 @@ import_file_on_module(JSContext  *context,
     return retval;
 }
 
-static JSBool
-do_import(JSContext  *context,
-          JSObject   *obj,
-          Importer   *priv,
-          const char *name)
+static bool
+do_import(JSContext       *context,
+          JS::HandleObject obj,
+          Importer        *priv,
+          const char      *name)
 {
     char *filename;
     char *full_path;
     char *dirname = NULL;
-    jsval search_path_val;
-    JSObject *search_path;
-    JSObject *module_obj = NULL;
+    JS::RootedObject search_path(context);
     guint32 search_path_len;
     guint32 i;
-    JSBool result;
+    bool result;
     GPtrArray *directories;
-    jsid search_path_name;
     GFile *gfile;
-    gboolean exists;
+    bool exists;
 
-    search_path_name = gjs_context_get_const_string(context, GJS_STRING_SEARCH_PATH);
-    if (!gjs_object_require_property(context, obj, "importer", search_path_name, &search_path_val)) {
-        return JS_FALSE;
+    JS::RootedId search_path_name(context,
+        gjs_context_get_const_string(context, GJS_STRING_SEARCH_PATH));
+
+    if (!gjs_object_require_property_value(context, obj, "importer",
+                                           search_path_name, &search_path)) {
+        return false;
     }
 
-    if (!JSVAL_IS_OBJECT(search_path_val)) {
-        gjs_throw(context, "searchPath property on importer is not an object");
-        return JS_FALSE;
-    }
-
-    search_path = JSVAL_TO_OBJECT(search_path_val);
-
     if (!JS_IsArrayObject(context, search_path)) {
         gjs_throw(context, "searchPath property on importer is not an array");
-        return JS_FALSE;
+        return false;
     }
 
     if (!JS_GetArrayLength(context, search_path, &search_path_len)) {
         gjs_throw(context, "searchPath array has no length");
-        return JS_FALSE;
+        return false;
     }
 
-    result = JS_FALSE;
+    result = false;
 
     filename = g_strdup_printf("%s.js", name);
     full_path = NULL;
     directories = NULL;
 
+    JS::RootedValue elem(context);
+    JS::RootedObject module_obj(context);
+
     /* First try importing an internal module like byteArray */
     if (priv->is_root &&
         gjs_is_registered_native_module(context, obj, name) &&
         import_native_file(context, obj, name)) {
         gjs_debug(GJS_DEBUG_IMPORTER,
                   "successfully imported module '%s'", name);
-        result = JS_TRUE;
+        result = true;
         goto out;
     }
 
     for (i = 0; i < search_path_len; ++i) {
-        jsval elem;
-
-        elem = JSVAL_VOID;
-        if (!JS_GetElement(context, search_path, i, &elem)) {
-            /* this means there was an exception, while elem == JSVAL_VOID
+        elem.setUndefined();
+        if (!JS_GetElement(context, search_path, i, elem.address())) {
+            /* this means there was an exception, while elem.isUndefined()
              * means no element found
              */
             goto out;
         }
 
-        if (JSVAL_IS_VOID(elem))
+        if (elem.isUndefined())
             continue;
 
-        if (!JSVAL_IS_STRING(elem)) {
+        if (!elem.isString()) {
             gjs_throw(context, "importer searchPath contains non-string");
             goto out;
         }
@@ -504,20 +488,19 @@ do_import(JSContext  *context,
         full_path = g_build_filename(dirname, MODULE_INIT_FILENAME,
                                      NULL);
 
-        module_obj = load_module_init(context, obj, full_path);
+        module_obj.set(load_module_init(context, obj, full_path));
         if (module_obj != NULL) {
-            jsval obj_val;
-
+            JS::RootedValue obj_val(context);
             if (JS_GetProperty(context,
                                module_obj,
                                name,
-                               &obj_val)) {
-                if (!JSVAL_IS_VOID(obj_val) &&
+                               obj_val.address())) {
+                if (!obj_val.isUndefined() &&
                     JS_DefineProperty(context, obj,
                                       name, obj_val,
                                       NULL, NULL,
                                       GJS_MODULE_PROP_FLAGS & ~JSPROP_PERMANENT)) {
-                    result = JS_TRUE;
+                    result = true;
                     goto out;
                 }
             }
@@ -573,7 +556,7 @@ do_import(JSContext  *context,
         if (import_file_on_module (context, obj, name, gfile)) {
             gjs_debug(GJS_DEBUG_IMPORTER,
                       "successfully imported module '%s'", name);
-            result = JS_TRUE;
+            result = true;
         }
 
         g_object_unref(gfile);
@@ -593,7 +576,7 @@ do_import(JSContext  *context,
                              (const char**) directories->pdata)) {
             gjs_debug(GJS_DEBUG_IMPORTER,
                       "successfully imported directory '%s'", name);
-            result = JS_TRUE;
+            result = true;
         }
     }
 
@@ -607,7 +590,7 @@ do_import(JSContext  *context,
         g_ptr_array_add(directories, NULL);
 
         str_array = (char**) directories->pdata;
-        g_ptr_array_free(directories, FALSE);
+        g_ptr_array_free(directories, false);
         g_strfreev(str_array);
     }
 
@@ -643,7 +626,7 @@ static void
 importer_iterator_free(ImporterIterator *iter)
 {
     g_ptr_array_foreach(iter->elements, (GFunc)g_free, NULL);
-    g_ptr_array_free(iter->elements, TRUE);
+    g_ptr_array_free(iter->elements, true);
     g_slice_free(ImporterIterator, iter);
 }
 
@@ -653,14 +636,14 @@ importer_iterator_free(ImporterIterator *iter)
  * JSENUMERATE_INIT: allocate private enum struct in state_p, return number
  * of elements in *id_p
  * JSENUMERATE_NEXT: return next property id in *id_p, and if no new property
- * free state_p and set to JSVAL_NULL
+ * free state_p and set to JS::NullValue()
  * JSENUMERATE_DESTROY : destroy state_p
  *
  * Note that in a for ... in loop, this will be called first on the object,
  * then on its prototype.
  *
  */
-static JSBool
+static bool
 importer_new_enumerate(JSContext  *context,
                        JS::HandleObject object,
                        JSIterateOp enum_op,
@@ -673,13 +656,11 @@ importer_new_enumerate(JSContext  *context,
     case JSENUMERATE_INIT_ALL:
     case JSENUMERATE_INIT: {
         Importer *priv;
-        JSObject *search_path;
-        jsval search_path_val;
+        JS::RootedObject search_path(context);
         guint32 search_path_len;
         guint32 i;
-        jsid search_path_name;
 
-        statep.set(JSVAL_NULL);
+        statep.setNull();
 
         idp.set(INT_TO_JSID(0));
 
@@ -687,59 +668,54 @@ importer_new_enumerate(JSContext  *context,
 
         if (!priv)
             /* we are enumerating the prototype properties */
-            return JS_TRUE;
+            return true;
 
-        search_path_name = gjs_context_get_const_string(context, GJS_STRING_SEARCH_PATH);
-        if (!gjs_object_require_property(context, object, "importer", search_path_name, &search_path_val))
-            return JS_FALSE;
-
-        if (!JSVAL_IS_OBJECT(search_path_val)) {
-            gjs_throw(context, "searchPath property on importer is not an object");
-            return JS_FALSE;
-        }
-
-        search_path = JSVAL_TO_OBJECT(search_path_val);
+        JS::RootedId search_path_name(context,
+            gjs_context_get_const_string(context, GJS_STRING_SEARCH_PATH));
+        if (!gjs_object_require_property_value(context, object, "importer",
+                                               search_path_name, &search_path))
+            return false;
 
         if (!JS_IsArrayObject(context, search_path)) {
             gjs_throw(context, "searchPath property on importer is not an array");
-            return JS_FALSE;
+            return false;
         }
 
         if (!JS_GetArrayLength(context, search_path, &search_path_len)) {
             gjs_throw(context, "searchPath array has no length");
-            return JS_FALSE;
+            return false;
         }
 
         iter = importer_iterator_new();
 
+        JS::RootedValue elem(context);
         for (i = 0; i < search_path_len; ++i) {
             char *dirname = NULL;
             char *init_path;
             const char *filename;
-            jsval elem;
             GDir *dir = NULL;
 
-            elem = JSVAL_VOID;
-            if (!JS_GetElement(context, search_path, i, &elem)) {
-                /* this means there was an exception, while elem == JSVAL_VOID
+            elem = JS::UndefinedValue();
+            if (!JS_GetElement(context, search_path, i, elem.address())) {
+                /* this means there was an exception, while elem.isUndefined()
                  * means no element found
                  */
                 importer_iterator_free(iter);
-                return JS_FALSE;
+                return false;
             }
 
-            if (JSVAL_IS_VOID(elem))
+            if (elem.isUndefined())
                 continue;
 
-            if (!JSVAL_IS_STRING(elem)) {
+            if (!elem.isString()) {
                 gjs_throw(context, "importer searchPath contains non-string");
                 importer_iterator_free(iter);
-                return JS_FALSE;
+                return false;
             }
 
             if (!gjs_string_to_utf8(context, elem, &dirname)) {
                 importer_iterator_free(iter);
-                return JS_FALSE; /* Error message already set */
+                return false; /* Error message already set */
             }
 
             init_path = g_build_filename(dirname, MODULE_INIT_FILENAME,
@@ -786,7 +762,7 @@ importer_new_enumerate(JSContext  *context,
             g_free(dirname);
         }
 
-        statep.set(PRIVATE_TO_JSVAL(iter));
+        statep.get().setPrivate(iter);
 
         idp.set(INT_TO_JSID(iter->elements->len));
 
@@ -794,25 +770,22 @@ importer_new_enumerate(JSContext  *context,
     }
 
     case JSENUMERATE_NEXT: {
-        jsval element_val;
+        if (statep.isNull()) /* Iterating prototype */
+            return true;
 
-        if (JSVAL_IS_NULL(statep)) /* Iterating prototype */
-            return JS_TRUE;
-
-        iter = (ImporterIterator*) JSVAL_TO_PRIVATE(statep);
+        iter = (ImporterIterator*) statep.get().toPrivate();
 
         if (iter->index < iter->elements->len) {
+            JS::RootedValue element_val(context);
             if (!gjs_string_from_utf8(context,
                                          (const char*) g_ptr_array_index(iter->elements,
                                                            iter->index++),
                                          -1,
                                          &element_val))
-                return JS_FALSE;
+                return false;
 
-            jsid id;
-            if (!JS_ValueToId(context, element_val, &id))
-                return JS_FALSE;
-            idp.set(id);
+            if (!JS_ValueToId(context, element_val, idp.address()))
+                return false;
 
             break;
         }
@@ -820,17 +793,20 @@ importer_new_enumerate(JSContext  *context,
     }
 
     case JSENUMERATE_DESTROY: {
-        if (!JSVAL_IS_NULL(statep)) {
-            iter = (ImporterIterator*) JSVAL_TO_PRIVATE(statep);
+        if (!statep.isNull()) {
+            iter = (ImporterIterator*) statep.get().toPrivate();
 
             importer_iterator_free(iter);
 
-            statep.set(JSVAL_NULL);
+            statep.setNull();
         }
     }
+
+    default:
+        ;
     }
 
-    return JS_TRUE;
+    return true;
 }
 
 /*
@@ -846,7 +822,7 @@ importer_new_enumerate(JSContext  *context,
  * was not resolved; and non-null, referring to obj or one of its prototypes,
  * if id was resolved.
  */
-static JSBool
+static bool
 importer_new_resolve(JSContext *context,
                      JS::HandleObject obj,
                      JS::HandleId id,
@@ -855,15 +831,15 @@ importer_new_resolve(JSContext *context,
 {
     Importer *priv;
     char *name;
-    JSBool ret = JS_TRUE;
+    bool ret = true;
     jsid module_init_name;
 
     module_init_name = gjs_context_get_const_string(context, GJS_STRING_MODULE_INIT);
     if (id == module_init_name)
-        return JS_TRUE;
+        return true;
 
     if (!gjs_get_string_id(context, id, &name))
-        return JS_FALSE;
+        return false;
 
     /* let Object.prototype resolve these */
     if (strcmp(name, "valueOf") == 0 ||
@@ -874,14 +850,14 @@ importer_new_resolve(JSContext *context,
 
     gjs_debug_jsprop(GJS_DEBUG_IMPORTER,
                      "Resolve prop '%s' hook obj %p priv %p",
-                     name, (void *)obj, priv);
+                     name, obj.get(), priv);
     if (priv == NULL) /* we are the prototype, or have the wrong class */
         goto out;
     JS_BeginRequest(context);
     if (do_import(context, obj, priv, name)) {
         objp.set(obj);
     } else {
-        ret = JS_FALSE;
+        ret = false;
     }
     JS_EndRequest(context);
 
@@ -929,23 +905,21 @@ struct JSClass gjs_importer_class = {
 };
 
 JSPropertySpec gjs_importer_proto_props[] = {
-    { NULL }
+    JS_PS_END
 };
 
 JSFunctionSpec gjs_importer_proto_funcs[] = {
-    { NULL }
+    JS_FS_END
 };
 
 static JSObject*
 importer_new(JSContext *context,
-             gboolean   is_root)
+             bool       is_root)
 {
-    JSObject *importer;
     Importer *priv;
-    JSObject *global;
     JSBool found;
 
-    global = gjs_get_import_global(context);
+    JS::RootedObject global(context, gjs_get_import_global(context));
 
     if (!JS_HasProperty(context, global, gjs_importer_class.name, &found))
         g_error("HasProperty call failed creating importer class");
@@ -981,7 +955,8 @@ importer_new(JSContext *context,
                   gjs_importer_class.name, prototype);
     }
 
-    importer = JS_NewObject(context, &gjs_importer_class, NULL, global);
+    JS::RootedObject importer(context,
+        JS_NewObject(context, &gjs_importer_class, NULL, global));
     if (importer == NULL)
         g_error("No memory to create importer importer");
 
@@ -994,7 +969,8 @@ importer_new(JSContext *context,
     JS_SetPrivate(importer, priv);
 
     gjs_debug_lifecycle(GJS_DEBUG_IMPORTER,
-                        "importer constructor, obj %p priv %p", importer, priv);
+                        "importer constructor, obj %p priv %p", importer.get(),
+                        priv);
 
     return importer;
 }
@@ -1042,7 +1018,7 @@ gjs_get_search_path(void)
 
         g_ptr_array_add(path, NULL);
 
-        search_path = (char**)g_ptr_array_free(path, FALSE);
+        search_path = (char**)g_ptr_array_free(path, false);
 
         gjs_search_path = search_path;
     } else {
@@ -1053,14 +1029,13 @@ gjs_get_search_path(void)
 }
 
 static JSObject*
-gjs_create_importer(JSContext    *context,
-                    const char   *importer_name,
-                    const char  **initial_search_path,
-                    gboolean      add_standard_search_path,
-                    gboolean      is_root,
-                    JSObject     *in_object)
+gjs_create_importer(JSContext       *context,
+                    const char      *importer_name,
+                    const char     **initial_search_path,
+                    bool             add_standard_search_path,
+                    bool             is_root,
+                    JS::HandleObject in_object)
 {
-    JSObject *importer;
     char **paths[2] = {0};
     char **search_path;
 
@@ -1072,7 +1047,7 @@ gjs_create_importer(JSContext    *context,
 
     search_path = gjs_g_strv_concat(paths, 2);
 
-    importer = importer_new(context, is_root);
+    JS::RootedObject importer(context, importer_new(context, is_root));
 
     /* API users can replace this property from JS, is the idea */
     if (!gjs_define_string_array(context, importer,
@@ -1090,25 +1065,27 @@ gjs_create_importer(JSContext    *context,
 }
 
 JSObject*
-gjs_define_importer(JSContext    *context,
-                    JSObject     *in_object,
-                    const char   *importer_name,
-                    const char  **initial_search_path,
-                    gboolean      add_standard_search_path)
+gjs_define_importer(JSContext       *context,
+                    JS::HandleObject in_object,
+                    const char      *importer_name,
+                    const char     **initial_search_path,
+                    bool             add_standard_search_path)
 
 {
     JSObject *importer;
 
-    importer = gjs_create_importer(context, importer_name, initial_search_path, add_standard_search_path, FALSE, in_object);
+    importer = gjs_create_importer(context, importer_name, initial_search_path,
+                                   add_standard_search_path, false, in_object);
 
-    if (!JS_DefineProperty(context, in_object,
-                           importer_name, OBJECT_TO_JSVAL(importer),
+    JS::RootedValue v_importer(context, JS::ObjectValue(*importer));
+    if (!JS_DefineProperty(context, in_object, importer_name, v_importer,
                            NULL, NULL,
                            GJS_MODULE_PROP_FLAGS))
         g_error("no memory to define importer property");
 
     gjs_debug(GJS_DEBUG_IMPORTER,
-              "Defined importer '%s' %p in %p", importer_name, importer, in_object);
+              "Defined importer '%s' %p in %p", importer_name, importer,
+              in_object.get());
 
     return importer;
 }
@@ -1117,65 +1094,65 @@ gjs_define_importer(JSContext    *context,
  * would basically be a bug, but checking for that is a lot of code so
  * we just ignore all calls after the first and hope the args are the same.
  */
-JSBool
+bool
 gjs_create_root_importer(JSContext   *context,
                          const char **initial_search_path,
-                         gboolean     add_standard_search_path)
+                         bool         add_standard_search_path)
 {
-    jsval importer;
+    JS::Value importer;
 
     JS_BeginRequest(context);
 
     importer = gjs_get_global_slot(context, GJS_GLOBAL_SLOT_IMPORTS);
 
-    if (G_UNLIKELY (!JSVAL_IS_VOID(importer))) {
+    if (G_UNLIKELY (!importer.isUndefined())) {
         gjs_debug(GJS_DEBUG_IMPORTER,
                   "Someone else already created root importer, ignoring second request");
 
         JS_EndRequest(context);
-        return JS_TRUE;
+        return true;
     }
 
-    importer = OBJECT_TO_JSVAL(gjs_create_importer(context, "imports",
-                                                   initial_search_path,
-                                                   add_standard_search_path,
-                                                   TRUE, NULL));
+    importer = JS::ObjectValue(*gjs_create_importer(context, "imports",
+                                                    initial_search_path,
+                                                    add_standard_search_path,
+                                                    true, JS::NullPtr()));
     gjs_set_global_slot(context, GJS_GLOBAL_SLOT_IMPORTS, importer);
 
     JS_EndRequest(context);
-    return JS_TRUE;
+    return true;
 }
 
-JSBool
+bool
 gjs_define_root_importer_object(JSContext        *context,
                                 JS::HandleObject  in_object,
                                 JS::HandleObject  root_importer)
 {
-    JSBool success;
+    bool success;
     jsid imports_name;
 
-    success = JS_FALSE;
+    success = false;
     JS_BeginRequest(context);
 
     JS::RootedValue importer (JS_GetRuntime(context),
-                              OBJECT_TO_JSVAL(root_importer));
+                              JS::ObjectValue(*root_importer));
     imports_name = gjs_context_get_const_string(context, GJS_STRING_IMPORTS);
     if (!JS_DefinePropertyById(context, in_object,
                                imports_name, importer,
                                NULL, NULL,
                                GJS_MODULE_PROP_FLAGS)) {
         gjs_debug(GJS_DEBUG_IMPORTER, "DefineProperty imports on %p failed",
-                  (JSObject *) in_object);
+                  in_object.get());
         goto fail;
     }
 
-    success = JS_TRUE;
+    success = true;
  fail:
     JS_EndRequest(context);
     return success;
 }
 
-JSBool
+bool
 gjs_define_root_importer(JSContext   *context,
                          JSObject    *in_object)
 {
@@ -1184,7 +1161,7 @@ gjs_define_root_importer(JSContext   *context,
     JS::RootedObject rooted_in_object(JS_GetRuntime(context),
                                       in_object);
     JS::RootedObject rooted_importer(JS_GetRuntime(context),
-                                     JSVAL_TO_OBJECT(importer));
+                                     &importer.toObject());
     return gjs_define_root_importer_object(context,
                                            rooted_in_object,
                                            rooted_importer);
