@@ -29,12 +29,13 @@
 
 #include "jsapi-util.h"
 #include "jsapi-private.h"
-#include "compat.h"
+#include "jsapi-wrapper.h"
 
 #include <string.h>
 #pragma GCC diagnostic push
 #pragma GCC diagnostic ignored "-Wstrict-prototypes"
 #pragma GCC diagnostic ignored "-Winvalid-offsetof"
+#pragma GCC diagnostic ignored "-Wredundant-decls"
 #include <jsfriendapi.h>
 #pragma GCC diagnostic pop
 
@@ -59,13 +60,12 @@ gjs_error_reporter(JSContext     *context,
         level = G_LOG_LEVEL_MESSAGE;
 
         /* suppress bogus warnings. See mozilla/js/src/js.msg */
-        switch (report->errorNumber) {
+        if (report->errorNumber == 162) {
             /* 162, JSMSG_UNDEFINED_PROP: warns every time a lazy property
              * is resolved, since the property starts out
              * undefined. When this is a real bug it should usually
              * fail somewhere else anyhow.
              */
-        case 162:
             return;
         }
     } else {
