@@ -24,7 +24,7 @@
 #include <config.h>
 
 #include "jsapi-util.h"
-#include "compat.h"
+#include "jsapi-wrapper.h"
 #include "gi/gerror.h"
 
 #include <util/log.h>
@@ -42,19 +42,18 @@
  * http://egachine.berlios.de/embedding-sm-best-practice/embedding-sm-best-practice.html#error-handling
  */
 static void
+G_GNUC_PRINTF(3, 0)
 gjs_throw_valist(JSContext       *context,
                  const char      *error_class,
                  const char      *format,
                  va_list          args)
 {
     char *s;
-    JSBool result;
-    jsval v_constructor, v_message;
-    JSObject *err_obj;
+    bool result;
 
     s = g_strdup_vprintf(format, args);
 
-    JSAutoCompartment compartment(context, JS_GetGlobalObject(context));
+    JSAutoCompartment compartment(context, gjs_get_import_global(context));
 
     JS_BeginRequest(context);
 
@@ -75,24 +74,28 @@ gjs_throw_valist(JSContext       *context,
         return;
     }
 
-    result = JS_FALSE;
+    JS::RootedObject constructor(context);
+    JS::RootedObject global(context, JS_GetGlobalForScopeChain(context));
+    JS::RootedValue v_constructor(context), v_message(context), new_exc(context);
+    result = false;
 
     if (!gjs_string_from_utf8(context, s, -1, &v_message)) {
         JS_ReportError(context, "Failed to copy exception string");
         goto out;
     }
 
-    if (!JS_GetProperty(context, JS_GetGlobalForScopeChain(context), error_class, &v_constructor) ||
-        !JSVAL_IS_OBJECT(v_constructor)) {
+    if (!JS_GetProperty(context, global, error_class, v_constructor.address()) ||
+        !v_constructor.isObject()) {
         JS_ReportError(context, "??? Missing Error constructor in global object?");
         goto out;
     }
 
     /* throw new Error(message) */
-    err_obj = JS_New(context, JSVAL_TO_OBJECT(v_constructor), 1, &v_message);
-    JS_SetPendingException(context, OBJECT_TO_JSVAL(err_obj));
+    constructor = &v_constructor.toObject();
+    new_exc.setObjectOrNull(JS_New(context, constructor, 1, v_message.address()));
+    JS_SetPendingException(context, new_exc);
 
-    result = JS_TRUE;
+    result = true;
 
  out:
 
@@ -171,17 +174,16 @@ void
 gjs_throw_g_error (JSContext       *context,
                    GError          *error)
 {
-    JSObject *err_obj;
-
     if (error == NULL)
         return;
 
     JS_BeginRequest(context);
 
-    err_obj = gjs_error_from_gerror(context, error, TRUE);
+    JS::RootedValue err(context,
+        JS::ObjectOrNullValue(gjs_error_from_gerror(context, error, true)));
     g_error_free (error);
-    if (err_obj)
-        JS_SetPendingException(context, OBJECT_TO_JSVAL(err_obj));
+    if (!err.isNull())
+        JS_SetPendingException(context, err);
 
     JS_EndRequest(context);
 }
