@@ -26,32 +26,31 @@
 #include <string.h>
 
 #include "jsapi-util.h"
-#include "compat.h"
+#include "jsapi-wrapper.h"
 
-gboolean
-gjs_string_to_utf8 (JSContext  *context,
-                    const jsval value,
-                    char      **utf8_string_p)
+JSBool
+gjs_string_to_utf8 (JSContext      *context,
+                    const JS::Value value,
+                    char          **utf8_string_p)
 {
-    JSString *str;
     gsize len;
     char *bytes;
 
     JS_BeginRequest(context);
 
-    if (!JSVAL_IS_STRING(value)) {
+    if (!value.isString()) {
         gjs_throw(context,
                   "Value is not a string, cannot convert to UTF-8");
         JS_EndRequest(context);
-        return JS_FALSE;
+        return false;
     }
 
-    str = JSVAL_TO_STRING(value);
+    JS::RootedString str(context, value.toString());
 
     len = JS_GetStringEncodingLength(context, str);
     if (len == (gsize)(-1)) {
         JS_EndRequest(context);
-        return JS_FALSE;
+        return false;
     }
 
     if (utf8_string_p) {
@@ -61,18 +60,17 @@ gjs_string_to_utf8 (JSContext  *context,
 
     JS_EndRequest(context);
 
-    return JS_TRUE;
+    return true;
 }
 
-JSBool
-gjs_string_from_utf8(JSContext  *context,
-                     const char *utf8_string,
-                     gssize      n_bytes,
-                     jsval      *value_p)
+bool
+gjs_string_from_utf8(JSContext             *context,
+                     const char            *utf8_string,
+                     ssize_t                n_bytes,
+                     JS::MutableHandleValue value_p)
 {
     jschar *u16_string;
     glong u16_string_length;
-    JSString *str;
     GError *error;
 
     /* intentionally using n_bytes even though glib api suggests n_chars; with
@@ -91,25 +89,25 @@ gjs_string_from_utf8(JSContext  *context,
                   "JS string: %s",
                   error->message);
                   g_error_free(error);
-        return JS_FALSE;
+        return false;
     }
 
     JS_BeginRequest(context);
 
     /* Avoid a copy - assumes that g_malloc == js_malloc == malloc */
-    str = JS_NewUCString(context, u16_string, u16_string_length);
-
-    if (str && value_p)
-        *value_p = STRING_TO_JSVAL(str);
+    JS::RootedString str(context,
+                         JS_NewUCString(context, u16_string, u16_string_length));
+    if (str)
+        value_p.setString(str);
 
     JS_EndRequest(context);
     return str != NULL;
 }
 
-gboolean
-gjs_string_to_filename(JSContext    *context,
-                       const jsval   filename_val,
-                       char        **filename_string_p)
+JSBool
+gjs_string_to_filename(JSContext      *context,
+                       const JS::Value filename_val,
+                       char          **filename_string_p)
 {
     GError *error;
     gchar *tmp, *filename_string;
@@ -118,7 +116,7 @@ gjs_string_to_filename(JSContext    *context,
 
     if (!gjs_string_to_utf8(context, filename_val, &tmp)) {
         /* exception already set */
-        return JS_FALSE;
+        return false;
     }
 
     error = NULL;
@@ -126,19 +124,19 @@ gjs_string_to_filename(JSContext    *context,
     if (!filename_string) {
         gjs_throw_g_error(context, error);
         g_free(tmp);
-        return FALSE;
+        return false;
     }
 
     *filename_string_p = filename_string;
     g_free(tmp);
-    return TRUE;
+    return true;
 }
 
-JSBool
-gjs_string_from_filename(JSContext  *context,
-                         const char *filename_string,
-                         gssize      n_bytes,
-                         jsval      *value_p)
+bool
+gjs_string_from_filename(JSContext             *context,
+                         const char            *filename_string,
+                         ssize_t                n_bytes,
+                         JS::MutableHandleValue value_p)
 {
     gsize written;
     GError *error;
@@ -154,21 +152,21 @@ gjs_string_from_filename(JSContext  *context,
                   error->message);
         g_error_free(error);
         g_free(utf8_string);
-        return JS_FALSE;
+        return false;
     }
 
     if (!gjs_string_from_utf8(context, utf8_string, written, value_p))
-        return JS_FALSE;
+        return false;
 
     g_free(utf8_string);
 
-    return JS_TRUE;
+    return true;
 }
 
 /**
  * gjs_string_get_uint16_data:
  * @context: js context
- * @value: a jsval
+ * @value: a JS::Value
  * @data_p: address to return allocated data buffer
  * @len_p: address to return length of data (number of 16-bit integers)
  *
@@ -176,37 +174,129 @@ gjs_string_from_filename(JSContext  *context,
  * contained in @value.
  * Throws a JS exception if value is not a string.
  *
- * Returns: JS_FALSE if exception thrown
+ * Returns: false if exception thrown
  **/
-JSBool
+bool
 gjs_string_get_uint16_data(JSContext       *context,
-                           jsval            value,
+                           JS::Value        value,
                            guint16        **data_p,
                            gsize           *len_p)
 {
     const jschar *js_data;
-    JSBool retval = JS_FALSE;
+    bool retval = false;
 
     JS_BeginRequest(context);
 
-    if (!JSVAL_IS_STRING(value)) {
+    if (!value.isString()) {
         gjs_throw(context,
                   "Value is not a string, can't return binary data from it");
         goto out;
     }
 
-    js_data = JS_GetStringCharsAndLength(context, JSVAL_TO_STRING(value), len_p);
+    js_data = JS_GetStringCharsAndLength(context, value.toString(), len_p);
     if (js_data == NULL)
         goto out;
 
     *data_p = (guint16*) g_memdup(js_data, sizeof(*js_data)*(*len_p));
 
-    retval = JS_TRUE;
+    retval = true;
 out:
     JS_EndRequest(context);
     return retval;
 }
 
+/**
+ * gjs_string_to_ucs4:
+ * @cx: a #JSContext
+ * @value: JS::Value containing a string
+ * @ucs4_string_p: return location for a #gunichar array
+ * @len_p: return location for @ucs4_string_p length
+ *
+ * Returns: true on success, false otherwise in which case a JS error is thrown
+ */
+bool
+gjs_string_to_ucs4(JSContext      *cx,
+                   JS::HandleValue value,
+                   gunichar      **ucs4_string_p,
+                   size_t         *len_p)
+{
+    if (ucs4_string_p == NULL)
+        return true;
+
+    if (!value.isString()) {
+        gjs_throw(cx, "Value is not a string, cannot convert to UCS-4");
+        return false;
+    }
+
+    JSAutoRequest ar(cx);
+    JS::RootedString str(cx, value.toString());
+    size_t utf16_len;
+    GError *error = NULL;
+
+    const jschar *utf16 = JS_GetStringCharsAndLength(cx, str, &utf16_len);
+    if (utf16 == NULL) {
+        gjs_throw(cx, "Failed to get UTF-16 string data");
+        return false;
+    }
+
+    if (ucs4_string_p != NULL) {
+        long length;
+        *ucs4_string_p = g_utf16_to_ucs4(reinterpret_cast<const gunichar2 *>(utf16),
+                                         utf16_len, NULL, &length, &error);
+        if (*ucs4_string_p == NULL) {
+            gjs_throw(cx, "Failed to convert UTF-16 string to UCS-4: %s",
+                      error->message);
+            g_clear_error(&error);
+            return false;
+        }
+        if (len_p != NULL)
+            *len_p = (size_t) length;
+    }
+
+    return true;
+}
+
+/**
+ * gjs_string_from_ucs4:
+ * @cx: a #JSContext
+ * @ucs4_string: string of #gunichar
+ * @n_chars: number of characters in @ucs4_string or -1 for zero-terminated
+ * @value_p: JS::Value that will be filled with a string
+ *
+ * Returns: true on success, false otherwise in which case a JS error is thrown
+ */
+bool
+gjs_string_from_ucs4(JSContext             *cx,
+                     const gunichar        *ucs4_string,
+                     ssize_t                n_chars,
+                     JS::MutableHandleValue value_p)
+{
+    uint16_t *u16_string;
+    long u16_string_length;
+    GError *error = NULL;
+
+    u16_string = g_ucs4_to_utf16(ucs4_string, n_chars, NULL,
+                                 &u16_string_length, &error);
+    if (!u16_string) {
+        gjs_throw(cx, "Failed to convert UCS-4 string to UTF-16: %s",
+                  error->message);
+        g_error_free(error);
+        return false;
+    }
+
+    JSAutoRequest ar(cx);
+    /* Avoid a copy - assumes that g_malloc == js_malloc == malloc */
+    JS::RootedString str(cx, JS_NewUCString(cx, u16_string, u16_string_length));
+
+    if (str == NULL) {
+        gjs_throw(cx, "Failed to convert UCS-4 string to UTF-16");
+        return false;
+    }
+
+    value_p.setString(str);
+    return true;
+}
+
 /**
  * gjs_get_string_id:
  * @context: a #JSContext
@@ -218,21 +308,21 @@ out:
  *
  * Returns: true if *name_p is non-%NULL
  **/
-JSBool
+bool
 gjs_get_string_id (JSContext       *context,
                    jsid             id,
                    char           **name_p)
 {
-    jsval id_val;
+    JS::RootedValue id_val(context);
 
-    if (!JS_IdToValue(context, id, &id_val))
-        return JS_FALSE;
+    if (!JS_IdToValue(context, id, id_val.address()))
+        return false;
 
-    if (JSVAL_IS_STRING(id_val)) {
+    if (id_val.isString()) {
         return gjs_string_to_utf8(context, id_val, name_p);
     } else {
         *name_p = NULL;
-        return JS_FALSE;
+        return false;
     }
 }
 
@@ -248,18 +338,18 @@ gjs_get_string_id (JSContext       *context,
  * If @string is empty, @result will be 0.  An exception will
  * be thrown if @string can not be represented as UTF-8.
  */
-gboolean
+bool
 gjs_unichar_from_string (JSContext *context,
-                         jsval      value,
+                         JS::Value  value,
                          gunichar  *result)
 {
     char *utf8_str;
     if (gjs_string_to_utf8(context, value, &utf8_str)) {
         *result = g_utf8_get_char(utf8_str);
         g_free(utf8_str);
-        return TRUE;
+        return true;
     }
-    return FALSE;
+    return false;
 }
 
 jsid
