@@ -24,7 +24,6 @@
 #include <config.h>
 
 #include "mem.h"
-#include "compat.h"
 #include <util/log.h>
 
 #define GJS_DEFINE_COUNTER(name)             \
@@ -72,7 +71,7 @@ static GjsMemCounter* counters[] = {
 
 void
 gjs_memory_report(const char *where,
-                  gboolean    die_if_leaks)
+                  bool        die_if_leaks)
 {
     int i;
     int n_counters;
