@@ -26,19 +26,22 @@
 #include <glib.h>
 #include <glib/gstdio.h>
 #include <girepository.h>
-#include <gjs/gjs-module.h>
-#include <gjs/coverage.h>
 #include <locale.h>
 
 #include <string.h>
 
+#include <gjs/gjs.h>
+
+#include "gjs/coverage.h"
+#include "gjs/mem.h"
+
 typedef struct {
     const char *coverage_prefix;
     const char *coverage_output_path;
     char       *filename;
 } GjsTestData;
 
-GjsTestData *
+static GjsTestData *
 gjs_unit_test_data_new(const char *coverage_prefix,
                        const char *coverage_output_path,
                        char       *filename)
@@ -50,7 +53,7 @@ gjs_unit_test_data_new(const char *coverage_prefix,
     return data;
 }
 
-void
+static void
 gjs_unit_test_data_free(gpointer test_data, gpointer user_data)
 {
     GjsTestData *data = (GjsTestData *) test_data;
@@ -100,9 +103,9 @@ teardown(GjsTestJSFixture *fix,
         g_clear_object(&fix->coverage);
     }
 
-    gjs_memory_report("before destroying context", FALSE);
+    gjs_memory_report("before destroying context", false);
     g_object_unref(fix->context);
-    gjs_memory_report("after destroying context", TRUE);
+    gjs_memory_report("after destroying context", true);
 }
 
 static void
@@ -110,7 +113,7 @@ test(GjsTestJSFixture *fix,
      gconstpointer     test_data)
 {
     GError *error = NULL;
-    gboolean success;
+    bool success;
     int code;
 
     GjsTestData *data = (GjsTestData *) test_data;
@@ -154,9 +157,9 @@ main(int argc, char **argv)
      * we leak objects.
      * https://bugzilla.gnome.org/show_bug.cgi?id=616193
      */
-    g_setenv("GJS_DISABLE_JIT", "1", FALSE);
+    g_setenv("GJS_DISABLE_JIT", "1", false);
     /* The fact that this isn't the default is kind of lame... */
-    g_setenv("GJS_DEBUG_OUTPUT", "stderr", FALSE);
+    g_setenv("GJS_DEBUG_OUTPUT", "stderr", false);
 
     setlocale(LC_ALL, "");
     g_test_init(&argc, &argv, NULL);
