@@ -69,6 +69,17 @@ function testCArray() {
     assertEquals(1, array[2]);
     assertEquals(5, array[3]);
 
+    array = GIMarshallingTests.array_bool_out();
+    assertEquals(4, array.length);
+    assertEquals(true, array[0]);
+    assertEquals(false, array[1]);
+    assertEquals(true, array[2]);
+    assertEquals(true, array[3]);
+
+    assertEquals('const \u2665 utf8', GIMarshallingTests.array_unichar_out());
+    assertEquals('const \u2665 utf8',
+        GIMarshallingTests.array_zero_terminated_return_unichar());
+
     array = GIMarshallingTests.array_inout([-1, 0, 1, 2]);
     assertEquals(5, array.length);
     assertEquals(-2, array[0]);
@@ -101,6 +112,9 @@ function testCArray() {
     GIMarshallingTests.array_struct_take_in(array);
 
     GIMarshallingTests.array_uint8_in ("abcd");
+    GIMarshallingTests.array_unichar_in('const \u2665 utf8');
+    GIMarshallingTests.array_unichar_in([0x63, 0x6f, 0x6e, 0x73, 0x74, 0x20,
+        0x2665, 0x20, 0x75, 0x74, 0x66, 0x38]);
     GIMarshallingTests.array_enum_in([GIMarshallingTests.Enum.VALUE1,
 				      GIMarshallingTests.Enum.VALUE2,
 				      GIMarshallingTests.Enum.VALUE3]);
@@ -111,6 +125,9 @@ function testCArray() {
     GIMarshallingTests.array_in_len_zero_terminated(array);
     GIMarshallingTests.array_in_guint64_len(array);
     GIMarshallingTests.array_in_guint8_len(array);
+    GIMarshallingTests.array_int64_in(array);
+    GIMarshallingTests.array_uint64_in(array);
+    GIMarshallingTests.array_bool_in(array);
 }
 
 function testGArray() {
@@ -135,6 +152,10 @@ function testGArray() {
 
     GIMarshallingTests.garray_int_none_in([-1, 0, 1, 2]);
     GIMarshallingTests.garray_utf8_none_in(["0", "1", "2"]);
+    GIMarshallingTests.garray_bool_none_in([-1, 0, 1, 2]);
+    GIMarshallingTests.garray_unichar_none_in('const \u2665 utf8');
+    GIMarshallingTests.garray_unichar_none_in([0x63, 0x6f, 0x6e, 0x73, 0x74,
+        0x20, 0x2665, 0x20, 0x75, 0x74, 0x66, 0x38]);
 
     array = GIMarshallingTests.garray_utf8_none_out();
     assertEquals("0", array[0]);
@@ -222,6 +243,54 @@ function testPtrArray() {
     assertArrayEquals(refArray, GIMarshallingTests.gptrarray_utf8_full_out());
 }
 
+function testGHashTable() {
+    function dictEquals(dict, ref) {
+        let dict_keys = Object.keys(dict);
+        let ref_keys = Object.keys(ref);
+        assertEquals(ref_keys.length, dict_keys.length);
+        ref_keys.forEach((key, ix) => {
+            assertEquals(key, dict_keys[ix]);
+            assertEquals(ref[key], dict[key]);
+        });
+    }
+
+    let INT_DICT = {
+        '-1': 1,
+        0: 0,
+        1: -1,
+        2: -2,
+    };
+    const STRING_DICT = {
+        '-1': '1',
+        0: '0',
+        1: '-1',
+        2: '-2',
+    };
+    const STRING_DICT_OUT = {
+        '-1': '1',
+        0: '0',
+        1: '1',
+    };
+
+    dictEquals(GIMarshallingTests.ghashtable_int_none_return(), INT_DICT);
+    dictEquals(GIMarshallingTests.ghashtable_utf8_none_return(), STRING_DICT);
+    dictEquals(GIMarshallingTests.ghashtable_utf8_container_return(), STRING_DICT);
+    dictEquals(GIMarshallingTests.ghashtable_utf8_full_return(), STRING_DICT);
+
+    GIMarshallingTests.ghashtable_int_none_in(INT_DICT);
+    GIMarshallingTests.ghashtable_utf8_none_in(STRING_DICT);
+
+    dictEquals(GIMarshallingTests.ghashtable_utf8_none_out(), STRING_DICT);
+    dictEquals(GIMarshallingTests.ghashtable_utf8_container_out(), STRING_DICT);
+    dictEquals(GIMarshallingTests.ghashtable_utf8_full_out(), STRING_DICT);
+
+    dictEquals(GIMarshallingTests.ghashtable_utf8_none_inout(STRING_DICT), STRING_DICT_OUT);
+    // FIXME: Container transfer for in parameters not supported
+    //dictEquals(GIMarshallingTests.ghashtable_utf8_container_inout(STRING_DICT), STRING_DICT_OUT);
+    // FIXME: Broken
+    //dictEquals(GIMarshallingTests.ghashtable_utf8_full_inout(STRING_DICT), STRING_DICT_OUT);
+}
+
 function testGValue() {
     assertEquals(42, GIMarshallingTests.gvalue_return());
     assertEquals(42, GIMarshallingTests.gvalue_out());
