@@ -22,8 +22,8 @@
 
 #include <config.h>
 
-#include <gjs/gjs-module.h>
-#include <gjs/compat.h>
+#include "gjs/jsapi-util.h"
+#include "gjs/jsapi-wrapper.h"
 #include <cairo.h>
 #include "cairo-private.h"
 
@@ -50,11 +50,11 @@ gjs_cairo_path_finalize(JSFreeOp *fop,
 
 /* Properties */
 JSPropertySpec gjs_cairo_path_proto_props[] = {
-    { NULL }
+    JS_PS_END
 };
 
 JSFunctionSpec gjs_cairo_path_proto_funcs[] = {
-    { NULL }
+    JS_FS_END
 };
 
 /**
@@ -69,13 +69,13 @@ JSObject *
 gjs_cairo_path_from_path(JSContext    *context,
                          cairo_path_t *path)
 {
-    JSObject *object;
     GjsCairoPath *priv;
 
     g_return_val_if_fail(context != NULL, NULL);
     g_return_val_if_fail(path != NULL, NULL);
 
-    object = JS_NewObject(context, &gjs_cairo_path_class, NULL, NULL);
+    JS::RootedObject object(context,
+                            JS_NewObject(context, &gjs_cairo_path_class, NULL, NULL));
     if (!object) {
         gjs_throw(context, "failed to create path");
         return NULL;
