@@ -22,8 +22,8 @@
 
 #include <config.h>
 
-#include <gjs/gjs-module.h>
-#include <gjs/compat.h>
+#include "gjs/jsapi-util.h"
+#include "gjs/jsapi-wrapper.h"
 #include <cairo.h>
 #include <cairo-gobject.h>
 #include "cairo-private.h"
@@ -52,7 +52,7 @@ gjs_cairo_pattern_finalize(JSFreeOp *fop,
 
 /* Properties */
 JSPropertySpec gjs_cairo_pattern_proto_props[] = {
-    { NULL }
+    JS_PS_END
 };
 
 /* Methods */
@@ -60,34 +60,32 @@ JSPropertySpec gjs_cairo_pattern_proto_props[] = {
 static JSBool
 getType_func(JSContext *context,
              unsigned   argc,
-             jsval     *vp)
+             JS::Value *vp)
 {
-    JS::CallReceiver rec = JS::CallReceiverFromVp(vp);
-    JSObject *obj = JSVAL_TO_OBJECT(rec.thisv());
-
+    GJS_GET_THIS(context, argc, vp, rec, obj);
     cairo_pattern_t *pattern;
     cairo_pattern_type_t type;
 
     if (argc > 1) {
         gjs_throw(context, "Pattern.getType() takes no arguments");
-        return JS_FALSE;
+        return false;
     }
 
     pattern = gjs_cairo_pattern_get_pattern(context, obj);
     type = cairo_pattern_get_type(pattern);
 
     if (!gjs_cairo_check_status(context, cairo_pattern_status(pattern), "pattern"))
-        return JS_FALSE;
+        return false;
 
     rec.rval().setInt32(type);
-    return JS_TRUE;
+    return true;
 }
 
 JSFunctionSpec gjs_cairo_pattern_proto_funcs[] = {
     // getMatrix
-    { "getType", JSOP_WRAPPER((JSNative)getType_func), 0, 0 },
+    JS_FS("getType", getType_func, 0, 0),
     // setMatrix
-    { NULL }
+    JS_FS_END
 };
 
 /* Public API */
@@ -105,7 +103,7 @@ JSFunctionSpec gjs_cairo_pattern_proto_funcs[] = {
  */
 void
 gjs_cairo_pattern_construct(JSContext       *context,
-                            JSObject        *object,
+                            JS::HandleObject object,
                             cairo_pattern_t *pattern)
 {
     GjsCairoPattern *priv;
@@ -169,13 +167,14 @@ gjs_cairo_pattern_from_pattern(JSContext       *context,
             return gjs_cairo_linear_gradient_from_pattern(context, pattern);
         case CAIRO_PATTERN_TYPE_RADIAL:
             return gjs_cairo_radial_gradient_from_pattern(context, pattern);
+        case CAIRO_PATTERN_TYPE_MESH:
+        case CAIRO_PATTERN_TYPE_RASTER_SOURCE:
         default:
-            break;
+            gjs_throw(context,
+                      "failed to create pattern, unsupported pattern type %d",
+                      cairo_pattern_get_type(pattern));
+            return NULL;
     }
-
-    gjs_throw(context, "failed to create pattern, unsupported pattern type %d",
-              cairo_pattern_get_type(pattern));
-    return NULL;
 }
 
 /**
