@@ -26,23 +26,24 @@
 #include "cairo-module.h"
 #include <cairo.h>
 
-JSBool           gjs_cairo_check_status                 (JSContext       *context,
+bool             gjs_cairo_check_status                 (JSContext       *context,
                                                          cairo_status_t   status,
                                                          const char      *name);
 
-jsval            gjs_cairo_region_create_proto          (JSContext       *context,
-                                                         JSObject        *module,
-                                                         const char      *proto_name,
-                                                         JSObject        *parent);
+JS::Value gjs_cairo_region_create_proto(JSContext       *context,
+                                        JS::HandleObject module,
+                                        const char      *proto_name,
+                                        JS::HandleObject parent);
+
 void             gjs_cairo_region_init                  (JSContext       *context);
 
+JS::Value gjs_cairo_context_create_proto(JSContext       *context,
+                                         JS::HandleObject module,
+                                         const char      *proto_name,
+                                         JS::HandleObject parent);
 
-jsval            gjs_cairo_context_create_proto         (JSContext       *context,
-                                                         JSObject        *module,
-                                                         const char      *proto_name,
-                                                         JSObject        *parent);
 cairo_t *        gjs_cairo_context_get_context          (JSContext       *context,
-                                                         JSObject        *object);
+                                                         JS::HandleObject object);
 JSObject *       gjs_cairo_context_from_context         (JSContext       *context,
                                                          cairo_t         *cr);
 void             gjs_cairo_context_init                 (JSContext       *context);
@@ -50,22 +51,24 @@ void             gjs_cairo_surface_init                 (JSContext       *contex
 
 
 /* cairo_path_t */
-jsval            gjs_cairo_path_create_proto            (JSContext       *context,
-                                                         JSObject        *module,
-                                                         const char      *proto_name,
-                                                         JSObject        *parent);
+JS::Value gjs_cairo_path_create_proto(JSContext       *context,
+                                      JS::HandleObject module,
+                                      const char      *proto_name,
+                                      JS::HandleObject parent);
+
 JSObject *       gjs_cairo_path_from_path               (JSContext       *context,
                                                          cairo_path_t    *path);
 cairo_path_t *   gjs_cairo_path_get_path                (JSContext       *context,
                                                          JSObject        *path_wrapper);
 
 /* surface */
-jsval            gjs_cairo_surface_create_proto         (JSContext       *context,
-                                                         JSObject        *module,
-                                                         const char      *proto_name,
-                                                         JSObject        *parent);
+JS::Value gjs_cairo_surface_create_proto(JSContext       *context,
+                                         JS::HandleObject module,
+                                         const char      *proto_name,
+                                         JS::HandleObject parent);
+
 void             gjs_cairo_surface_construct            (JSContext       *context,
-                                                         JSObject        *object,
+                                                         JS::HandleObject object,
                                                          cairo_surface_t *surface);
 void             gjs_cairo_surface_finalize_surface     (JSFreeOp        *fop,
                                                          JSObject        *object);
@@ -75,10 +78,11 @@ cairo_surface_t* gjs_cairo_surface_get_surface          (JSContext       *contex
                                                          JSObject        *object);
 
 /* image surface */
-jsval            gjs_cairo_image_surface_create_proto   (JSContext       *context,
-                                                         JSObject        *module,
-                                                         const char      *proto_name,
-                                                         JSObject        *parent);
+JS::Value gjs_cairo_image_surface_create_proto(JSContext       *context,
+                                               JS::HandleObject module,
+                                               const char      *proto_name,
+                                               JS::HandleObject parent);
+
 void             gjs_cairo_image_surface_init           (JSContext       *context,
                                                          JSObject        *object);
 JSObject *       gjs_cairo_image_surface_from_surface   (JSContext       *context,
@@ -86,41 +90,42 @@ JSObject *       gjs_cairo_image_surface_from_surface   (JSContext       *contex
 
 /* postscript surface */
 #ifdef CAIRO_HAS_PS_SURFACE
-jsval            gjs_cairo_ps_surface_create_proto      (JSContext       *context,
-                                                         JSObject        *module,
-                                                         const char      *proto_name,
-                                                         JSObject        *parent);
+JS::Value gjs_cairo_ps_surface_create_proto(JSContext       *context,
+                                            JS::HandleObject module,
+                                            const char      *proto_name,
+                                            JS::HandleObject parent);
 #endif
 JSObject *       gjs_cairo_ps_surface_from_surface       (JSContext       *context,
                                                           cairo_surface_t *surface);
 
 /* pdf surface */
 #ifdef CAIRO_HAS_PDF_SURFACE
-jsval            gjs_cairo_pdf_surface_create_proto     (JSContext       *context,
-                                                         JSObject        *module,
-                                                         const char      *proto_name,
-                                                         JSObject        *parent);
+JS::Value gjs_cairo_pdf_surface_create_proto(JSContext       *context,
+                                             JS::HandleObject module,
+                                             const char      *proto_name,
+                                             JS::HandleObject parent);
 #endif
 JSObject *       gjs_cairo_pdf_surface_from_surface     (JSContext       *context,
                                                          cairo_surface_t *surface);
 
 /* svg surface */
 #ifdef CAIRO_HAS_SVG_SURFACE
-jsval            gjs_cairo_svg_surface_create_proto     (JSContext       *context,
-                                                         JSObject        *module,
-                                                         const char      *proto_name,
-                                                         JSObject        *parent);
+JS::Value gjs_cairo_svg_surface_create_proto(JSContext       *context,
+                                             JS::HandleObject module,
+                                             const char      *proto_name,
+                                             JS::HandleObject parent);
 #endif
 JSObject *       gjs_cairo_svg_surface_from_surface     (JSContext       *context,
                                                          cairo_surface_t *surface);
 
 /* pattern */
-jsval            gjs_cairo_pattern_create_proto         (JSContext       *context,
-                                                         JSObject        *module,
-                                                         const char      *proto_name,
-                                                         JSObject        *parent);
+JS::Value gjs_cairo_pattern_create_proto(JSContext       *context,
+                                         JS::HandleObject module,
+                                         const char      *proto_name,
+                                         JS::HandleObject parent);
+
 void             gjs_cairo_pattern_construct            (JSContext       *context,
-                                                         JSObject        *object,
+                                                         JS::HandleObject object,
                                                          cairo_pattern_t *pattern);
 void             gjs_cairo_pattern_finalize_pattern     (JSFreeOp        *fop,
                                                          JSObject        *object);
@@ -130,40 +135,44 @@ cairo_pattern_t* gjs_cairo_pattern_get_pattern          (JSContext       *contex
                                                          JSObject        *object);
 
 /* gradient */
-jsval            gjs_cairo_gradient_create_proto        (JSContext       *context,
-                                                         JSObject        *module,
-                                                         const char      *proto_name,
-                                                         JSObject        *parent);
+JS::Value gjs_cairo_gradient_create_proto(JSContext       *context,
+                                          JS::HandleObject module,
+                                          const char      *proto_name,
+                                          JS::HandleObject parent);
 
 /* linear gradient */
-jsval            gjs_cairo_linear_gradient_create_proto (JSContext       *context,
-                                                         JSObject        *module,
-                                                         const char      *proto_name,
-                                                         JSObject        *parent);
+JS::Value gjs_cairo_linear_gradient_create_proto(JSContext       *context,
+                                                 JS::HandleObject module,
+                                                 const char      *proto_name,
+                                                 JS::HandleObject parent);
+
 JSObject *       gjs_cairo_linear_gradient_from_pattern (JSContext       *context,
                                                          cairo_pattern_t *pattern);
 
 /* radial gradient */
-jsval            gjs_cairo_radial_gradient_create_proto (JSContext       *context,
-                                                         JSObject        *module,
-                                                         const char      *proto_name,
-                                                         JSObject        *parent);
+JS::Value gjs_cairo_radial_gradient_create_proto(JSContext       *context,
+                                                 JS::HandleObject module,
+                                                 const char      *proto_name,
+                                                 JS::HandleObject parent);
+
 JSObject *       gjs_cairo_radial_gradient_from_pattern (JSContext       *context,
                                                          cairo_pattern_t *pattern);
 
 /* surface pattern */
-jsval            gjs_cairo_surface_pattern_create_proto (JSContext       *context,
-                                                         JSObject        *module,
-                                                         const char      *proto_name,
-                                                         JSObject        *parent);
+JS::Value gjs_cairo_surface_pattern_create_proto(JSContext       *context,
+                                                 JS::HandleObject module,
+                                                 const char      *proto_name,
+                                                 JS::HandleObject parent);
+
 JSObject *       gjs_cairo_surface_pattern_from_pattern (JSContext       *context,
                                                          cairo_pattern_t *pattern);
 
 /* solid pattern */
-jsval            gjs_cairo_solid_pattern_create_proto   (JSContext       *context,
-                                                         JSObject        *module,
-                                                         const char      *proto_name,
-                                                         JSObject        *parent);
+JS::Value gjs_cairo_solid_pattern_create_proto(JSContext       *context,
+                                               JS::HandleObject module,
+                                               const char      *proto_name,
+                                               JS::HandleObject parent);
+
 JSObject *       gjs_cairo_solid_pattern_from_pattern   (JSContext       *context,
                                                          cairo_pattern_t *pattern);
 
