@@ -22,8 +22,8 @@
 
 #include <config.h>
 
-#include <gjs/gjs-module.h>
-#include <gjs/compat.h>
+#include "gjs/jsapi-util-args.h"
+#include "gjs/jsapi-wrapper.h"
 #include <cairo.h>
 #include "cairo-private.h"
 
@@ -37,26 +37,26 @@ GJS_NATIVE_CONSTRUCTOR_DECLARE(cairo_radial_gradient)
 
     GJS_NATIVE_CONSTRUCTOR_PRELUDE(cairo_radial_gradient);
 
-    if (!gjs_parse_call_args(context, "RadialGradient", "ffffff", argv,
-                        "cx0", &cx0,
-                        "cy0", &cy0,
-                        "radius0", &radius0,
-                        "cx1", &cx1,
-                        "cy1", &cy1,
-                        "radius1", &radius1))
-        return JS_FALSE;
+    if (!gjs_parse_call_args(context, "RadialGradient", argv, "ffffff",
+                             "cx0", &cx0,
+                             "cy0", &cy0,
+                             "radius0", &radius0,
+                             "cx1", &cx1,
+                             "cy1", &cy1,
+                             "radius1", &radius1))
+        return false;
 
     pattern = cairo_pattern_create_radial(cx0, cy0, radius0, cx1, cy1, radius1);
 
     if (!gjs_cairo_check_status(context, cairo_pattern_status(pattern), "pattern"))
-        return JS_FALSE;
+        return false;
 
     gjs_cairo_pattern_construct(context, object, pattern);
     cairo_pattern_destroy(pattern);
 
     GJS_NATIVE_CONSTRUCTOR_FINISH(cairo_radial_gradient);
 
-    return JS_TRUE;
+    return true;
 }
 
 static void
@@ -67,25 +67,24 @@ gjs_cairo_radial_gradient_finalize(JSFreeOp *fop,
 }
 
 JSPropertySpec gjs_cairo_radial_gradient_proto_props[] = {
-    { NULL }
+    JS_PS_END
 };
 
 JSFunctionSpec gjs_cairo_radial_gradient_proto_funcs[] = {
     // getRadialCircles
-    { NULL }
+    JS_FS_END
 };
 
 JSObject *
 gjs_cairo_radial_gradient_from_pattern(JSContext       *context,
                                        cairo_pattern_t *pattern)
 {
-    JSObject *object;
-
     g_return_val_if_fail(context != NULL, NULL);
     g_return_val_if_fail(pattern != NULL, NULL);
     g_return_val_if_fail(cairo_pattern_get_type(pattern) == CAIRO_PATTERN_TYPE_RADIAL, NULL);
 
-    object = JS_NewObject(context, &gjs_cairo_radial_gradient_class, NULL, NULL);
+    JS::RootedObject object(context,
+                            JS_NewObject(context, &gjs_cairo_radial_gradient_class, NULL, NULL));
     if (!object) {
         gjs_throw(context, "failed to create radial gradient pattern");
         return NULL;
