@@ -22,9 +22,9 @@
 
 #include <config.h>
 
-#include <gjs/gjs-module.h>
-#include <gjs/compat.h>
-#include <gi/foreign.h>
+#include "gi/foreign.h"
+#include "gjs/jsapi-util-args.h"
+#include "gjs/jsapi-wrapper.h"
 
 #include <cairo.h>
 #include <cairo-gobject.h>
@@ -40,8 +40,9 @@ GJS_DEFINE_PROTO_WITH_GTYPE("CairoRegion", cairo_region, CAIRO_GOBJECT_TYPE_REGI
 GJS_DEFINE_PRIV_FROM_JS(GjsCairoRegion, gjs_cairo_region_class);
 
 static cairo_region_t *
-get_region(JSContext *context,
-           JSObject *obj) {
+get_region(JSContext       *context,
+           JS::HandleObject obj)
+{
     GjsCairoRegion *priv = priv_from_js(context, obj);
     if (priv == NULL)
         return NULL;
@@ -49,14 +50,14 @@ get_region(JSContext *context,
         return priv->region;
 }
 
-static JSBool
-fill_rectangle(JSContext *context, JSObject *obj,
+static bool
+fill_rectangle(JSContext             *context,
+               JS::HandleObject       obj,
                cairo_rectangle_int_t *rect);
 
-#define PRELUDE                                                 \
-    JS::CallArgs argv = JS::CallArgsFromVp (argc, vp);          \
-    JSObject *obj = JSVAL_TO_OBJECT(argv.thisv());              \
-    cairo_region_t *this_region = get_region(context, obj);
+#define PRELUDE                                                       \
+    GJS_GET_PRIV(context, argc, vp, argv, obj, GjsCairoRegion, priv); \
+    cairo_region_t *this_region = priv ? priv->region : NULL;
 
 #define RETURN_STATUS                                           \
     return gjs_cairo_check_status(context, cairo_region_status(this_region), "region");
@@ -65,20 +66,19 @@ fill_rectangle(JSContext *context, JSObject *obj,
     static JSBool                                               \
     method##_func(JSContext *context,                           \
                   unsigned argc,                                \
-                  jsval *vp)                                    \
+                  JS::Value *vp)                                \
     {                                                           \
         PRELUDE;                                                \
-        JSObject *other_obj;                                    \
+        JS::RootedObject other_obj(context);                    \
         cairo_region_t *other_region;                           \
-        if (!gjs_parse_call_args(context, #method, "o", argv,   \
-                            "other_region", &other_obj))        \
-            return JS_FALSE;                                    \
+        if (!gjs_parse_call_args(context, #method, argv, "o",   \
+                                 "other_region", &other_obj))   \
+            return false;                                       \
                                                                 \
-        this_region = get_region(context, obj);                 \
         other_region = get_region(context, other_obj);          \
                                                                 \
         cairo_region_##method(this_region, other_region);       \
-            argv.rval().set(JSVAL_VOID);               \
+            argv.rval().setUndefined();                         \
             RETURN_STATUS;                                      \
     }
 
@@ -86,20 +86,20 @@ fill_rectangle(JSContext *context, JSObject *obj,
     static JSBool                                               \
     method##_rectangle_func(JSContext *context,                 \
                             unsigned argc,                      \
-                            jsval *vp)                          \
+                            JS::Value *vp)                      \
     {                                                           \
         PRELUDE;                                                \
-        JSObject *rect_obj;                                     \
+        JS::RootedObject rect_obj(context);                     \
         cairo_rectangle_int_t rect;                             \
-        if (!gjs_parse_call_args(context, #method, "o", argv,   \
-                            "rect", &rect_obj))                 \
-            return JS_FALSE;                                    \
+        if (!gjs_parse_call_args(context, #method, argv, "o",   \
+                                 "rect", &rect_obj))            \
+            return false;                                       \
                                                                 \
         if (!fill_rectangle(context, rect_obj, &rect))          \
-            return JS_FALSE;                                    \
+            return false;                                       \
                                                                 \
         cairo_region_##method##_rectangle(this_region, &rect);  \
-            argv.rval().set(JSVAL_VOID);               \
+            argv.rval().setUndefined();                         \
             RETURN_STATUS;                                      \
     }
 
@@ -113,33 +113,34 @@ REGION_DEFINE_RECT_FUNC(subtract)
 REGION_DEFINE_RECT_FUNC(intersect)
 REGION_DEFINE_RECT_FUNC(xor)
 
-static JSBool
-fill_rectangle(JSContext *context, JSObject *obj,
+static bool
+fill_rectangle(JSContext             *context,
+               JS::HandleObject       obj,
                cairo_rectangle_int_t *rect)
 {
-    jsval val;
+    JS::RootedValue val(context);
 
     if (!gjs_object_get_property_const(context, obj, GJS_STRING_X, &val))
-        return JS_FALSE;
-    if (!JS_ValueToInt32(context, val, &rect->x))
-        return JS_FALSE;
+        return false;
+    if (!JS::ToInt32(context, val, &rect->x))
+        return false;
 
     if (!gjs_object_get_property_const(context, obj, GJS_STRING_Y, &val))
-        return JS_FALSE;
-    if (!JS_ValueToInt32(context, val, &rect->y))
-        return JS_FALSE;
+        return false;
+    if (!JS::ToInt32(context, val, &rect->y))
+        return false;
 
     if (!gjs_object_get_property_const(context, obj, GJS_STRING_WIDTH, &val))
-        return JS_FALSE;
-    if (!JS_ValueToInt32(context, val, &rect->width))
-        return JS_FALSE;
+        return false;
+    if (!JS::ToInt32(context, val, &rect->width))
+        return false;
 
     if (!gjs_object_get_property_const(context, obj, GJS_STRING_HEIGHT, &val))
-        return JS_FALSE;
-    if (!JS_ValueToInt32(context, val, &rect->height))
-        return JS_FALSE;
+        return false;
+    if (!JS::ToInt32(context, val, &rect->height))
+        return false;
 
-    return JS_TRUE;
+    return true;
 }
 
 static JSObject *
@@ -147,18 +148,18 @@ make_rectangle(JSContext *context,
                cairo_rectangle_int_t *rect)
 {
     JSObject *rect_obj = JS_NewObject(context, NULL, NULL, NULL);
-    jsval val;
+    JS::Value val;
 
-    val = INT_TO_JSVAL(rect->x);
+    val = JS::Int32Value(rect->x);
     JS_SetProperty(context, rect_obj, "x", &val);
 
-    val = INT_TO_JSVAL(rect->y);
+    val = JS::Int32Value(rect->y);
     JS_SetProperty(context, rect_obj, "y", &val);
 
-    val = INT_TO_JSVAL(rect->width);
+    val = JS::Int32Value(rect->width);
     JS_SetProperty(context, rect_obj, "width", &val);
 
-    val = INT_TO_JSVAL(rect->height);
+    val = JS::Int32Value(rect->height);
     JS_SetProperty(context, rect_obj, "height", &val);
 
     return rect_obj;
@@ -167,67 +168,64 @@ make_rectangle(JSContext *context,
 static JSBool
 num_rectangles_func(JSContext *context,
                     unsigned argc,
-                    jsval *vp)
+                    JS::Value *vp)
 {
     PRELUDE;
     int n_rects;
-    jsval retval;
 
-    if (!gjs_parse_call_args(context, "num_rectangles", "", argv))
-        return JS_FALSE;
+    if (!gjs_parse_call_args(context, "num_rectangles", argv, ""))
+        return false;
 
     n_rects = cairo_region_num_rectangles(this_region);
-    retval = INT_TO_JSVAL(n_rects);
-    argv.rval().set(retval);
+    argv.rval().setInt32(n_rects);
     RETURN_STATUS;
 }
 
 static JSBool
 get_rectangle_func(JSContext *context,
                    unsigned argc,
-                   jsval *vp)
+                   JS::Value *vp)
 {
     PRELUDE;
     int i;
     JSObject *rect_obj;
     cairo_rectangle_int_t rect;
-    jsval retval;
 
-    if (!gjs_parse_call_args(context, "get_rectangle", "i", argv, "rect", &i))
-        return JS_FALSE;
+    if (!gjs_parse_call_args(context, "get_rectangle", argv, "i",
+                             "rect", &i))
+        return false;
 
     cairo_region_get_rectangle(this_region, i, &rect);
     rect_obj = make_rectangle(context, &rect);
 
-    retval = OBJECT_TO_JSVAL(rect_obj);
-    argv.rval().set(retval);
+    argv.rval().setObjectOrNull(rect_obj);
     RETURN_STATUS;
 }
 
 JSPropertySpec gjs_cairo_region_proto_props[] = {
-    { NULL }
+    JS_PS_END
 };
 
 JSFunctionSpec gjs_cairo_region_proto_funcs[] = {
-    { "union", (JSNative)union_func, 0, 0 },
-    { "subtract", (JSNative)subtract_func, 0, 0 },
-    { "intersect", (JSNative)intersect_func, 0, 0 },
-    { "xor", (JSNative)xor_func, 0, 0 },
+    JS_FS("union", union_func, 0, 0),
+    JS_FS("subtract", subtract_func, 0, 0),
+    JS_FS("intersect", intersect_func, 0, 0),
+    JS_FS("xor", xor_func, 0, 0),
 
-    { "unionRectangle", (JSNative)union_rectangle_func, 0, 0 },
-    { "subtractRectangle", (JSNative)subtract_rectangle_func, 0, 0 },
-    { "intersectRectangle", (JSNative)intersect_rectangle_func, 0, 0 },
-    { "xorRectangle", (JSNative)xor_rectangle_func, 0, 0 },
+    JS_FS("unionRectangle", union_rectangle_func, 0, 0),
+    JS_FS("subtractRectangle", subtract_rectangle_func, 0, 0),
+    JS_FS("intersectRectangle", intersect_rectangle_func, 0, 0),
+    JS_FS("xorRectangle", xor_rectangle_func, 0, 0),
 
-    { "numRectangles", (JSNative)num_rectangles_func, 0, 0 },
-    { "getRectangle", (JSNative)get_rectangle_func, 0, 0 },
-    { NULL }
+    JS_FS("numRectangles", num_rectangles_func, 0, 0),
+    JS_FS("getRectangle", get_rectangle_func, 0, 0),
+    JS_FS_END
 };
 
 static void
-_gjs_cairo_region_construct_internal(JSContext *context,
-                                     JSObject *obj,
-                                     cairo_region_t *region)
+_gjs_cairo_region_construct_internal(JSContext       *context,
+                                     JS::HandleObject obj,
+                                     cairo_region_t  *region)
 {
     GjsCairoRegion *priv;
 
@@ -248,8 +246,8 @@ GJS_NATIVE_CONSTRUCTOR_DECLARE(cairo_region)
 
     GJS_NATIVE_CONSTRUCTOR_PRELUDE(cairo_region);
 
-    if (!gjs_parse_call_args(context, "Region", "", argv))
-        return JS_FALSE;
+    if (!gjs_parse_call_args(context, "Region", argv, ""))
+        return false;
 
     region = cairo_region_create();
 
@@ -258,7 +256,7 @@ GJS_NATIVE_CONSTRUCTOR_DECLARE(cairo_region)
 
     GJS_NATIVE_CONSTRUCTOR_FINISH(cairo_region);
 
-    return JS_TRUE;
+    return true;
 }
 
 static void
@@ -278,9 +276,8 @@ static JSObject *
 gjs_cairo_region_from_region(JSContext *context,
                              cairo_region_t *region)
 {
-    JSObject *object;
-
-    object = JS_NewObject(context, &gjs_cairo_region_class, NULL, NULL);
+    JS::RootedObject object(context,
+                            JS_NewObject(context, &gjs_cairo_region_class, NULL, NULL));
     if (!object)
         return NULL;
 
@@ -289,51 +286,50 @@ gjs_cairo_region_from_region(JSContext *context,
     return object;
 }
 
-static JSBool
+static bool
 region_to_g_argument(JSContext      *context,
-                     jsval           value,
+                     JS::Value       value,
                      const char     *arg_name,
                      GjsArgumentType argument_type,
                      GITransfer      transfer,
-                     gboolean        may_be_null,
+                     bool            may_be_null,
                      GArgument      *arg)
 {
-    JSObject *obj;
+    JS::RootedObject obj(context, &value.toObject());
     cairo_region_t *region;
 
-    obj = JSVAL_TO_OBJECT(value);
     region = get_region(context, obj);
     if (!region)
-        return JS_FALSE;
+        return false;
     if (transfer == GI_TRANSFER_EVERYTHING)
         cairo_region_destroy(region);
 
     arg->v_pointer = region;
-    return JS_TRUE;
+    return true;
 }
 
-static JSBool
-region_from_g_argument(JSContext  *context,
-                       jsval      *value_p,
-                       GArgument  *arg)
+static bool
+region_from_g_argument(JSContext             *context,
+                       JS::MutableHandleValue value_p,
+                       GIArgument            *arg)
 {
     JSObject *obj;
 
     obj = gjs_cairo_region_from_region(context, (cairo_region_t*)arg->v_pointer);
     if (!obj)
-        return JS_FALSE;
+        return false;
 
-    *value_p = OBJECT_TO_JSVAL(obj);
-    return JS_TRUE;
+    value_p.setObject(*obj);
+    return true;
 }
 
-static JSBool
+static bool
 region_release_argument(JSContext  *context,
                         GITransfer  transfer,
                         GArgument  *arg)
 {
     cairo_region_destroy((cairo_region_t*)arg->v_pointer);
-    return JS_TRUE;
+    return true;
 }
 
 static GjsForeignInfo foreign_info = {
