@@ -22,8 +22,8 @@
 
 #include <config.h>
 
-#include <gjs/gjs-module.h>
-#include <gjs/compat.h>
+#include "gjs/jsapi-util-args.h"
+#include "gjs/jsapi-wrapper.h"
 #include <cairo.h>
 #include "cairo-private.h"
 
@@ -37,83 +37,82 @@ gjs_cairo_solid_pattern_finalize(JSFreeOp *fop,
 }
 
 JSPropertySpec gjs_cairo_solid_pattern_proto_props[] = {
-    { NULL }
+    JS_PS_END
 };
 
 static JSBool
 createRGB_func(JSContext *context,
                unsigned   argc,
-               jsval     *vp)
+               JS::Value *vp)
 {
     JS::CallArgs argv = JS::CallArgsFromVp (argc, vp);
     double red, green, blue;
     cairo_pattern_t *pattern;
     JSObject *pattern_wrapper;
 
-    if (!gjs_parse_call_args(context, "createRGB", "fff", argv,
-                        "red", &red,
-                        "green", &green,
-                        "blue", &blue))
-        return JS_FALSE;
+    if (!gjs_parse_call_args(context, "createRGB", argv, "fff",
+                             "red", &red,
+                             "green", &green,
+                             "blue", &blue))
+        return false;
 
     pattern = cairo_pattern_create_rgb(red, green, blue);
     if (!gjs_cairo_check_status(context, cairo_pattern_status(pattern), "pattern"))
-        return JS_FALSE;
+        return false;
 
     pattern_wrapper = gjs_cairo_solid_pattern_from_pattern(context, pattern);
     cairo_pattern_destroy(pattern);
 
-    argv.rval().set(OBJECT_TO_JSVAL(pattern_wrapper));
+    argv.rval().setObjectOrNull(pattern_wrapper);
 
-    return JS_TRUE;
+    return true;
 }
 
 static JSBool
 createRGBA_func(JSContext *context,
                 unsigned   argc,
-                jsval     *vp)
+                JS::Value *vp)
 {
     JS::CallArgs argv = JS::CallArgsFromVp (argc, vp);
     double red, green, blue, alpha;
     cairo_pattern_t *pattern;
     JSObject *pattern_wrapper;
 
-    if (!gjs_parse_call_args(context, "createRGBA", "ffff", argv,
-                        "red", &red,
-                        "green", &green,
-                        "blue", &blue,
-                        "alpha", &alpha))
-        return JS_FALSE;
+    if (!gjs_parse_call_args(context, "createRGBA", argv, "ffff",
+                             "red", &red,
+                             "green", &green,
+                             "blue", &blue,
+                             "alpha", &alpha))
+        return false;
 
     pattern = cairo_pattern_create_rgba(red, green, blue, alpha);
     if (!gjs_cairo_check_status(context, cairo_pattern_status(pattern), "pattern"))
-        return JS_FALSE;
+        return false;
 
     pattern_wrapper = gjs_cairo_solid_pattern_from_pattern(context, pattern);
     cairo_pattern_destroy(pattern);
 
-    argv.rval().set(OBJECT_TO_JSVAL(pattern_wrapper));
+    argv.rval().setObjectOrNull(pattern_wrapper);
 
-    return JS_TRUE;
+    return true;
 }
 
 JSFunctionSpec gjs_cairo_solid_pattern_proto_funcs[] = {
-    { "createRGB", JSOP_WRAPPER((JSNative)createRGB_func), 0, 0 },
-    { "createRGBA", JSOP_WRAPPER((JSNative)createRGBA_func), 0, 0 },
-    { NULL }
+    JS_FS("createRGB", createRGB_func, 0, 0),
+    JS_FS("createRGBA", createRGBA_func, 0, 0),
+    JS_FS_END
 };
 
 JSObject *
 gjs_cairo_solid_pattern_from_pattern(JSContext       *context,
                                      cairo_pattern_t *pattern)
 {
-    JSObject *object;
-
     g_return_val_if_fail(context != NULL, NULL);
     g_return_val_if_fail(pattern != NULL, NULL);
     g_return_val_if_fail(cairo_pattern_get_type(pattern) == CAIRO_PATTERN_TYPE_SOLID, NULL);
 
-    object = JS_NewObject(context, &gjs_cairo_solid_pattern_class, NULL, NULL);
+    JS::RootedObject object(context,
+                            JS_NewObject(context, &gjs_cairo_solid_pattern_class, NULL, NULL));
     if (!object) {
         gjs_throw(context, "failed to create solid pattern");
         return NULL;
