@@ -22,9 +22,9 @@
 
 #include <config.h>
 
-#include <gjs/gjs-module.h>
-#include <gjs/compat.h>
-#include <gi/foreign.h>
+#include "gi/foreign.h"
+#include "gjs/jsapi-util-args.h"
+#include "gjs/jsapi-wrapper.h"
 #include <cairo.h>
 #include <cairo-gobject.h>
 #include "cairo-private.h"
@@ -53,70 +53,66 @@ gjs_cairo_surface_finalize(JSFreeOp *fop,
 
 /* Properties */
 JSPropertySpec gjs_cairo_surface_proto_props[] = {
-    { NULL }
+    JS_PS_END
 };
 
 /* Methods */
 static JSBool
 writeToPNG_func(JSContext *context,
                 unsigned   argc,
-                jsval     *vp)
+                JS::Value *vp)
 {
-    JS::CallArgs argv = JS::CallArgsFromVp (argc, vp);
-    JSObject *obj = JSVAL_TO_OBJECT(argv.thisv());
-
+    GJS_GET_THIS(context, argc, vp, argv, obj);
     char *filename;
     cairo_surface_t *surface;
 
-    if (!gjs_parse_call_args(context, "writeToPNG", "s", argv,
-                        "filename", &filename))
-        return JS_FALSE;
+    if (!gjs_parse_call_args(context, "writeToPNG", argv, "F",
+                             "filename", &filename))
+        return false;
 
     surface = gjs_cairo_surface_get_surface(context, obj);
     if (!surface) {
         g_free(filename);
-        return JS_FALSE;
+        return false;
     }
     cairo_surface_write_to_png(surface, filename);
     g_free(filename);
     if (!gjs_cairo_check_status(context, cairo_surface_status(surface),
                                 "surface"))
-        return JS_FALSE;
-    argv.rval().set(JSVAL_VOID);
-    return JS_TRUE;
+        return false;
+    argv.rval().setUndefined();
+    return true;
 }
 
 static JSBool
 getType_func(JSContext *context,
              unsigned   argc,
-             jsval     *vp)
+             JS::Value *vp)
 {
-    JS::CallReceiver rec = JS::CallReceiverFromVp(vp);
-    JSObject *obj = JSVAL_TO_OBJECT(rec.thisv());
-
+    GJS_GET_THIS(context, argc, vp, rec, obj);
     cairo_surface_t *surface;
     cairo_surface_type_t type;
 
     if (argc > 1) {
         gjs_throw(context, "Surface.getType() takes no arguments");
-        return JS_FALSE;
+        return false;
     }
 
     surface = gjs_cairo_surface_get_surface(context, obj);
     type = cairo_surface_get_type(surface);
     if (!gjs_cairo_check_status(context, cairo_surface_status(surface),
                                 "surface"))
-        return JS_FALSE;
+        return false;
 
     rec.rval().setInt32(type);
-    return JS_TRUE;
+    return true;
 }
 
 JSFunctionSpec gjs_cairo_surface_proto_funcs[] = {
     // flush
     // getContent
     // getFontOptions
-    { "getType", JSOP_WRAPPER((JSNative)getType_func), 0, 0},
+    JS_FS("getType", getType_func, 0, 0),
     // markDirty
     // markDirtyRectangle
     // setDeviceOffset
@@ -126,8 +122,8 @@ JSFunctionSpec gjs_cairo_surface_proto_funcs[] = {
     // copyPage
     // showPage
     // hasShowTextGlyphs
-    { "writeToPNG", JSOP_WRAPPER((JSNative)writeToPNG_func), 0, 0 },
-    { NULL }
+    JS_FS("writeToPNG", writeToPNG_func, 0, 0),
+    JS_FS_END
 };
 
 /* Public API */
@@ -145,7 +141,7 @@ JSFunctionSpec gjs_cairo_surface_proto_funcs[] = {
  */
 void
 gjs_cairo_surface_construct(JSContext       *context,
-                            JSObject        *object,
+                            JS::HandleObject object,
                             cairo_surface_t *surface)
 {
     GjsCairoSurface *priv;
@@ -196,25 +192,21 @@ JSObject *
 gjs_cairo_surface_from_surface(JSContext       *context,
                                cairo_surface_t *surface)
 {
-    JSObject *object;
-
     g_return_val_if_fail(context != NULL, NULL);
     g_return_val_if_fail(surface != NULL, NULL);
 
-    switch (cairo_surface_get_type(surface)) {
-        case CAIRO_SURFACE_TYPE_IMAGE:
-            return gjs_cairo_image_surface_from_surface(context, surface);
-        case CAIRO_SURFACE_TYPE_PDF:
-            return gjs_cairo_pdf_surface_from_surface(context, surface);
-        case CAIRO_SURFACE_TYPE_PS:
-            return gjs_cairo_ps_surface_from_surface(context, surface);
-        case CAIRO_SURFACE_TYPE_SVG:
-            return gjs_cairo_svg_surface_from_surface(context, surface);
-        default:
-            break;
-    }
+    cairo_surface_type_t type = cairo_surface_get_type(surface);
+    if (type == CAIRO_SURFACE_TYPE_IMAGE)
+        return gjs_cairo_image_surface_from_surface(context, surface);
+    if (type == CAIRO_SURFACE_TYPE_PDF)
+        return gjs_cairo_pdf_surface_from_surface(context, surface);
+    if (type == CAIRO_SURFACE_TYPE_PS)
+        return gjs_cairo_ps_surface_from_surface(context, surface);
+    if (type == CAIRO_SURFACE_TYPE_SVG)
+        return gjs_cairo_svg_surface_from_surface(context, surface);
 
-    object = JS_NewObject(context, &gjs_cairo_surface_class, NULL, NULL);
+    JS::RootedObject object(context,
+                            JS_NewObject(context, &gjs_cairo_surface_class, NULL, NULL));
     if (!object) {
         gjs_throw(context, "failed to create surface");
         return NULL;
@@ -248,51 +240,51 @@ gjs_cairo_surface_get_surface(JSContext *context,
     return priv->surface;
 }
 
-static JSBool
+static bool
 surface_to_g_argument(JSContext      *context,
-                      jsval           value,
+                      JS::Value       value,
                       const char     *arg_name,
                       GjsArgumentType argument_type,
                       GITransfer      transfer,
-                      gboolean        may_be_null,
+                      bool            may_be_null,
                       GArgument      *arg)
 {
     JSObject *obj;
     cairo_surface_t *s;
 
-    obj = JSVAL_TO_OBJECT(value);
+    obj = &value.toObject();
     s = gjs_cairo_surface_get_surface(context, obj);
     if (!s)
-        return JS_FALSE;
+        return false;
     if (transfer == GI_TRANSFER_EVERYTHING)
         cairo_surface_destroy(s);
 
     arg->v_pointer = s;
-    return JS_TRUE;
+    return true;
 }
 
-static JSBool
-surface_from_g_argument(JSContext  *context,
-                        jsval      *value_p,
-                        GArgument  *arg)
+static bool
+surface_from_g_argument(JSContext             *context,
+                        JS::MutableHandleValue value_p,
+                        GIArgument            *arg)
 {
     JSObject *obj;
 
     obj = gjs_cairo_surface_from_surface(context, (cairo_surface_t*)arg->v_pointer);
     if (!obj)
-        return JS_FALSE;
+        return false;
 
-    *value_p = OBJECT_TO_JSVAL(obj);
-    return JS_TRUE;
+    value_p.setObject(*obj);
+    return true;
 }
 
-static JSBool
+static bool
 surface_release_argument(JSContext  *context,
                          GITransfer  transfer,
                          GArgument  *arg)
 {
     cairo_surface_destroy((cairo_surface_t*)arg->v_pointer);
-    return JS_TRUE;
+    return true;
 }
 
 static GjsForeignInfo foreign_info = {
