@@ -22,8 +22,8 @@
 
 #include <config.h>
 
-#include <gjs/gjs-module.h>
-#include <gjs/compat.h>
+#include "gjs/jsapi-util-args.h"
+#include "gjs/jsapi-wrapper.h"
 #include <cairo.h>
 #include "cairo-private.h"
 
@@ -41,18 +41,18 @@ GJS_NATIVE_CONSTRUCTOR_DECLARE(cairo_svg_surface)
 
     GJS_NATIVE_CONSTRUCTOR_PRELUDE(cairo_svg_surface);
 
-    if (!gjs_parse_call_args(context, "SVGSurface", "sff", argv,
-                        "filename", &filename,
-                        "width", &width,
-                        "height", &height))
-        return JS_FALSE;
+    if (!gjs_parse_call_args(context, "SVGSurface", argv, "Fff",
+                             "filename", &filename,
+                             "width", &width,
+                             "height", &height))
+        return false;
 
     surface = cairo_svg_surface_create(filename, width, height);
 
     if (!gjs_cairo_check_status(context, cairo_surface_status(surface),
                                 "surface")) {
         g_free(filename);
-        return JS_FALSE;
+        return false;
     }
 
     gjs_cairo_surface_construct(context, object, surface);
@@ -61,7 +61,7 @@ GJS_NATIVE_CONSTRUCTOR_DECLARE(cairo_svg_surface)
 
     GJS_NATIVE_CONSTRUCTOR_FINISH(cairo_svg_surface);
 
-    return JS_TRUE;
+    return true;
 }
 
 static void
@@ -72,24 +72,23 @@ gjs_cairo_svg_surface_finalize(JSFreeOp *fop,
 }
 
 JSPropertySpec gjs_cairo_svg_surface_proto_props[] = {
-    { NULL }
+    JS_PS_END
 };
 
 JSFunctionSpec gjs_cairo_svg_surface_proto_funcs[] = {
-    { NULL }
+    JS_FS_END
 };
 
 JSObject *
 gjs_cairo_svg_surface_from_surface(JSContext       *context,
                                    cairo_surface_t *surface)
 {
-    JSObject *object;
-
     g_return_val_if_fail(context != NULL, NULL);
     g_return_val_if_fail(surface != NULL, NULL);
     g_return_val_if_fail(cairo_surface_get_type(surface) == CAIRO_SURFACE_TYPE_SVG, NULL);
 
-    object = JS_NewObject(context, &gjs_cairo_svg_surface_class, NULL, NULL);
+    JS::RootedObject object(context,
+                            JS_NewObject(context, &gjs_cairo_svg_surface_class, NULL, NULL));
     if (!object) {
         gjs_throw(context, "failed to create svg surface");
         return NULL;
