@@ -22,12 +22,11 @@
 
 #include <config.h>
 
-#include <gjs/gjs-module.h>
-#include <gjs/compat.h>
-
+#include "gjs/jsapi-util.h"
+#include "gjs/jsapi-wrapper.h"
 #include "cairo-private.h"
 
-#if CAIRO_HAS_XLIB_SURFACE
+#ifdef CAIRO_HAS_XLIB_SURFACE
 #include "cairo-xlib.h"
 
 class XLibConstructor {
@@ -40,7 +39,7 @@ class XLibConstructor {
 static XLibConstructor constructor;
 #endif
 
-JSBool
+bool
 gjs_cairo_check_status(JSContext      *context,
                        cairo_status_t  status,
                        const char     *name)
@@ -50,101 +49,99 @@ gjs_cairo_check_status(JSContext      *context,
                   name,
                   cairo_status_to_string(status),
                   status);
-        return JS_FALSE;
+        return false;
     }
 
-    return JS_TRUE;
+    return true;
 }
 
-JSBool
-gjs_js_define_cairo_stuff(JSContext *context,
-                          JSObject **module_out)
+bool
+gjs_js_define_cairo_stuff(JSContext              *context,
+                          JS::MutableHandleObject module)
 {
-    jsval obj;
-    JSObject *module;
-    JSObject *surface_proto, *pattern_proto, *gradient_proto;
+    JS::Value obj;
+    JS::RootedObject surface_proto(context), pattern_proto(context),
+        gradient_proto(context);
 
-    module = JS_NewObject (context, NULL, NULL, NULL);
+    module.set(JS_NewObject(context, NULL, NULL, NULL));
 
     obj = gjs_cairo_region_create_proto(context, module,
-                                        "Region", NULL);
-    if (JSVAL_IS_NULL(obj))
-        return JS_FALSE;
+                                        "Region", JS::NullPtr());
+    if (obj.isNull())
+        return false;
     gjs_cairo_region_init(context);
 
     obj = gjs_cairo_context_create_proto(context, module,
-                                         "Context", NULL);
-    if (JSVAL_IS_NULL(obj))
-        return JS_FALSE;
+                                         "Context", JS::NullPtr());
+    if (obj.isNull())
+        return false;
     gjs_cairo_context_init(context);
     gjs_cairo_surface_init(context);
 
     obj = gjs_cairo_surface_create_proto(context, module,
-                                         "Surface", NULL);
-    if (JSVAL_IS_NULL(obj))
-        return JS_FALSE;
-    surface_proto = JSVAL_TO_OBJECT(obj);
+                                         "Surface", JS::NullPtr());
+    if (obj.isNull())
+        return false;
+    surface_proto = &obj.toObject();
 
     obj = gjs_cairo_image_surface_create_proto(context, module,
                                                "ImageSurface", surface_proto);
-    if (JSVAL_IS_NULL(obj))
-        return JS_FALSE;
-    gjs_cairo_image_surface_init(context, JSVAL_TO_OBJECT(obj));
+    if (obj.isNull())
+        return false;
+    gjs_cairo_image_surface_init(context, &obj.toObject());
 
 #if CAIRO_HAS_PS_SURFACE
     obj = gjs_cairo_ps_surface_create_proto(context, module,
                                             "PSSurface", surface_proto);
-    if (JSVAL_IS_NULL(obj))
-        return JS_FALSE;
+    if (obj.isNull())
+        return false;
 #endif
 
 #if CAIRO_HAS_PDF_SURFACE
     obj = gjs_cairo_pdf_surface_create_proto(context, module,
                                              "PDFSurface", surface_proto);
-    if (JSVAL_IS_NULL(obj))
-        return JS_FALSE;
+    if (obj.isNull())
+        return false;
 #endif
 
 #if CAIRO_HAS_SVG_SURFACE
     obj = gjs_cairo_svg_surface_create_proto(context, module,
                                              "SVGSurface", surface_proto);
-    if (JSVAL_IS_NULL(obj))
-        return JS_FALSE;
+    if (obj.isNull())
+        return false;
 #endif
 
     obj = gjs_cairo_pattern_create_proto(context, module,
-                                         "Pattern", NULL);
-    if (JSVAL_IS_NULL(obj))
-        return JS_FALSE;
-    pattern_proto = JSVAL_TO_OBJECT(obj);
+                                         "Pattern", JS::NullPtr());
+    if (obj.isNull())
+        return false;
+    pattern_proto = &obj.toObject();
 
     obj = gjs_cairo_gradient_create_proto(context, module,
                                          "Gradient", pattern_proto);
-    if (JSVAL_IS_NULL(obj))
-        return JS_FALSE;
-    gradient_proto = JSVAL_TO_OBJECT(obj);
+    if (obj.isNull())
+        return false;
+    gradient_proto = &obj.toObject();
 
     obj = gjs_cairo_linear_gradient_create_proto(context, module,
                                                  "LinearGradient", gradient_proto);
-    if (JSVAL_IS_NULL(obj))
-        return JS_FALSE;
+    if (obj.isNull())
+        return false;
 
     obj = gjs_cairo_radial_gradient_create_proto(context, module,
                                                  "RadialGradient", gradient_proto);
-    if (JSVAL_IS_NULL(obj))
-        return JS_FALSE;
+    if (obj.isNull())
+        return false;
 
     obj = gjs_cairo_surface_pattern_create_proto(context, module,
                                                  "SurfacePattern", pattern_proto);
-    if (JSVAL_IS_NULL(obj))
-        return JS_FALSE;
+    if (obj.isNull())
+        return false;
 
     obj = gjs_cairo_solid_pattern_create_proto(context, module,
                                                "SolidPattern", pattern_proto);
-    if (JSVAL_IS_NULL(obj))
-        return JS_FALSE;
+    if (obj.isNull())
+        return false;
 
-    *module_out = module;
-
-    return JS_TRUE;
+    return true;
 }
