@@ -43,7 +43,7 @@
 #include <stdlib.h>
 #include <string.h>
 
-#ifdef HAVE_LIBREADLINE
+#ifdef HAVE_READLINE_READLINE_H
 #include <stdio.h>
 #include <readline/readline.h>
 #include <readline/history.h>
@@ -51,11 +51,10 @@
 
 #include <glib.h>
 #include <glib/gprintf.h>
-#include <gjs/gjs-module.h>
-#include <gjs/compat.h>
-#include <gjs/jsapi-private.h>
 
 #include "console.h"
+#include "gjs/jsapi-private.h"
+#include "gjs/jsapi-wrapper.h"
 
 static void
 gjs_console_error_reporter(JSContext *cx, const char *message, JSErrorReport *report)
@@ -127,41 +126,41 @@ gjs_console_error_reporter(JSContext *cx, const char *message, JSErrorReport *re
     g_free(prefix);
 }
 
-#ifdef HAVE_LIBREADLINE
-static JSBool
+#ifdef HAVE_READLINE_READLINE_H
+static bool
 gjs_console_readline(JSContext *cx, char **bufp, FILE *file, const char *prompt)
 {
     char *line;
     line = readline(prompt);
     if (!line)
-        return JS_FALSE;
+        return false;
     if (line[0] != '\0')
         add_history(line);
     *bufp = line;
-    return JS_TRUE;
+    return true;
 }
 #else
-static JSBool
+static bool
 gjs_console_readline(JSContext *cx, char **bufp, FILE *file, const char *prompt)
 {
     char line[256];
     fprintf(stdout, "%s", prompt);
     fflush(stdout);
     if (!fgets(line, sizeof line, file))
-        return JS_FALSE;
+        return false;
     *bufp = g_strdup(line);
-    return JS_TRUE;
+    return true;
 }
 #endif
 
-JSBool
+static JSBool
 gjs_console_interact(JSContext *context,
                      unsigned   argc,
-                     jsval     *vp)
+                     JS::Value *vp)
 {
-    JSObject *object = JS_THIS_OBJECT(context, vp);
-    gboolean eof = FALSE;
-    jsval result;
+    GJS_GET_THIS(context, argc, vp, argv, object);
+    bool eof = false;
+    JS::RootedValue result(context);
     JSString *str;
     GString *buffer = NULL;
     char *temp_buf = NULL;
@@ -185,7 +184,7 @@ gjs_console_interact(JSContext *context,
         do {
             if (!gjs_console_readline(context, &temp_buf, file,
                                       startline == lineno ? "gjs> " : ".... ")) {
-                eof = JS_TRUE;
+                eof = true;
                 break;
             }
             g_string_append(buffer, temp_buf);
@@ -196,15 +195,14 @@ gjs_console_interact(JSContext *context,
         JS::CompileOptions options(context);
         options.setUTF8(true)
                .setFileAndLine("typein", startline);
-        js::RootedObject rootedObj(context, object);
-        JS::Evaluate(context, rootedObj, options, buffer->str, buffer->len,  &result);
+        JS::Evaluate(context, object, options, buffer->str, buffer->len, result.address());
 
         gjs_schedule_gc_if_needed(context);
 
-        if (JS_GetPendingException(context, &result)) {
+        if (JS_GetPendingException(context, result.address())) {
             str = JS_ValueToString(context, result);
             JS_ClearPendingException(context);
-        } else if (JSVAL_IS_VOID(result)) {
+        } else if (result.isUndefined()) {
             goto next;
         } else {
             str = JS_ValueToString(context, result);
@@ -220,7 +218,7 @@ gjs_console_interact(JSContext *context,
         }
 
  next:
-        g_string_free(buffer, TRUE);
+        g_string_free(buffer, true);
     } while (!eof);
 
     g_fprintf(stdout, "\n");
@@ -228,23 +226,14 @@ gjs_console_interact(JSContext *context,
     if (file != stdin)
         fclose(file);
 
-    return JS_TRUE;
+    return true;
 }
 
-JSBool
-gjs_define_console_stuff(JSContext  *context,
-                         JSObject  **module_out)
+bool
+gjs_define_console_stuff(JSContext              *context,
+                         JS::MutableHandleObject module)
 {
-    JSObject *module;
-
-    module = JS_NewObject (context, NULL, NULL, NULL);
-
-    if (!JS_DefineFunction(context, module,
-                           "interact",
-                           (JSNative) gjs_console_interact,
-                           1, GJS_MODULE_PROP_FLAGS))
-        return JS_FALSE;
-
-    *module_out = module;
-    return JS_TRUE;
+    module.set(JS_NewObject(context, NULL, NULL, NULL));
+    return JS_DefineFunction(context, module, "interact", gjs_console_interact,
+                             1, GJS_MODULE_PROP_FLAGS);
 }
