@@ -28,104 +28,102 @@
 #include <unistd.h>
 #include <time.h>
 
-#include <gjs/gjs-module.h>
-#include <gi/object.h>
+#include <gjs/context.h>
+
+#include "gi/object.h"
+#include "gjs/jsapi-util-args.h"
 #include "system.h"
 
 static JSBool
 gjs_address_of(JSContext *context,
                unsigned   argc,
-               jsval     *vp)
+               JS::Value *vp)
 {
     JS::CallArgs argv = JS::CallArgsFromVp (argc, vp);
-    JSObject *target_obj;
-    JSBool ret;
+    JS::RootedObject target_obj(context);
+    bool ret;
     char *pointer_string;
-    jsval retval;
 
-    if (!gjs_parse_call_args(context, "addressOf", "o", argv, "object", &target_obj))
-        return JS_FALSE;
+    if (!gjs_parse_call_args(context, "addressOf", argv, "o",
+                             "object", &target_obj))
+        return false;
 
-    pointer_string = g_strdup_printf("%p", target_obj);
+    pointer_string = g_strdup_printf("%p", target_obj.get());
+
+    ret = gjs_string_from_utf8(context, pointer_string, -1, argv.rval());
 
-    ret = gjs_string_from_utf8(context, pointer_string, -1, &retval);
     g_free(pointer_string);
-
-    if (ret)
-        argv.rval().set(retval);
-
     return ret;
 }
 
 static JSBool
 gjs_refcount(JSContext *context,
              unsigned   argc,
-             jsval     *vp)
+             JS::Value *vp)
 {
     JS::CallArgs argv = JS::CallArgsFromVp (argc, vp);
-    jsval retval;
-    JSObject *target_obj;
+    JS::RootedObject target_obj(context);
     GObject *obj;
 
-    if (!gjs_parse_call_args(context, "refcount", "o", argv, "object", &target_obj))
-        return JS_FALSE;
+    if (!gjs_parse_call_args(context, "refcount", argv, "o",
+                             "object", &target_obj))
+        return false;
 
-    if (!gjs_typecheck_object(context, target_obj,
-                              G_TYPE_OBJECT, JS_TRUE))
-        return JS_FALSE;
+    if (!gjs_typecheck_object(context, target_obj, G_TYPE_OBJECT, true))
+        return false;
 
     obj = gjs_g_object_from_object(context, target_obj);
     if (obj == NULL)
-        return JS_FALSE;
+        return false;
 
-    retval = INT_TO_JSVAL(obj->ref_count);
-    argv.rval().set(retval);
-    return JS_TRUE;
+    argv.rval().setInt32(obj->ref_count);
+    return true;
 }
 
 static JSBool
 gjs_breakpoint(JSContext *context,
                unsigned   argc,
-               jsval     *vp)
+               JS::Value *vp)
 {
     JS::CallArgs argv = JS::CallArgsFromVp (argc, vp);
-    if (!gjs_parse_call_args(context, "breakpoint", "", argv))
-        return JS_FALSE;
+    if (!gjs_parse_call_args(context, "breakpoint", argv, ""))
+        return false;
     G_BREAKPOINT();
-    argv.rval().set(JSVAL_VOID);
-    return JS_TRUE;
+    argv.rval().setUndefined();
+    return true;
 }
 
 static JSBool
 gjs_gc(JSContext *context,
        unsigned   argc,
-       jsval     *vp)
+       JS::Value *vp)
 {
     JS::CallArgs argv = JS::CallArgsFromVp (argc, vp);
-    if (!gjs_parse_call_args(context, "gc", "", argv))
-        return JS_FALSE;
+    if (!gjs_parse_call_args(context, "gc", argv, ""))
+        return false;
     JS_GC(JS_GetRuntime(context));
-    argv.rval().set(JSVAL_VOID);
-    return JS_TRUE;
+    argv.rval().setUndefined();
+    return true;
 }
 
 static JSBool
 gjs_exit(JSContext *context,
          unsigned   argc,
-         jsval     *vp)
+         JS::Value *vp)
 {
     JS::CallArgs argv = JS::CallArgsFromVp (argc, vp);
     gint32 ecode;
-    if (!gjs_parse_call_args(context, "exit", "i", argv, "ecode", &ecode))
-        return JS_FALSE;
+    if (!gjs_parse_call_args(context, "exit", argv, "i",
+                             "ecode", &ecode))
+        return false;
     exit(ecode);
-    return JS_TRUE;
+    return true;
 }
 
 static JSBool
 gjs_clear_date_caches(JSContext *context,
-             unsigned   argc,
-             jsval     *vp)
+                      unsigned   argc,
+                      JS::Value *vp)
 {
     JS::CallReceiver rec = JS::CallReceiverFromVp(vp);
     JS_BeginRequest(context);
@@ -137,42 +135,41 @@ gjs_clear_date_caches(JSContext *context,
     JS_ClearDateCaches(context);
     JS_EndRequest(context);
 
-    rec.rval().set(JSVAL_VOID);
-    return JS_TRUE;
+    rec.rval().setUndefined();
+    return true;
 }
 
 static JSFunctionSpec module_funcs[] = {
-    { "addressOf", JSOP_WRAPPER (gjs_address_of), 1, GJS_MODULE_PROP_FLAGS },
-    { "refcount", JSOP_WRAPPER (gjs_refcount), 1, GJS_MODULE_PROP_FLAGS },
-    { "breakpoint", JSOP_WRAPPER (gjs_breakpoint), 0, GJS_MODULE_PROP_FLAGS },
-    { "gc", JSOP_WRAPPER (gjs_gc), 0, GJS_MODULE_PROP_FLAGS },
-    { "exit", JSOP_WRAPPER (gjs_exit), 0, GJS_MODULE_PROP_FLAGS },
-    { "clearDateCaches", JSOP_WRAPPER (gjs_clear_date_caches), 0, GJS_MODULE_PROP_FLAGS },
-    { NULL },
+    JS_FS("addressOf", gjs_address_of, 1, GJS_MODULE_PROP_FLAGS),
+    JS_FS("refcount", gjs_refcount, 1, GJS_MODULE_PROP_FLAGS),
+    JS_FS("breakpoint", gjs_breakpoint, 0, GJS_MODULE_PROP_FLAGS),
+    JS_FS("gc", gjs_gc, 0, GJS_MODULE_PROP_FLAGS),
+    JS_FS("exit", gjs_exit, 0, GJS_MODULE_PROP_FLAGS),
+    JS_FS("clearDateCaches", gjs_clear_date_caches, 0, GJS_MODULE_PROP_FLAGS),
+    JS_FS_END
 };
 
-JSBool
-gjs_js_define_system_stuff(JSContext  *context,
-                           JSObject  **module_out)
+bool
+gjs_js_define_system_stuff(JSContext              *context,
+                           JS::MutableHandleObject module)
 {
     GjsContext *gjs_context;
     char *program_name;
-    jsval value;
-    JSBool retval;
-    JSObject *module;
+    bool retval;
 
-    module = JS_NewObject (context, NULL, NULL, NULL);
+    module.set(JS_NewObject(context, NULL, NULL, NULL));
 
     if (!JS_DefineFunctions(context, module, &module_funcs[0]))
-        return JS_FALSE;
+        return false;
 
-    retval = JS_FALSE;
+    retval = false;
 
     gjs_context = (GjsContext*) JS_GetContextPrivate(context);
     g_object_get(gjs_context,
                  "program-name", &program_name,
                  NULL);
 
+    JS::RootedValue value(context);
     if (!gjs_string_from_utf8(context, program_name,
                               -1, &value))
         goto out;
@@ -189,17 +186,15 @@ gjs_js_define_system_stuff(JSContext  *context,
 
     if (!JS_DefineProperty(context, module,
                            "version",
-                           INT_TO_JSVAL(GJS_VERSION),
+                           JS::Int32Value(GJS_VERSION),
                            JS_PropertyStub,
                            JS_StrictPropertyStub,
                            GJS_MODULE_PROP_FLAGS | JSPROP_READONLY))
         goto out;
 
-    retval = JS_TRUE;
+    retval = true;
 
  out:
     g_free(program_name);
-    *module_out = module;
-
     return retval;
 }
